//=============================================================================
//  TMidiEvent Implementation
//=============================================================================
#include <string.h>
#include <stdio.h>
#include <iostream.h>
#include <mcEvent.h>
#include <mcStatics.h>

//=============================================================================
// constructor TMidiEvent
// 25.02.2000 Adrian Meyer
//=============================================================================
TMidiEvent::TMidiEvent( TMidiTrack* aOwner, long aOffset )
{
  fOwner = aOwner;
  fWorkFlag = cWFOK;
  fData1 = 0;
  fData2 = 0;
  fData3 = 0;
	fNoteOff = false;
	fResult = 0;
  Offset = aOffset;
  Ref = 0;

  GetOwner()->GetOwner()->CallMCMessage( this, cMCEvent, cMMEventCreate, "midi event created");
	GetOwner()->GetOwner()->SetChanged( true );
}

//=============================================================================
// destructor TMidiEvent
// 25.02.2000 Adrian Meyer
//=============================================================================
TMidiEvent::~TMidiEvent( void )
{
  GetOwner()->GetOwner()->CallMCMessage( this, cMCEvent, cMMEventDestroy, "midi event destroyed");
	GetOwner()->GetOwner()->SetChanged( true );
}

//=============================================================================
// read offset
// 25.02.2000 Adrian Meyer
//=============================================================================
long TMidiEvent::GetOffset( void )
{
  return Offset;
}

//=============================================================================
// read offset
// 25.02.2000 Adrian Meyer
//=============================================================================
void TMidiEvent::SetOffset( long aOffset )
{
  fOwner->SetEventOffset( this, aOffset );
  GetOwner()->GetOwner()->CallMCMessage( this, cMCEvent, cMMEventSetOffset, "offset set");
	GetOwner()->GetOwner()->SetChanged( true );
}

//=============================================================================
// writing channel number
// 27.02.2000 Adrian Meyer
//=============================================================================
void TMidiEvent::SetChannel( short aChannel )
{
	if ( aChannel >= 1 && ( aChannel <= 16 ) && fData1 <= 0xEF )
	{
		fData1 &= 0xF0;
		fData1 |= ( aChannel - 1 );		
	}
  GetOwner()->GetOwner()->CallMCMessage( this, cMCEvent, cMMEventSetChannel, "channel set");
	GetOwner()->GetOwner()->SetChanged( true );
}						

//=============================================================================
// reading channel
// 27.02.2000 Adrian Meyer
//=============================================================================
short TMidiEvent::GetChannel( void )
{
	if ( fData1 <= 0xEF )
		{ return ( fData1 & 0x0F ) + 1; }
	else
		{ return -1; }
}

//=============================================================================
// writing note off
// 01.03.2000 Adrian Meyer
//=============================================================================
void TMidiEvent::SetNoteOff( bool aNoteOff )
{
	fNoteOff = aNoteOff;
}
//=============================================================================
// reading note off
// 01.03.2000 Adrian Meyer
//=============================================================================
short TMidiEvent::GetNoteOff( void )
{
	return fNoteOff;
}

//=============================================================================
// read and write data
// 27.02.2000 Adrian Meyer
//=============================================================================
short TMidiEvent::GetData1( void ){ return fData1; }
void TMidiEvent::SetData1( short aData1 )
{
	fData1 = aData1;
	GetOwner()->GetOwner()->CallMCMessage( this, cMCEvent, cMMEventSetData1, "data 1 set");
	GetOwner()->GetOwner()->SetChanged( true );
}

short TMidiEvent::GetData2( void ){ return fData2; }
void TMidiEvent::SetData2( short aData2 )
{
	fData2 = aData2;
	GetOwner()->GetOwner()->CallMCMessage( this, cMCEvent, cMMEventSetData2, "data 2 set");
	GetOwner()->GetOwner()->SetChanged( true );
}

short TMidiEvent::GetData3( void ){ return fData3; }
void TMidiEvent::SetData3( short aData3 )
{
	fData3 = aData3;
	GetOwner()->GetOwner()->CallMCMessage( this, cMCEvent, cMMEventSetData3, "data 3 set");
	GetOwner()->GetOwner()->SetChanged( true );
}


//=============================================================================
// show info
// 29.02.2000 Adrian Meyer
//=============================================================================
void TMidiEvent::ShowInfo( void )
{
	cout.fill('0');
	cout << "TME - Info: offset: ";
	cout.width( 8 );
	cout << Offset;
	cout.width( 2 );
	cout << hex << " data1: " << fData1;
	cout << " data2: " << fData2;
	cout << " data3: " << fData3 << dec;
	if ( Ref != 0 )
	{
		cout.fill('0');
		cout << " ref to: ";
		cout.width( 8 );
		cout << Ref->GetOffset();
	}
 	cout << endl;
}

//=============================================================================
// get note name and number														30.03.2000 Adrian Meyer
//=============================================================================
char* TMidiEvent::GetNoteName()
{
	if ( fResult == 0 ) { fResult = new char[5]; }			// create result

//	if ( fOwner->GetOwner()->
	sprintf( fResult, "%-2s%d", cMajorScale[ fData2%12 ], fData2/12 );
	return fResult;
}

//=============================================================================
// get owner																					05.04.2000 Adrian Meyer
//=============================================================================
TMidiTrack* TMidiEvent::GetOwner( void )
{
	return fOwner;
}

//=============================================================================
// read index of event
// 04.09.2000 Adrian Meyer
//=============================================================================
/*int TMidiEvent::GetIndex( void )
{
	for ( int i=0; i<GetOwner()->GetEventCount(); i++ )
	{
		if ( GetOwner()->GetEvent(i) == this )
		{
			return i;
		}
	}
	return 0;	
}*/