//=============================================================================
// TMidiList implementation
//=============================================================================

#include "mcList.h"
#include <iostream.h>

//=============================================================================
// constructor TMidiItem
// 19.04.2001 Adrian Meyer
//=============================================================================
TMidiItem::TMidiItem( void )
{
  Next = 0;
  Prev = 0;
  Offset = 0;
}

//=============================================================================
// constructor
// 10.04.2001 Adrian Meyer
//=============================================================================
TMidiList::TMidiList( void )
{
	fFirstItem = 0;                                     // init values
	fLastItem = 0;
  fValidIndex = false;
  fIndex = 0;
}

//=============================================================================
// destructor
// 10.04.2001 Adrian Meyer
//=============================================================================
TMidiList::~TMidiList( void )
{
/*  TMidiItem* tmpItem = fLastItem;
  TMidiItem* oldItem = 0;

  while (tmpItem)																			// delete list items
  {
    oldItem = tmpItem;
    tmpItem = tmpItem->Prev;
    delete oldItem;
  }*/

  if ( fIndex ) delete fIndex;												// delete old index
}

//=============================================================================
// add item on top
// 10.04.2001 Adrian Meyer
//=============================================================================
void TMidiList::AddTop( TMidiItem* aMidiItem )
{
  aMidiItem->Prev = 0;                                // fill values
	aMidiItem->Next = fFirstItem;

	if ( fFirstItem ) fFirstItem->Prev = aMidiItem;     // set reverse pointer
  fFirstItem = aMidiItem;                             // set new 1st element
  if (!fLastItem) fLastItem = aMidiItem;							// first item at all
  fValidIndex = false;
}

//=============================================================================
// add item at the end
// 10.04.2001 Adrian Meyer
//=============================================================================
void TMidiList::AddEnd( TMidiItem* aMidiItem )
{
	aMidiItem->Next = 0;                                // fill values
  aMidiItem->Prev = fLastItem;

	if ( fLastItem ) fLastItem->Next = aMidiItem;       // set last item
  fLastItem = aMidiItem;                              // set new last item
	if (!fFirstItem) fFirstItem = aMidiItem;						// first item at all
  fValidIndex = false;
}

//=============================================================================
// add item at the right position
// 10.04.2001 Adrian Meyer
//=============================================================================
void TMidiList::Add( TMidiItem* aMidiItem )
{
  if ( !fFirstItem )																	// first item!
  {
    AddTop( aMidiItem );
    return;
	}

  TMidiItem* tmpItem = fFirstItem;										// position item

  do
  {
    if ( tmpItem->Offset > aMidiItem->Offset ) break;
    tmpItem = tmpItem->Next;
  }
  while ( tmpItem );

  if ( !tmpItem )											            	  // add at bottom
  {
    AddEnd( aMidiItem );
    return;
	}

  if ( tmpItem == fFirstItem )												// add on top
  {
    AddTop( aMidiItem );
    return;
	}
																											// add in front of tmpItem
	aMidiItem->Next = tmpItem;                            // fill values
  aMidiItem->Prev = tmpItem->Prev;

  tmpItem->Prev = aMidiItem;                            // adjust next
  if ( aMidiItem->Prev ) aMidiItem->Prev->Next = aMidiItem; // adjust previous
  fValidIndex = false;
}

//=============================================================================
// build index array
// 16.04.2001 Adrian Meyer
//=============================================================================
void TMidiList::BuildIndex( void )
{
  if (!fFirstItem)
	{
		fCount = 0;
    fValidIndex = true;
		return;                            // list empty
	}

  fCount = 0;
  TMidiItem* tmpItem = fFirstItem;
  while ( tmpItem )																		// count items
  {
    fCount++;
    tmpItem = tmpItem->Next;
  }

  if ( fIndex ) delete fIndex;												// delete old index
	fIndex = new TMidiItem*[ fCount ];                  // build new index

  tmpItem = fFirstItem;
  long tmpIndex = 0;
  while ( tmpItem )    																// fill index array
  {
		fIndex[ tmpIndex ] = tmpItem;
    tmpIndex++;
    tmpItem = tmpItem->Next;
  }

  fValidIndex = true;
}

//=============================================================================
// read item count
// 16.04.2001 Adrian Meyer
//=============================================================================
long TMidiList::GetCount( void )
{
  if (!fValidIndex) BuildIndex();											// build index
  return fCount;
}

//=============================================================================
// read item
// 16.04.2001 Adrian Meyer
//=============================================================================
TMidiItem* TMidiList::GetItem( long aIndex )
{
  if (!fValidIndex) BuildIndex();											// build index
  if ( aIndex > fCount - 1 )
  {
//    cout << "TMidiList index out of bounds " << aIndex << endl;
    return 0;
	}
  else
		return fIndex[ aIndex ];                          // read item
}


//=============================================================================
// set new offset and keep sort
//=============================================================================
// Description:
// - 1. find the item with the pointer
// - 2. find to end if offset grater than original
// - 3. find to top if offset less than original
//=============================================================================
// 16.04.2001 Adrian Meyer
//=============================================================================
void TMidiList::SetOffsetSorted( long aOffset, TMidiItem* aMidiItem )
{
  if ( !aMidiItem )
  {
    cout << "TMidiList item with pointer not found: " << aMidiItem << endl;
    return;
  }

  TMidiItem* tmpPosItem = aMidiItem;                    // current position
  if ( aMidiItem->Next )
  {
    if ( aOffset > aMidiItem->Next->Offset )            // search forward
    {
      tmpPosItem = aMidiItem->Next;
      while ( tmpPosItem )
      {
        if ( aOffset < tmpPosItem->Offset ) break;    // offset found
        tmpPosItem = tmpPosItem->Next;
      }

			// unhook item to change offset everything
      if ( aMidiItem == fFirstItem ) fFirstItem = aMidiItem->Next;  // reset 1st item
      if ( aMidiItem == fLastItem ) fLastItem = aMidiItem->Prev; // reset last item
      if ( aMidiItem->Prev ) aMidiItem->Prev->Next = aMidiItem->Next; // prev to next
      if ( aMidiItem->Next ) aMidiItem->Next->Prev = aMidiItem->Prev; // next to prev

      if ( tmpPosItem )
      {                                               // move back before tmpPosItem
																											// set hooks
        if ( tmpPosItem->Prev ) tmpPosItem->Prev->Next = aMidiItem;
        aMidiItem->Prev = tmpPosItem->Prev;
        tmpPosItem->Prev = aMidiItem;																					
				aMidiItem->Next = tmpPosItem;
      }
      else
      {                                     					// move to last position
        aMidiItem->Prev = fLastItem;										// grab last item
        if ( fLastItem ) fLastItem->Next = aMidiItem;
        fLastItem = aMidiItem;                     			// point to new
        aMidiItem->Next = 0;
      }
    }
  }

  if ( aMidiItem->Prev )
	{
    if ( aOffset < aMidiItem->Prev->Offset )            // search backward
		{
      tmpPosItem = aMidiItem->Prev;
      while ( tmpPosItem )
			{
        if ( aOffset > tmpPosItem->Offset ) break;    // offset found
        tmpPosItem = tmpPosItem->Prev;
			}

			if ( aMidiItem != tmpPosItem )
      {
  			// unhook item to change offset everything
        if ( aMidiItem == fFirstItem ) fFirstItem = aMidiItem->Next;  // reset 1st item
        if ( aMidiItem == fLastItem ) fLastItem = aMidiItem->Prev; // reset last item
        if ( aMidiItem->Prev ) aMidiItem->Prev->Next = aMidiItem->Next; // prev to next
        if ( aMidiItem->Next ) aMidiItem->Next->Prev = aMidiItem->Prev; // next to prev

        if ( tmpPosItem )
    		{																									// move to after tmpPosItem
          if ( fLastItem == tmpPosItem ) fLastItem = aMidiItem;
          if ( tmpPosItem->Next ) tmpPosItem->Next->Prev = aMidiItem;
          aMidiItem->Next = tmpPosItem->Next;
          tmpPosItem->Next = aMidiItem;
          aMidiItem->Prev = tmpPosItem;
    		}
    		else
    		{																									// move to top
    			aMidiItem->Next = fFirstItem;
          fFirstItem->Prev = aMidiItem;
          fFirstItem = aMidiItem;
    			aMidiItem->Prev = 0;
    		}
			}
		}
  }

  aMidiItem->Offset = aOffset;													// set new offset
  fValidIndex = false;
}

//=============================================================================
// delete item
// 17.04.2001 Adrian Meyer
//=============================================================================
void TMidiList::Delete( TMidiItem* aMidiItem )
{
  // unhook item to change offset everything
  if ( aMidiItem == fFirstItem ) fFirstItem = aMidiItem->Next;    // reset 1st item
  if ( aMidiItem == fLastItem ) fLastItem = aMidiItem->Prev;      // reset last item
  if ( aMidiItem->Prev ) aMidiItem->Prev->Next = aMidiItem->Next; // prev to next
  if ( aMidiItem->Next ) aMidiItem->Next->Prev = aMidiItem->Prev; // next to prev

  delete aMidiItem;
  fValidIndex = false;
}