//=============================================================================
// TMidiMapper class implementation
//=============================================================================
#include <string.h>
#include <iostream.h>
#include <mcMapper.h>
#include <stdio.h>

//=============================================================================
// constructor
// 20.07.2001 Adrian Meyer
//=============================================================================
TMidiMapper::TMidiMapper( TMidiIO* aIO, char* aConfFileName )
{
  fInputDeviceList = new TMidiList();									// create lists
  fOutputDeviceList = new TMidiList();
  fConfFile = LoadIni( aConfFileName );
	fIO = aIO;
  fPresetName = new char[1000];
  LoadPreset( 0 );                                    // init to default
}

//=============================================================================
// destructor
// 20.07.2001 Adrian Meyer
//=============================================================================
TMidiMapper::~TMidiMapper()
{
  ClearDevices();

  delete fInputDeviceList;														// delete lists
  delete fOutputDeviceList;

  if ( fPresetName ) delete fPresetName;
}

//=============================================================================
// empty device lists
// 20.07.2001 Adrian Meyer
//=============================================================================
void TMidiMapper::ClearDevices( void )
{
  for ( long i = fInputDeviceList->GetCount() - 1; i>=0 ; i-- )  // empty lists
    fInputDeviceList->Delete(fInputDeviceList->GetItem(i));

  for ( long i = fOutputDeviceList->GetCount() - 1; i>=0 ; i-- )  // empty lists
    fOutputDeviceList->Delete(fOutputDeviceList->GetItem(i));
}


//=============================================================================
// reset to default
// 20.07.2001 Adrian Meyer
//=============================================================================
void TMidiMapper::ResetToDefault( void )
{
  ClearDevices();

	for ( short i = 0; i<fIO->GetInputCount(); i++ )				
    AddInputDevice( fIO->GetInputName(i), fIO->GetInputName(i), i );

	for ( short i = 0; i<fIO->GetOutputCount(); i++ )				
    AddOutputDevice( fIO->GetOutputName(i), fIO->GetOutputName(i), i );
}

//=============================================================================
// set preset name
// 20.07.2001 Adrian Meyer
//=============================================================================
void TMidiMapper::SetPresetName( char* aPresetName )
{
  if ( fPresetName ) delete fPresetName;
  fPresetName = new char[ strlen( aPresetName ) + 1 ];
  strcpy( fPresetName, aPresetName );
}

//=============================================================================
// adding input device
// 20.07.2001 Adrian Meyer
//=============================================================================
void TMidiMapper::AddInputDevice( const char* aName, const char* aDevice, short aPort )
{
  TMidiDevice* tmpMidiDevice = new TMidiDevice( aName, aDevice, cDTInput, aPort );
	fInputDeviceList->Add( tmpMidiDevice );
}

//=============================================================================
// delete a input device
// 20.07.2001 Adrian Meyer
//=============================================================================
void TMidiMapper::DeleteInputDevice( long aID )
{
  fInputDeviceList->Delete( GetInputDevice( aID ));
}

//=============================================================================
// read input device
// 20.07.2001 Adrian Meyer
//=============================================================================
TMidiDevice* TMidiMapper::GetInputDevice( long aID )
{
  return (TMidiDevice*)fInputDeviceList->GetItem( aID );
}

//=============================================================================
// get input device count
// 20.07.2001 Adrian Meyer
//=============================================================================
long TMidiMapper::GetInputDeviceCount( void )
{
  return fInputDeviceList->GetCount();
}

//=============================================================================
// adding Output device
// 20.07.2001 Adrian Meyer
//=============================================================================
void TMidiMapper::AddOutputDevice( const char* aName, const char* aDevice, short aPort )
{
  TMidiDevice* tmpMidiDevice = new TMidiDevice( aName, aDevice, cDTOutput, aPort );
	fOutputDeviceList->Add( tmpMidiDevice );
}

//=============================================================================
// delete a Output device
// 20.07.2001 Adrian Meyer
//=============================================================================
void TMidiMapper::DeleteOutputDevice( long aID )
{
  fOutputDeviceList->Delete( GetOutputDevice( aID ));
}

//=============================================================================
// read Output device
// 20.07.2001 Adrian Meyer
//=============================================================================
TMidiDevice* TMidiMapper::GetOutputDevice( long aID )
{
  return (TMidiDevice*)fOutputDeviceList->GetItem( aID );
}

//=============================================================================
// get Output device count
// 20.07.2001 Adrian Meyer
//=============================================================================
long TMidiMapper::GetOutputDeviceCount( void )
{
  return fOutputDeviceList->GetCount();
}

//=============================================================================
// read preset count
// 23.10.2001 Adrian Meyer
//=============================================================================
long TMidiMapper::GetPresetCount( void )
{
  string tmpString ("0");

  tmpString = GetIniSetting( fConfFile, "Device Mapping", "PresetCount", "0" );

  long tmpCount = atoi( tmpString.c_str());
  tmpCount++;
  return tmpCount;
}

//=============================================================================
// get the preset name
// 23.10.2001 Adrian Meyer
//=============================================================================
char* TMidiMapper::GetPresetName( long aID )
{
  if ( aID==0 )
  {
    strcpy( fPresetName, cDefaultPresetName );
    return cDefaultPresetName;
  }
  else
  {
    char* tmpConfName = new char[500];
    sprintf( tmpConfName, "PresetName%d", (int)aID-1 );
    strcpy( fPresetName, GetIniSetting( fConfFile, "Device Mapping", tmpConfName, "" ).c_str());
    delete tmpConfName;
    return fPresetName;
  }
}

//=============================================================================
// load a preset
// 23.10.2001 Adrian Meyer
//=============================================================================
void TMidiMapper::LoadPreset( long aID )
{
  if ( aID==0 )
  {
    strcpy( fPresetName, cDefaultPresetName );
    ResetToDefault();
    fPresetID = 0;
  }
}

//=============================================================================
// read current preset number
// 23.10.2001 Adrian Meyer
//=============================================================================
long TMidiMapper::GetPresetID( void )
{
  return fPresetID;
}

//=============================================================================
// to be removed after IO for each track
// 23.10.2001 Adrian Meyer
//=============================================================================
TMidiIO* TMidiMapper::GetIO( void )
{
  return fIO;
}

//=============================================================================
// get actual device id for port
// 23.10.2001 Adrian Meyer
//=============================================================================
int TMidiMapper::GetOutputDeviceID( long aPortID )
{
  for ( int i = 0; i<fIO->GetOutputCount(); i++ )
    if ( strcmp( fIO->GetOutputName(i),GetOutputDevice( aPortID )->GetDevice()) == 0 )
    {
      cout << "output device ID mapped to: " << i << " from port: " << aPortID << endl;
      return i;
    }
  return 0;
}