//=============================================================================
//  TMidiMapper class definition
//=============================================================================
// Author  : Adrian Meyer
// Version : 0.3.0
// Log     :
//=============================================================================
// 20.07.2001 building class
// 21.10.2001 adding preset saving to conf file
//=============================================================================
// to do:
//=============================================================================

#ifndef __MCMAPPER__
#define __MCMAPPER__

#include <mcBase.h>
#include <mcList.h>
#include <mcIO.h>
#include <mcDevice.h>
#include <stlini.h>

static char* cDefaultPresetName = "<Default Preset>";

class TMidiMapper: public TMidiBase
{
private:
  TMidiIO* fIO;                                       // io to work of
  char* fPresetName;                                  // name of current preset
  TMidiList* fInputDeviceList;                        // input device list
  TMidiList* fOutputDeviceList;                       // output device list
  void ClearDevices( void );                          // empty device lists
  long fPresetID;                                     // currently used preset ID
  INIFile fConfFile;                                  // conf file for settings

public:
  TMidiMapper( TMidiIO* aIO, char* aConfFileName );   // constructor
  ~TMidiMapper();                                     // destructor

  TMidiIO* GetIO( void ); // to be removed after IO for each track

  long GetPresetCount( void );                        // read preset count
  char* GetPresetName( long aID );                    // get the preset name
  long GetPresetID( void );                           // read current preset ID

  void LoadPreset( long aID );                        // load a preset
  void ResetToDefault( void );                        // reset devices to default
  void SetPresetName( char* aPresetName );            // set preset name

																											// add inpout device
  void AddInputDevice( const char* aName, const char* aDevice, short aPort );
  void DeleteInputDevice( long aID );                 // delete a input device
  TMidiDevice* GetInputDevice( long aID );            // read input device
  long GetInputDeviceCount( void );                   // get input device count

  void AddOutputDevice( const char* aName, const char* aDevice, short aPort );
  void DeleteOutputDevice( long aID );                // delete a Output device
  TMidiDevice* GetOutputDevice( long aID );           // read Output device
  long GetOutputDeviceCount( void );                  // get utput device count
  int GetOutputDeviceID( long aPortID );              // get actual device id for port
};

#endif