//=============================================================================
// implementation class midi IO
//=============================================================================
#include <linux/soundcard.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <string.h>
#include <iostream.h>
#include <mcOSSIO.h>


// define static variables:
const char* TMidiOSSIO::sequencer       = "/dev/music";
int    TMidiOSSIO::class_count          =  0;
uchar  TMidiOSSIO::midi_write_packet[4] = {SEQ_MIDIPUTC, 0, 0, 0};
uchar  TMidiOSSIO::midi_read_packet[4];


// static variables for MIDI I/O information database
int    TMidiOSSIO::indevcount      = 0;
int    TMidiOSSIO::outdevcount     = 0;

int*   TMidiOSSIO::indevnum        = NULL;
int*   TMidiOSSIO::outdevnum       = NULL;

int*   TMidiOSSIO::indevtype       = NULL;
int*   TMidiOSSIO::outdevtype      = NULL;

char** TMidiOSSIO::indevnames      = NULL;
char** TMidiOSSIO::outdevnames     = NULL;

#define sequencer_buffer     1024
SEQ_DEFINEBUF (sequencer_buffer);

//int seqfd   = -1;
//int mididev = -1;

void TMidiOSSIO::seqbuf_dump(void)
{
	if (_seqbufptr)
	{
		if (write(getFd(), _seqbuf, _seqbufptr) == -1)
		{
			cerr << "Error writing /dev/music" << endl;
			return;
		}
		_seqbufptr = 0;
	}
}


//=============================================================================
// constructor
// 04.03.2001 Adrian Meyer
//=============================================================================
TMidiOSSIO::TMidiOSSIO( int autoOpen )
{

	_seqbuflen = SEQ_BUFSIZE;
	_seqbufptr = 0;
	seq_fd = -1;
	fCurrentTime = 0;
	fStartTime = 0;
	fEchoTime = 0;

	if ( autoOpen )
		open(); 

	if ( GetInputCount() > 0 )
		fInputDevice = 0;
	else
		fInputDevice = -1;

	if ( GetOutputCount() > 0 )
		fOutputDevice = 0;
	else
		fOutputDevice = -1;

	if (class_count < 0) {
		cerr << "Unusual class instatiation count: " << class_count << endl;
		exit(1);
	} else if (class_count == 0) {
		buildInfoDatabase();
	}

	class_count++;
}

//=============================================================================
// destructor
// 04.03.2001 Adrian Meyer
//=============================================================================
TMidiOSSIO::~TMidiOSSIO( void )
{
   class_count--;

   if (class_count == 0) {
      close();
      removeInfoDatabase();
   } else if (class_count < 0) {
      cerr << "Unusual class instatiation count: " << class_count << endl;
      exit(1);
   }
}

//=============================================================================
// read input device
// 05.03.2001 Adrian Meyer
//=============================================================================
int TMidiOSSIO::GetInputDevice( void )
{
  return fInputDevice;
}

//=============================================================================
// set input device
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiOSSIO::SetInputDevice( int aInputDevice )
{
  fInputDevice = aInputDevice;
}

//=============================================================================
// read output device
// 05.03.2001 Adrian Meyer
//=============================================================================
int TMidiOSSIO::GetOutputDevice( void )
{
  return fOutputDevice;
}

//=============================================================================
// set output device
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiOSSIO::SetOutputDevice( int aOutputDevice )
{
  fOutputDevice = aOutputDevice;
}

//=============================================================================
// close the sequencer device
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiOSSIO::close( void )
{
  ::close( getFd() );
}

//
//
void
TMidiOSSIO::Start(void)
{
	SEQ_START_TIMER();
	sendTimingEcho(100);
	SEQ_DUMPBUF();
}

//
// Nothing to do for OSS
//
void
TMidiOSSIO::Stop(void)
{
	SEQ_STOP_TIMER();
	seqbuf_dump();
}

//=============================================================================
// read input name
// 05.03.2001 Adrian Meyer
//=============================================================================
const char* TMidiOSSIO::GetInputName(int aDevice) const {
   if (aDevice >= GetInputCount()) {
      cerr << "Error: " << aDevice << " is greater than max in ("
           << GetInputCount() << ")" << endl;
      exit(1);
   }

   return indevnames[aDevice];
}

//=============================================================================
// read number of inputs
// 05.03.2001 Adrian Meyer
//=============================================================================
int TMidiOSSIO::GetInputCount(void) const {
   return indevcount;
}

//=============================================================================
// read amount of outputs
// 05.03.2001 Adrian Meyer
//=============================================================================
int TMidiOSSIO::GetOutputCount(void) const {
   return outdevcount;
}

//=============================================================================
// return midi output name
// 05.03.2001 Adrian Meyer
//=============================================================================
const char* TMidiOSSIO::GetOutputName(int aDevice) const {
   if (aDevice >= GetOutputCount()) {
      cerr << "Error: " << aDevice << " is greater than max out ("
           << GetOutputCount() << ")" << endl;
      exit(1);
   }

   return outdevnames[aDevice];
}

//=============================================================================
// is the device open?
// 05.03.2001 Adrian Meyer
//=============================================================================
int TMidiOSSIO::is_open(void) const {
   if (getFd() > 0) {
      return 1;
   } else {
      return 0;
   }
}

//=============================================================================
// open the device
// 05.03.2001 Adrian Meyer
//=============================================================================
int TMidiOSSIO::open(void) {
	int fd;

	if (getFd() <= 0) {
		fd = ::open(sequencer, O_RDWR | O_NDELAY);
		setFd(fd);

		SetTimebase(96);
		SetTempo(120, 0);
	}

	return is_open();
}

//=============================================================================
// reads MIDI bytes and also stores the
// device from which the byte was read from.  Timing is not
// saved from the device.  If needed, then it would have to
// be saved in this function, or just return the raw packet
// data (use rawread function).
// 05.03.2001 Adrian Meyer
//=============================================================================
TMidiEvent *TMidiOSSIO::read()
{
#ifdef notyet
   int i = 0;
   while (i < count) {
      ::read(getFd(), midi_read_packet, sizeof(midi_read_packet));
      if (midi_read_packet[1] == SEQ_MIDIPUTC) {
         buf[i] = midi_read_packet[1];
         dev[i] = midi_read_packet[2];
         i++;
      }
   }
#endif
   return NULL;
}


//=============================================================================
// rebuild internal database
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiOSSIO::rebuildInfoDatabase(void) {
   removeInfoDatabase();
   buildInfoDatabase();
}

//=============================================================================
// write on ebyte
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiOSSIO::Write(int aDevice, TMidiEvent *ev)
{
	int chan = ev->GetChannel() - 1;

	switch (ev->GetStatus()) {
	case cESNoteOff:
		SEQ_STOP_NOTE(aDevice, chan, ev->GetData2(), ev->GetData3());
	    break;
	case cESNoteOn:
		SEQ_START_NOTE(aDevice, chan, ev->GetData2(), ev->GetData3());
		break;
	case cESPolyphonicAfterTouch:
		SEQ_KEY_PRESSURE(aDevice, chan, ev->GetData2(), ev->GetData3());
		break;
	case cESControlChange:
		SEQ_CONTROL(aDevice, chan, ev->GetData2(), ev->GetData3());
		break;
	case cESProgramChange:
		SEQ_SET_PATCH(aDevice, chan, ev->GetData2());
		break;
	case cESChannelAfterTouch:
		SEQ_CHN_PRESSURE(aDevice, chan, ev->GetData2());
		break;
	case cESPitchBend:
		SEQ_BENDER(aDevice, chan, (ev->GetData2()<<7) | (ev->GetData3()&0x07));
		break;
	default:
		// Its a system exclusive or other system command.
		cout<< "sysex and 0xfx commands not yet implemented" << endl;
		break;
	}
}

//=============================================================================
// build information database of in- and outputs
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiOSSIO::buildInfoDatabase(void) {
   int status;

   // read number of inputs available (external MIDI devices only)
   status = ioctl(getFd(), SNDCTL_SEQ_NRMIDIS, &indevcount);
   if (status!= 0) {
      cerr << "Error determining the number of MIDI inputs" << endl;
      exit(1);
   }

   // read number of output available
   int extmidi = indevcount;
   int intmidi;
   status = ioctl(getFd(), SNDCTL_SEQ_NRSYNTHS, &intmidi);
   if (status!= 0) {
      cerr << "Error determining the number of MIDI inputs" << endl;
      exit(1);
   }
   outdevcount = extmidi + intmidi;

   // allocate space for names and device number arrays
   if (indevnum != NULL || outdevnum != NULL || indevnames != NULL ||
         outdevnames != NULL || indevtype != NULL || outdevtype != NULL) {
      cerr << "Error: buildInfoDatabase called twice." << endl;
      exit(1);
   }

   indevnum = new int[indevcount];
   outdevnum = new int[outdevcount];

   indevtype = new int[indevcount];
   outdevtype = new int[outdevcount];

   indevnames = new char*[indevcount];
   outdevnames = new char*[outdevcount];


   // fill in the device translation table and fill in the device names
   int i;
   struct midi_info midiinfo;
   for (i=0; i<indevcount; i++) {
      midiinfo.device = i;
      status = ioctl(getFd(), SNDCTL_MIDI_INFO, &midiinfo);
      if (status != 0) {
         cerr << "Error while reading MIDI device " << i << endl;
         exit(1);
      }

      indevnum[i]    = midiinfo.device;
      outdevnum[i]   = midiinfo.device;
      indevtype[i]   = MIDI_EXTERNAL;
      outdevtype[i]  = MIDI_EXTERNAL;
      indevnames[i]  = new char[strlen(midiinfo.name) + 1 + 10];
      outdevnames[i] = new char[strlen(midiinfo.name) + 1 + 11];
      strcpy(indevnames[i], midiinfo.name);
      strcpy(outdevnames[i], midiinfo.name);
      strcat(indevnames[i], " (MIDI In)");
      strcat(outdevnames[i], " (MIDI Out)");
   }

   char tempstring[1024] = {0};
   struct synth_info synthinfo;
   for (i=0; i<intmidi; i++) {
      synthinfo.device = i;
      status = ioctl(getFd(), SNDCTL_SYNTH_INFO, &synthinfo);
      if (status != 0) {
         cerr << "Error while reading MIDI device " << i << endl;
         exit(1);
      }
      outdevnum[extmidi+i] = i;
      outdevtype[extmidi + i] = MIDI_INTERNAL;

      strcpy(tempstring, synthinfo.name);
      switch (synthinfo.synth_type) {
         case SYNTH_TYPE_FM:           // 0
            strcat(tempstring, " (FM");
            switch (synthinfo.synth_subtype) {
               case FM_TYPE_ADLIB:     // 0
                  strcat(tempstring, " Adlib");
                  break;
               case FM_TYPE_OPL3:      // 1
                  strcat(tempstring, " OPL3");
                  break;
            }
            strcat(tempstring, ")");
            break;
         case SYNTH_TYPE_SAMPLE:       // 1
            strcat(tempstring, " (Wavetable)");
            break;
         case SYNTH_TYPE_MIDI:         // 2
            strcat(tempstring, " (MIDI Interface");
            switch (synthinfo.synth_subtype) {
               case SYNTH_TYPE_MIDI:   // 0x401
                  strcat(tempstring, " MPU401");
                  break;
            }
            strcat(tempstring, ")");
            break;
      }
      outdevnames[i+extmidi] = new char[strlen(tempstring) + 1];
      strcpy(outdevnames[i+extmidi], tempstring);
   }
}

//
// Get the current time.
//
long TMidiOSSIO::GetTime()
{
	getEvents();
	sendTimingEcho(fCurrentTime + 100);
	return fCurrentTime;
}

void TMidiOSSIO::SetTimebase(int base)
{
cout<<"TMidiOSSIO: Set timebase " << base << endl;
	ioctl(getFd(), SNDCTL_TMR_TIMEBASE, &base);
}

void TMidiOSSIO::SetTempo(int tempo, long tick)
{
cout<<"TMidiOSSIO: Set tempo " << tempo << endl;
	ioctl(getFd(), SNDCTL_TMR_TEMPO, &tempo);
}

//
// This code derived from that in Jazz++ by Andreas Voss and Per Sigmond.
//
void
TMidiOSSIO::getEvents()
{
	unsigned char buf[256];
	int  size;
	while ((size = ::read(getFd(), buf, sizeof(buf))) > 0)
	{
		for (int i = 0; i < size; )
		{
			switch (buf[i]) {
			case EV_TIMING:
				switch (buf[i+1]) {
				case TMR_ECHO:
					fCurrentTime = *(unsigned long *)&buf[i+4] + fStartTime;
					break;

				default:
					cout << "unknown EV_TIMING" << hex << buf[i+1];
					break;
				}
				i += 8;
				break;
			}
		}
	}
}


//=============================================================================
// read input device number
// 05.03.2001 Adrian Meyer
//=============================================================================
int TMidiOSSIO::getInDeviceValue(int aDevice) const {
   if (aDevice >= GetInputCount()) {
      cerr << "Error: " << aDevice << " is greater than max in ("
           << GetInputCount() << ")" << endl;
      exit(1);
   }

   return indevnum[aDevice];
}

//=============================================================================
// returns 1 = external MIDI, 2 = internal MIDI
// 05.03.2001 Adrian Meyer
//=============================================================================
int TMidiOSSIO::getInputType(int aDevice) const {
   if (aDevice >= GetInputCount()) {
      cerr << "Error: " << aDevice << " is greater than max in ("
           << GetInputCount() << ")" << endl;
      exit(1);
   }

   return indevtype[aDevice];
}

//=============================================================================
// read output device number
// 05.03.2001 Adrian Meyer
//=============================================================================
int TMidiOSSIO::getOutDeviceValue(int aDevice) const {
   if (aDevice >= GetOutputCount()) {
      cerr << "Error: " << aDevice << " is greater than max out ("
           << GetOutputCount() << ")" << endl;
      exit(1);
   }

   return outdevnum[aDevice];
}

//=============================================================================
// returns 1 = external MIDI, 2 = internal MIDI
// 05.03.2001 Adrian Meyer
//=============================================================================
int TMidiOSSIO::getOutputType(int aDevice) const {
   if (aDevice >= GetOutputCount()) {
      cerr << "Error: " << aDevice << " is greater than max out ("
           << GetOutputCount() << ")" << endl;
      exit(1);
   }

   return outdevtype[aDevice];
}

//=============================================================================
// kill info database
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiOSSIO::removeInfoDatabase(void) {
   if (indevnum   != NULL)   delete [] indevnum;
   if (outdevnum  != NULL)   delete [] outdevnum;
   if (indevtype  != NULL)   delete [] indevtype;
   if (outdevtype != NULL)   delete [] outdevtype;

   int i;
   if (indevnames != NULL) {
      for (i=0; i<indevcount; i++) {
         if (indevnames[i] != NULL)    delete [] indevnames[i];
      }
      delete [] indevnames;
   }

   if (outdevnames != NULL) {
      for (i=0; i<outdevcount; i++) {
         if (outdevnames[i] != NULL)   delete [] outdevnames[i];
      }
      delete [] outdevnames;
   }

   indevnum    = NULL;
   outdevnum   = NULL;
   indevtype   = NULL;
   outdevtype  = NULL;
   indevnames  = NULL;
   outdevnames = NULL;

   indevcount = 0;
   outdevcount = 0;
}

/**
 * Send echo commands that will be used for timing.
 * @param util send out a series of echo commands up until this time
 */
void TMidiOSSIO::sendTimingEcho(long until)
{
	const int ADVANCE = 20;

	for (; fEchoTime < until; fEchoTime += ADVANCE)
	{
		SEQ_WAIT_TIME(fEchoTime - fStartTime);		
		SEQ_ECHO_BACK(fEchoTime - fStartTime);		
	}
	SEQ_DUMPBUF();
}
