//=============================================================================
//  TMidiSequence Class Implementation
//=============================================================================
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <mcSequence.h>
#include <stdio.h>
#include <iomanip.h>

//=============================================================================
// constructor TMidiSequence with message hook
// 28.08.2000 Adrian Meyer
//=============================================================================
TMidiSequence::TMidiSequence( tMCMessage * aMCMessage, void* aUserData )
{
	InitAll();

  AddMCMessage( aMCMessage, aUserData );
  CallMCMessage( this, cMCSequence, cMMSequenceCreate, "midi sequence created");

	fChanged = false;	
}

//=============================================================================
// destructor TMidiSequence
// 18.02.2000 Adrian Meyer
//=============================================================================
TMidiSequence::~TMidiSequence( void )
{                                                      // delete lists
	RemoveAll();

  CallMCMessage( this, cMCSequence, cMMSequenceDestroy, "midi sequence destroyed");
	SetChanged( true );
}

//=============================================================================
// write ficed with number to file
// 24.11.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::WriteFixNumber( long aNumber, long aWidth )
{
  for ( int i = aWidth - 1; i>=0; i-- )
  {
    fSaveFile << char( ( aNumber >> (i*8)) & 0xFF );
  }
}

//=============================================================================
// save objects to file
// 24.11.2000 Adrian Meyer
//=============================================================================
short TMidiSequence::SaveToFile( char* aFileName )
{
	char* tmpFileName = new char[ strlen( aFileName ) + 1 ];
	strcpy( tmpFileName, aFileName );
	fSaveFile.open( tmpFileName );											// open write file stream
	cout << aFileName << endl;

  fSaveFile << "MThd";																// file header
	WriteFixNumber( 6, 4 );															// header chunk length
	WriteFixNumber( GetFileType(), 2 );								  // write track count
	WriteFixNumber( GetTrackCount(), 2 );								// write track count
	WriteFixNumber( GetDeltaTimesType(), 2 );						// write track count

	TMidiTrack* tmpTrack = 0;														// working track
	TMidiEvent* tmpEvent = 0;														// working event

  for ( int i = 0; i<GetTrackCount(); i++ )
	{
    fSaveFile << "MTrk";														  // track header
    tmpTrack = GetTrack(i);
		long tmpTrackSize = 0;
		long tmpSize = tmpTrack->GetEventCount() +				// "normal" track size
			tmpTrack->GetLyricCount() +
			tmpTrack->GetChannelPrefixCount() +
			( strlen( tmpTrack->GetTrackName() ) > 0 ) +
			( strlen( tmpTrack->GetInstrumentName() ) > 0 ) +
			( tmpTrack->GetInstrumentPatch() > -1 ) +
			tmpTrack->GetTextCount();
                 	
		if ( i == 0 )	tmpSize += GetMarkerCount() +       // add master track items to size																								
			GetCuePointCount() +
			GetTempoCount() +
			GetTimeSignatureCount() +
			GetKeySignatureCount() +
			GetMetaDataCount() +
			( strlen( GetSequenceName() ) > 0 ) +
			( strlen( GetCopyrightNotice() ) > 0 );
																											// build data tocken array
	  sDataTocken* tmpDataChain = new sDataTocken[ tmpSize + 1 ];
		long tmpChainPos = 0;

 		if ( strlen( tmpTrack->GetTrackName() ) > 0 )			// track name
 		{
 			tmpDataChain[ tmpChainPos ].Offset = 0;
 			tmpDataChain[ tmpChainPos ].DataSize = strlen( tmpTrack->GetTrackName() ) + 3;
 			tmpDataChain[ tmpChainPos ].Data = new char[ tmpDataChain[ tmpChainPos ].DataSize ];
 			tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
 			tmpDataChain[ tmpChainPos ].Data[1] = char( 0x03 );
 			tmpDataChain[ tmpChainPos ].Data[2] = char( strlen( tmpTrack->GetTrackName() ) );

 			for ( long k=0; k<(long)strlen( tmpTrack->GetTrackName() ); k++ )
 				tmpDataChain[ tmpChainPos ].Data[3+k] = tmpTrack->GetTrackName()[k];

 			tmpChainPos++;
 		}

 		if ( strlen( tmpTrack->GetInstrumentName() ) > 0 )			// instrument name
 		{
 			tmpDataChain[ tmpChainPos ].Offset = 0;
 			tmpDataChain[ tmpChainPos ].DataSize = strlen( tmpTrack->GetInstrumentName() ) + 3;
 			tmpDataChain[ tmpChainPos ].Data = new char[ tmpDataChain[ tmpChainPos ].DataSize ];
 			tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
 			tmpDataChain[ tmpChainPos ].Data[1] = char( 0x04 );
 			tmpDataChain[ tmpChainPos ].Data[2] = char( strlen( tmpTrack->GetInstrumentName() ) );

 			for ( long k=0; k<(long)strlen( tmpTrack->GetInstrumentName() ); k++ )
 				tmpDataChain[ tmpChainPos ].Data[3+k] = tmpTrack->GetInstrumentName()[k];

 			tmpChainPos++;
 		}

		if ( i == 0 )																			// add master track items
		{
			if ( strlen( GetSequenceName() ) )							// sequence name
			{
				tmpDataChain[ tmpChainPos ].Offset = 0;
				tmpDataChain[ tmpChainPos ].DataSize = strlen( GetSequenceName() ) + 3;
				tmpDataChain[ tmpChainPos ].Data = new char[ tmpDataChain[ tmpChainPos ].DataSize ];
				tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
				tmpDataChain[ tmpChainPos ].Data[1] = char( 0x03 );
	 			tmpDataChain[ tmpChainPos ].Data[2] = char( strlen( GetSequenceName() ) );

	 			for ( long k=0; k<(long)strlen( GetSequenceName() ); k++ )
 					tmpDataChain[ tmpChainPos ].Data[3+k] = GetSequenceName()[k];

	 			tmpChainPos++;
			}

			if ( strlen( GetCopyrightNotice() ) > 0 )				// copyright
			{
				tmpDataChain[ tmpChainPos ].Offset = 0;
				tmpDataChain[ tmpChainPos ].DataSize = strlen( GetCopyrightNotice() ) + 3;
				tmpDataChain[ tmpChainPos ].Data = new char[ tmpDataChain[ tmpChainPos ].DataSize ];
				tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
				tmpDataChain[ tmpChainPos ].Data[1] = char( 0x02 );
	 			tmpDataChain[ tmpChainPos ].Data[2] = char( strlen( GetCopyrightNotice() ) );

	 			for ( long k=0; k<(long)strlen( GetCopyrightNotice() ); k++ )
 					tmpDataChain[ tmpChainPos ].Data[3+k] = GetCopyrightNotice()[k];

	 			tmpChainPos++;
			}

			for ( long j = 0; j<GetMarkerCount(); j++ )			// marker
			{
				tmpDataChain[ tmpChainPos ].Offset = GetMarker(j)->Offset;
				tmpDataChain[ tmpChainPos ].DataSize = strlen( GetMarker(j)->GetName() ) + 3;
				tmpDataChain[ tmpChainPos ].Data = new char[ tmpDataChain[ tmpChainPos ].DataSize ];
				tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
				tmpDataChain[ tmpChainPos ].Data[1] = char( 0x06 );
				tmpDataChain[ tmpChainPos ].Data[2] = char( strlen( GetMarker(j)->GetName() ) );
				for ( long k=0; k<(long)strlen( GetMarker(j)->GetName() ); k++ )
					tmpDataChain[ tmpChainPos ].Data[3+k] = GetMarker(j)->GetName()[k];
				tmpChainPos++;
			}				

			for ( long j = 0; j<GetCuePointCount(); j++ )		// cue point
			{
				tmpDataChain[ tmpChainPos ].Offset = GetCuePoint(j)->Offset;
				tmpDataChain[ tmpChainPos ].DataSize = strlen( GetCuePoint(j)->CuePointName ) + 3;
				tmpDataChain[ tmpChainPos ].Data = new char[ tmpDataChain[ tmpChainPos ].DataSize ];
				tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
				tmpDataChain[ tmpChainPos ].Data[1] = char( 0x07 );
				tmpDataChain[ tmpChainPos ].Data[2] = char( strlen( GetCuePoint(j)->CuePointName ) );
				for ( long k=0; k < (long)strlen( GetCuePoint(j)->CuePointName ); k++ )
					tmpDataChain[ tmpChainPos ].Data[3+k] = GetCuePoint(j)->CuePointName[k];
				tmpChainPos++;
			}				

			for ( long j = 0; j<GetTempoCount(); j++ )			// tempo
			{
				tmpDataChain[ tmpChainPos ].Offset = GetTempo(j)->Offset;
				tmpDataChain[ tmpChainPos ].DataSize = 6;
				tmpDataChain[ tmpChainPos ].Data = new char[ 6 ];
				tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
				tmpDataChain[ tmpChainPos ].Data[1] = char( 0x51 );
				tmpDataChain[ tmpChainPos ].Data[2] = char( 0x03 );
				tmpDataChain[ tmpChainPos ].Data[3] = char( ((GetTempo(j)->Tempo) >> 16 ) & 0xFF );
				tmpDataChain[ tmpChainPos ].Data[4] = char( ((GetTempo(j)->Tempo) >> 8 ) & 0xFF );
				tmpDataChain[ tmpChainPos ].Data[5] = char( (GetTempo(j)->Tempo) & 0xFF );
				tmpChainPos++;
			}				

			for ( long j = 0; j<GetTimeSignatureCount(); j++ )// key signatures
			{
				tmpDataChain[ tmpChainPos ].Offset = GetTimeSignature(j)->Offset;
				tmpDataChain[ tmpChainPos ].DataSize = 7;
				tmpDataChain[ tmpChainPos ].Data = new char[ 7 ];
				tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
				tmpDataChain[ tmpChainPos ].Data[1] = char( 0x58 );
				tmpDataChain[ tmpChainPos ].Data[2] = char( 0x04 );
				tmpDataChain[ tmpChainPos ].Data[3] = char( (GetTimeSignature(j)->Numerator) & 0xFF );
				tmpDataChain[ tmpChainPos ].Data[4] = char( (GetTimeSignature(j)->Denominator) & 0xFF );
				tmpDataChain[ tmpChainPos ].Data[5] = char( (GetTimeSignature(j)->MidiClocksPerQuarter) & 0xFF );
				tmpDataChain[ tmpChainPos ].Data[6] = char( (GetTimeSignature(j)->NotesPerQuarter) & 0xFF );
				tmpChainPos++;
			}				

			for ( long j = 0; j<GetKeySignatureCount(); j++ )// key signatures
			{
				tmpDataChain[ tmpChainPos ].Offset = GetKeySignature(j)->Offset;
				tmpDataChain[ tmpChainPos ].DataSize = 5;
				tmpDataChain[ tmpChainPos ].Data = new char[ 5 ];
				tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
				tmpDataChain[ tmpChainPos ].Data[1] = char( 0x59 );
				tmpDataChain[ tmpChainPos ].Data[2] = char( 0x02 );
				tmpDataChain[ tmpChainPos ].Data[3] = char( (GetKeySignature(j)->SharpFlatCount) & 0xFF );
				tmpDataChain[ tmpChainPos ].Data[4] = char( (GetKeySignature(j)->IsMinor) & 0xFF );
				tmpChainPos++;
			}				

			for ( long j = 0; j<GetMetaDataCount(); j++ )		// meta data
			{
				tmpDataChain[ tmpChainPos ].Offset = GetMetaData(j)->Offset;
				tmpDataChain[ tmpChainPos ].DataSize = GetMetaData(j)->Size + 3;
				tmpDataChain[ tmpChainPos ].Data = new char[ tmpDataChain[ tmpChainPos ].DataSize ];
				tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
				tmpDataChain[ tmpChainPos ].Data[1] = char( 0x7F );
				tmpDataChain[ tmpChainPos ].Data[2] = char(  );
				for ( long k=0; k<GetMetaData(j)->Size; k++ )
					tmpDataChain[ tmpChainPos ].Data[3+k] = GetMetaData(j)->MetaData[k];
				tmpChainPos++;
			}				
		}

 		for ( long j = 0; j<tmpTrack->GetLyricCount(); j++ )// lyrics
 		{
			sLyricList* tmpLyric = tmpTrack->GetLyric(j);
 			tmpDataChain[ tmpChainPos ].Offset = tmpLyric->Offset;
 			tmpDataChain[ tmpChainPos ].DataSize = strlen( tmpLyric->Lyric ) + 3;
 			tmpDataChain[ tmpChainPos ].Data = new char[ tmpDataChain[ tmpChainPos ].DataSize ];
 			tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
 			tmpDataChain[ tmpChainPos ].Data[1] = char( 0x05 );
 			tmpDataChain[ tmpChainPos ].Data[2] = char( strlen( tmpLyric->Lyric ) );

 			for ( long k=0; k<(long)strlen( tmpLyric->Lyric ); k++ )
 				tmpDataChain[ tmpChainPos ].Data[3+k] = tmpLyric->Lyric[k];

 			tmpChainPos++;
 		}

 		for ( long j = 0; j<tmpTrack->GetTextCount(); j++ )// text
 		{
			sTextList* tmpText = tmpTrack->GetText(j);
 			tmpDataChain[ tmpChainPos ].Offset = tmpText->Offset;
 			tmpDataChain[ tmpChainPos ].DataSize = strlen( tmpText->Text ) + 3;
 			tmpDataChain[ tmpChainPos ].Data = new char[ tmpDataChain[ tmpChainPos ].DataSize ];
 			tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
 			tmpDataChain[ tmpChainPos ].Data[1] = char( 0x01 );
 			tmpDataChain[ tmpChainPos ].Data[2] = char( strlen( tmpText->Text ) );

 			for ( long k=0; k<(long)strlen( tmpText->Text ); k++ )
 				tmpDataChain[ tmpChainPos ].Data[3+k] = tmpText->Text[k];

 			tmpChainPos++;
 		}

 		for ( long j = 0; j<tmpTrack->GetChannelPrefixCount(); j++ )// key signatures
 		{
 			tmpDataChain[ tmpChainPos ].Offset = tmpTrack->GetChannelPrefix(j)->Offset;
 			tmpDataChain[ tmpChainPos ].DataSize = 4;
 			tmpDataChain[ tmpChainPos ].Data = new char[ 4 ];
 			tmpDataChain[ tmpChainPos ].Data[0] = char( 0xFF );
 			tmpDataChain[ tmpChainPos ].Data[1] = char( 0x20 );
 			tmpDataChain[ tmpChainPos ].Data[2] = char( 0x01 );
 			tmpDataChain[ tmpChainPos ].Data[3] = char( (tmpTrack->GetChannelPrefix(j)->ChannelPrefix) & 0xFF );
 			tmpChainPos++;
 		}	

 		if ( tmpTrack->GetInstrumentPatch() > -1 )			// track's instrument patch
 		{
 			tmpDataChain[ tmpChainPos ].Offset = 0;
 			tmpDataChain[ tmpChainPos ].DataSize = 2;
 			tmpDataChain[ tmpChainPos ].Data = new char[ 2 ];
 			tmpDataChain[ tmpChainPos ].Data[0] = char( 0xC0 | (tmpTrack->GetChannel()-1) );
 			tmpDataChain[ tmpChainPos ].Data[1] = char( tmpTrack->GetInstrumentPatch() );

 			tmpChainPos++;
 		}

		for ( long j=0; j<tmpTrack->GetEventCount(); j++ )// add events
		{
      tmpEvent = tmpTrack->GetEvent(j);

  		tmpDataChain[ tmpChainPos ].Offset = tmpEvent->GetOffset();

      if ( (tmpEvent->GetData1())&0xF0 == 0xC0 || (tmpEvent->GetData1())&0xF0 == 0xD0 )
	      {	tmpDataChain[ tmpChainPos ].DataSize = 2; }
			else
				{	tmpDataChain[ tmpChainPos ].DataSize = 3; }

  		tmpDataChain[ tmpChainPos ].Data = new char[	tmpDataChain[ tmpChainPos ].DataSize ];
			tmpDataChain[ tmpChainPos ].Data[0] = (char)tmpEvent->GetData1();
			tmpDataChain[ tmpChainPos ].Data[1] = (char)tmpEvent->GetData2();

			if ( tmpDataChain[ tmpChainPos ].DataSize == 3 )
			{
				tmpDataChain[ tmpChainPos ].Data[2] = (char)tmpEvent->GetData3();
			}
			tmpChainPos++;
		}

//		cout << "Chain Size: " << tmpSize << endl;
//		cout << "Last chain pos: " << tmpChainPos << endl;

		long tmpOldOffset = 0;
		for ( long j = 0; j<tmpSize; j++ ){								// sort by offset
			for ( long k = j+1; k<tmpSize; k++ ){

				if ( tmpDataChain[j].Offset > tmpDataChain[k].Offset )
				{
					sDataTocken tmpDataTocken = tmpDataChain[j];
					tmpDataChain[j] = tmpDataChain[k];
					tmpDataChain[k] = tmpDataTocken;
				}  	
      }
                                                      // build dynamic offset
			long tmpOffset = tmpDataChain[j].Offset - tmpOldOffset;
			register long tmpBuffer = tmpOffset & 0x0000007F;
			long tmpCount = 1;

			while (( tmpOffset >>= 7 )> 0 )
			{
				tmpBuffer <<= 8;
				tmpBuffer |= 0x80;
				tmpBuffer += (tmpOffset & 0x7F);
				tmpCount ++;
			}

			tmpDataChain[j].DynOffset = new char[ tmpCount ];
			tmpDataChain[j].DynOffsetSize = tmpCount;
			tmpTrackSize += tmpCount + tmpDataChain[j].DataSize;
			tmpCount = 0;

			while ( true )
			{
				tmpDataChain[j].DynOffset[ tmpCount ] = (char)(tmpBuffer & 0xFF);
				tmpCount++;
				if ( tmpBuffer & 0x80 )
					tmpBuffer >>=8;
				else
					break;
			}

			tmpOldOffset = tmpDataChain[j].Offset;
		}

		tmpDataChain[ tmpSize ].Offset = 0;								// end of track
		tmpDataChain[ tmpSize ].DataSize = 3;
		tmpDataChain[ tmpSize ].DynOffsetSize = 1;
		tmpDataChain[ tmpSize ].Data = new char[3];
		tmpDataChain[ tmpSize ].DynOffset = new char[1];
		tmpDataChain[ tmpSize ].DynOffset[0] = 0;
		tmpDataChain[ tmpSize ].Data[0] = char( 0xFF );
		tmpDataChain[ tmpSize ].Data[1] = char( 0x2F );
		tmpDataChain[ tmpSize ].Data[2] = char( 0x00 );
		tmpSize++;

    char* tmpBuffer = new char[ tmpTrackSize ];
		tmpTrackSize = 0;
		char tmpLastStatus = 0;

		for ( long j = 0; j<tmpSize; j++ )								// work through chain
		{
			// O F F S E T
			// print offset byte only on first event or event data only 1 byte or not 0 or meta data
//			if ( j==0 || tmpDataChain[j].DataSize<=1 || ( tmpDataChain[j].DynOffset[0]&0xFF ) > 0 || ( tmpDataChain[j].Data[0] == tmpLastStatus ) )
			{
				cout << "offset: " ;

				for ( long k=0; k<tmpDataChain[j].DynOffsetSize; k++ )
				{
					tmpBuffer[ tmpTrackSize ] = tmpDataChain[j].DynOffset[k];

					cout << hex << ( tmpBuffer[ tmpTrackSize ] & 0xFF ) << " ";

					tmpTrackSize++;
				}
			}

			// S T A T U S   B Y T E
			// only write when ordered to use (GetUseRunningStatus) or for meta events
			if ( tmpDataChain[j].Data[0]&0xFF == 0xFF || tmpDataChain[j].Data[0] != tmpLastStatus || !GetUseRunningStatus() )
			{
				tmpBuffer[ tmpTrackSize ] = tmpDataChain[j].Data[0];

				cout << ", status: " << hex << ( tmpBuffer[ tmpTrackSize ] & 0xFF );
				tmpTrackSize++;				
			}
			tmpLastStatus = tmpDataChain[j].Data[0];

			// D A T A   B Y T E S
			// save the data bytes without status byte
			cout << ", data  : " << hex;

			for ( long k=1; k<tmpDataChain[j].DataSize; k++ )
			{
 				tmpBuffer[ tmpTrackSize ] = tmpDataChain[j].Data[k];

 				cout << ( tmpBuffer[ tmpTrackSize ] & 0xFF ) << " ";

 				tmpTrackSize++;
			}			
			cout << endl;
		}

		cout << "-------------size  : " << hex << tmpTrackSize << endl;

		WriteFixNumber( tmpTrackSize, 4 );								// header chunk length

		for ( long j = 0; j<tmpTrackSize; j++ )
		{
 			fSaveFile << char( tmpBuffer[j]&0xFF );
		}

		delete tmpBuffer;
	}

	fSaveFile.close();

	SetFileName( tmpFileName );
  CallMCMessage( this, cMCSequence, cMMSequenceSaveFile, "file saved" );
	SetChanged( false );

	return 0;
}

//=============================================================================
// initialize settings
// 31.08.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::InitAll( void )
{
  fCopyrightNotice = new char[strlen("copyright (c)") + 1];
  strcpy( fCopyrightNotice, "copyright (c)" );

	fFileName = new char[1];
	fFileName[0] = 0;

  fSequenceName = new char[1];
	fSequenceName[0] = 0;

	fDeltaTimesType = 120;
  fGrid = 120;
	fMarkerList = new TMidiList();
	fTempoList = 0;
	fTimeSignatureList = 0;
	fCuePointList = 0;
	fKeySignatureList = 0;
	fMetaDataList = 0;
	fTrackList = 0;
	fTrackCount = 0;
	fUseNoteOff = false;
	fUseRunningStatus = false;
	fSelectionStart = 0;
	fSelectionEnd = 0;
	fUse14BitController = true;

	fFileBuffer = 0;
	fBufferSize = 0;
	fChanged = false;
}

//=============================================================================
// remove all objects
// 21.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::RemoveAll( void )
{
	for ( int i = GetMarkerCount()-1; i>=0 ; i-- ) {	DeleteMarker( GetMarker(i) ); }
	for ( int i = GetTempoCount()-1; i>=0 ; i-- ) {	DeleteTempo( i ); }
	for ( int i = GetTimeSignatureCount()-1; i>=0 ; i-- ) { DeleteTimeSignature( i ); }
	for ( int i = GetCuePointCount()-1; i>=0 ; i-- ) {	DeleteCuePoint( i ); }
	for ( int i = GetKeySignatureCount()-1; i>=0 ; i-- ) {	DeleteKeySignature( i ); }
	for ( int i = GetMetaDataCount()-1; i>=0 ; i-- ) {	DeleteMetaData( i ); }
	for ( int i = GetTrackCount()-1; i>=0; i-- ) { DeleteTrack( i ); }

	if ( !fCopyrightNotice ) delete fCopyrightNotice;
  if ( !fFileName )	delete fFileName;
  if ( !fSequenceName ) delete fSequenceName;

	fTrackList = 0;
}

//=============================================================================
// load objects from file
// 27.02.2000 Adrian Meyer
//=============================================================================
short TMidiSequence::LoadFromFile( char* aFileName )
{
	char* tmpFileName = new char[strlen( aFileName ) + 1];
	strcpy( tmpFileName, aFileName );	

	RemoveAll();																				// remove everything
	InitAll();																					// init settings

	fFile.open( tmpFileName );													// open file
	if ( fFile == 0 ) { return 1; }
	
	ReadBuffer( 4 );    	                              // read header
	if ( strcmp( fFileBuffer, "MThd" ) != 0 ){ return 2; }
	ReadBuffer( 4 );    	                              // chunk size
	SetFileType( ReadShort( 2 ));												// file type
	SetTrackCount( ReadShort( 2 ));											// track count
	fTrackCount=0;																			// back to 0, see AddTrack
  SetDeltaTimesType( ReadShort( 2 ));									// time type

	ReadBuffer( 4 );    																
	while ( strcmp( fFileBuffer, "MTrk" ) == 0 )				// track chunk
	{
		TMidiTrack* tmpTrack = AddTrack();								// add track
		ReadBuffer( ReadLong( 4 ));
		
		long tmpPos = 0;    															// position in track
		long tmpCount = 0;																// count for dyn value
		long tmpOffset = 0;																// total offset
		short tmpStatus = 0;															// running status

		while ( tmpPos < fBufferSize )										// read events
		{
 			tmpOffset += ReadDynamic( tmpPos, &tmpCount );// read offset
 			tmpPos += tmpCount;
						                                         // channel event
			if (( fFileBuffer[ tmpPos ] &	0xF0 ) != 0xF0 ||
					( fFileBuffer[ tmpPos ] &	0x80 ) == 0 )	
			{
																											// first event is instrument patch of track
				if ( tmpOffset == 0  && ( fFileBuffer[ tmpPos ] & 0xF0 ) == 0xC0 )
				{
					tmpTrack->SetInstrumentPatch( fFileBuffer[ tmpPos + 1 ] );
					tmpPos++;
				}
				else																					// "normal" event
				{
  				TMidiEvent* tmpEvent = tmpTrack->AddEventEnd( tmpOffset );

  				if (( fFileBuffer[ tmpPos ] &	0x80 ) == 0 )	// running status
  				{											
            if ( GetUseRunningStatus() == false )
  						{	SetUseRunningStatus( true ); }

    				tmpEvent->SetData1( tmpStatus );
    				tmpEvent->SetData2( fFileBuffer[ tmpPos ] & 0xff );
    				tmpEvent->SetData3( fFileBuffer[ tmpPos + 1 ] & 0xff );

  					if ( (fFileBuffer[tmpPos]&0xF0) != 0xC0 && (fFileBuffer[tmpPos]&0xF0) != 0xD0 )
  						{	tmpPos++; }
  				}
  				else
  				{																							// normal event
    				tmpEvent->SetData1( fFileBuffer[ tmpPos ] & 0xff );
    				tmpEvent->SetData2( fFileBuffer[ tmpPos + 1 ] & 0xff );
    				tmpEvent->SetData3( fFileBuffer[ tmpPos + 2 ] & 0xff );
  					tmpStatus = ( fFileBuffer[ tmpPos ] & 0xff );
  					if ( (fFileBuffer[tmpPos]&0xF0) != 0xC0 && (fFileBuffer[tmpPos]&0xF0) != 0xD0 )
  						{	tmpPos += 2; }
  					else
  						{ tmpPos++; }
  				}
				}
			}
			else
			{
				if ((fFileBuffer[ tmpPos ] & 0xFF )== 0xFF )	// meta stuff
				{
					tmpPos++;
																											// read data
					long tmpSize = ( fFileBuffer[ tmpPos + 1 ] & 0xFF );
					char* tmpStr = new char[ tmpSize + 1 ];
 					tmpStr[ tmpSize ] = char( 0 );
					strncpy( tmpStr, &fFileBuffer[ tmpPos + 2 ], tmpSize ); 						

					/* switch 2nd byte of FF xx */
					switch ( fFileBuffer[tmpPos] & 0xff )       // apply data
					{
						case 0x00: { SetSequenceNumber( ((tmpStr[0]&0xff)<<8) +(tmpStr[1]&0xff) );	}; break;
						case 0x01: { tmpTrack->AddText( tmpStr, tmpOffset );	}; break;
						case 0x02: { SetCopyrightNotice( tmpStr );	}; break;
						case 0x03:
						{
							if ( tmpTrack->GetIndex() == 0 )
								SetSequenceName( tmpStr );	
  	  				else
								tmpTrack->SetTrackName( tmpStr );	
						}; break;

						case 0x04: { tmpTrack->SetInstrumentName( tmpStr );	}; break;
						case 0x05: { tmpTrack->AddLyric( tmpStr, tmpOffset );	}; break;
						case 0x06: { AddMarker( tmpOffset, tmpStr );	}; break;
						case 0x07: { AddCuePoint( tmpStr, tmpOffset );	}; break;
						case 0x20: { tmpTrack->AddChannelPrefix( tmpStr[0]&0xff, tmpOffset );	}; break;
						case 0x2f: { /* end of track*/  }; break;
   	  			case 0x51: { AddTempo( ((tmpStr[0]&0xff)<<16) + ((tmpStr[1]&0xff)<<8) + (tmpStr[2]&0xff), tmpOffset ); }; break;
						case 0x54: { /* SMPTE offset*/  }; break;
						case 0x58: { AddTimeSignature( tmpStr[0]&0xff, tmpStr[1]&0xff, tmpStr[2]&0xff, tmpStr[3]&0xff, tmpOffset	); }; break;
						case 0x59: { AddKeySignature( tmpStr[0]&0xff,	tmpStr[1]&0xff, tmpOffset	); }; break;
						case 0x7F: { AddMetaData( tmpStr,	tmpSize, tmpOffset	); }; break;

						default:
						{
//					cout << "Unknown meta data at: " << tmpOffset << " type: " << hex << (fFileBuffer[tmpPos] & 0xff) << dec << endl;
						};
					}

					delete tmpStr;
					tmpPos++;																		// goto size byte
					tmpPos += ( fFileBuffer[tmpPos] & 0xFF );		// jump over meta data
				}
			}

			++tmpPos;
		}

		TMidiEvent* tmpStartEvent = 0;

		if ( tmpTrack->GetEventCount() > 0 ) tmpStartEvent = tmpTrack->GetEvent(0);

		TMidiEvent* tmpEndEvent = 0;

		if ( tmpTrack->GetEventCount() > 0 )								// set prefered channel
		{
			tmpTrack->SetChannel( tmpTrack->GetEvent(0)->GetChannel());
		}

		while ( tmpStartEvent != 0 )											// position note on
		{
			tmpEndEvent = tmpStartEvent;

			if (( tmpStartEvent->GetData1()&0xF0 ) == 0x90 && tmpStartEvent->Ref == 0 )
			{
	  		while (	tmpEndEvent != 0 )										// find note off
				{            																			
					if ( tmpStartEvent->GetChannel() == tmpEndEvent->GetChannel() &&
					 	   ( (tmpEndEvent->GetData1()&0xF0) == 0x90 ||
								 (tmpEndEvent->GetData1()&0xF0) == 0x80	) &&
							 tmpStartEvent->GetData2() == tmpEndEvent->GetData2() &&
							 tmpEndEvent->GetData3() == 0 )
					{
						if (( tmpEndEvent->GetData1()&0xF0 ) == 0x80 )
							{	fUseNoteOff = true;	}

						tmpEndEvent->SetNoteOff( true );
						tmpEndEvent->Ref = tmpStartEvent;
						tmpStartEvent->Ref = tmpEndEvent;

						break;
					}
					tmpEndEvent = (TMidiEvent*)tmpEndEvent->Next;
				}
			}
	
			tmpStartEvent = (TMidiEvent*)tmpStartEvent->Next;			
		}

		ReadBuffer( 4 );		
	}

	fFile.close();	

	SetFileName( tmpFileName );
  SetGrid( GetDeltaTimesType() );

  CallMCMessage( this, cMCSequence, cMMSequenceOpenFile, "file opened" );

	SetChanged( false );

	return 0;
}

//=============================================================================
// new sequence
// 31.08.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::New( void )
{
	RemoveAll();
	InitAll();
	fChanged = true;
	SetChanged( false );
  CallMCMessage( this, cMCSequence, cMMSequenceNew, "new sequence" );
}

//=============================================================================
// show fFileBuffer
// 27.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::ShowBuffer( void )
{
	cout << dec;
	cout << "file buffer size: " << fBufferSize << " = ";
	cout << hex;

	for ( int i = 0; i<fBufferSize; i++ )
	{
		cout.width( 2 );
		cout.fill( '0' );
		cout << short( 0x000000FF & fFileBuffer[i] ) << " ";
	}
	cout << endl << dec;
}

//=============================================================================
// read dynamic value from file
// 29.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::ReadDynamic( long aPosition, long* aCount )
{
	long tmpReturn = 0;
	short i = aPosition;

	if (( tmpReturn = fFileBuffer[ i ] ) & 0x80 )
	{
		tmpReturn &= 0x7f;
		do
		{
			++i;
			tmpReturn = ( tmpReturn << 7 ) + ( fFileBuffer[ i ] & 0x7f );
			
		}	while ( fFileBuffer[ i ] & 0x80 );
	}
	*aCount = 1 + ( i - aPosition );
	return tmpReturn;
}


//=============================================================================
// read long from file
// 29.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::ReadLong( long Size )
{
	ReadBuffer( Size );

	long tmpReturn = 0;
	for ( int i = Size - 1; i>=0; i-- )
	{
		tmpReturn += ( fFileBuffer[ i ] & 0xFF ) << ( Size - i - 1 ) * 8;
	}
	return tmpReturn;
}

//=============================================================================
// read short from file
// 27.02.2000 Adrian Meyer
//=============================================================================
short TMidiSequence::ReadShort( long Size )
{
	ReadBuffer( Size );

	short tmpReturn = 0;
	for ( int i = Size - 1; i>=0; i-- )
	{
		tmpReturn += ( fFileBuffer[ i ] & 0xFF ) << ( Size - i - 1 ) * 8;
	}
	return tmpReturn;
}

//=============================================================================
// read buffer from file
// 27.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::ReadBuffer( long Size )
{
	if ( fFileBuffer !=0 )
		{	delete fFileBuffer; }

	fBufferSize = Size;
  fFileBuffer = 0;
	fFileBuffer = new char[ Size + 1 ];	

	char tmpChar;
	for ( long i = 0; i<Size; i++ )
	{
		fFile.get( tmpChar );
		fFileBuffer[i]=tmpChar;		
	}
	fFileBuffer[ Size ] = char( 0 );
}

//=============================================================================
// set 14bit controller use
// 04.04.2001 Adrian Meyer
//=============================================================================
void TMidiSequence::SetUse14BitController( bool aUse14BitController )
{
  fUse14BitController = aUse14BitController;
  CallMCMessage( this, cMCSequence, cMMSequenceSetUse14BitController, "use 14 bit controller set" );
}

//=============================================================================
// read 14 bit controller use
// 04.04.2001 Adrian Meyer
//=============================================================================
bool TMidiSequence::GetUse14BitController( void )
{
  return fUse14BitController;
}

//=============================================================================
// read measure/beat/tick															
// 05.03.2000 Adrian Meyer
//=============================================================================
char* TMidiSequence::GetMeasureBeatTick( long aOffset )
{
	long tmpMeasure = 0;
	long tmpBeat = 0;
	long tmpTick = 0;

	GetMeasureBeatTick( aOffset, &tmpMeasure, &tmpBeat, &tmpTick );

	sprintf( fMeasureBeatTick, "%d.%d.%.4d", (int)tmpMeasure, (int)tmpBeat, (int)tmpTick );

	return fMeasureBeatTick;	
}

//=============================================================================
// read measure beat and tick in different values     11.04.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::GetMeasureBeatTick( long aOffset, long* aMeasure, long* aBeat, long* aTick )
{
	sTimeSignatureList* tmpTS = 0;

	if ( fTimeSignatureList == 0 )
	{
		tmpTS = new sTimeSignatureList;
		tmpTS->Offset = 0;
		tmpTS->Next = 0;
		tmpTS->Prev = 0;
		tmpTS->Numerator = 4;
		tmpTS->Denominator = 2;
		tmpTS->MidiClocksPerQuarter = 8;
		tmpTS->NotesPerQuarter = 24;
	}
	else
	{
		tmpTS = fTimeSignatureList;
	}

	long tmpTicksPerBeat = 0;
	long tmpOffset = aOffset;

	long tmpMeasureCount = 1;
	long tmpBeatCount = 0;
	long tmpTickCount = 0;

  do
	{
		tmpTicksPerBeat = ((GetDeltaTimesType()&0x7FFF) * 4 ) /
										( 2 << ( tmpTS->Denominator - 1 ));

		if ( tmpTS->Next == 0 ) 								// last loop
		{
			tmpOffset = aOffset - tmpTS->Offset;
		}
		else
		{	
			if ( tmpTS->Next->Offset <= aOffset )
			{
				tmpOffset = tmpTS->Next->Offset - tmpTS->Offset;
			}
			else
			{
				tmpOffset = aOffset - tmpTS->Offset;
			}
		}		

		if ( tmpOffset < 0 ) { break; }

		tmpMeasureCount += tmpOffset / ( tmpTicksPerBeat * tmpTS->Numerator );
		tmpBeatCount = ( tmpOffset % ( tmpTicksPerBeat * tmpTS->Numerator ))
										/ tmpTicksPerBeat + 1;
		tmpTickCount = tmpOffset % tmpTicksPerBeat;

		tmpTS = tmpTS->Next;
	}	while ( tmpTS != 0 );

	*aMeasure = tmpMeasureCount;
	*aBeat = tmpBeatCount;
	*aTick = tmpTickCount;
}

//=============================================================================
// calculate offset with measure, beat and tick				29.08.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetOffset( long aMeasure, long aBeat, long Tick )
{
  long tmpOffset = 0;																	// temporary offset
  long tmpBeatCount;
  long tmpTicksPerBeat;

  for ( long i = aMeasure - 1 ; i>0 ; i-- )						// smaller than measure
  {
    GetTimeInformation( i, &tmpBeatCount, &tmpTicksPerBeat );
    tmpOffset += tmpBeatCount * tmpTicksPerBeat;
  }
																											// rest of measure
  GetTimeInformation( aMeasure, &tmpBeatCount, &tmpTicksPerBeat );
  tmpOffset += ( aBeat - 1 ) * tmpTicksPerBeat + Tick;

	return tmpOffset;
}

//=============================================================================
// read time information at measure										11.04.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::GetTimeInformation( long aMeasure, long* aBeatCount, long* aTicksPerBeat )
{
	long tmpBeatCount = 4;
	long tmpTicksPerBeat = (GetDeltaTimesType()&0x7FFF);
	long tmpMeasure = 0;
	long tmpOffset = 0;

	sTimeSignatureList* tmpTS = 0;
	if ( fTimeSignatureList == 0 )
	{
		tmpTS = new sTimeSignatureList;
		tmpTS->Offset = 0;
		tmpTS->Next = 0;
		tmpTS->Prev = 0;
		tmpTS->Numerator = 4;
		tmpTS->Denominator = 2;
		tmpTS->MidiClocksPerQuarter = 8;
		tmpTS->NotesPerQuarter = 24;
	}
	else
	{
		tmpTS = fTimeSignatureList;
	}

	while ( aMeasure>tmpMeasure )
	{
//		sTimeSignatureList* tmpTS = fTimeSignatureList;
		for ( int i = 0; i< GetTimeSignatureCount(); i++ )
		{
			if ( GetTimeSignature(i)->Offset > tmpOffset )
				{	break;}
			else
				{	tmpTS = GetTimeSignature( i ); }
		}

		tmpTicksPerBeat = ((GetDeltaTimesType()&0x7FFF) * 4 ) / ( 1 << tmpTS->Denominator );

		tmpBeatCount = tmpTS->Numerator;
		tmpOffset += tmpBeatCount * tmpTicksPerBeat;

		tmpMeasure++;		
	}
	*aBeatCount = tmpBeatCount;
	*aTicksPerBeat = tmpTicksPerBeat;
}

//=============================================================================
// read max event offset															11.04.2000 Adrian Meyer
//=============================================================================	
long TMidiSequence::GetMaxOffset( void )
{
	long tmpMaxOffset = 0;
	for ( int i = 0; i < GetTrackCount(); i++ )
	{
		for ( int j = 0; j < GetTrack(i)->GetEventCount(); j++ )
		{
			if ( GetTrack(i)->GetEvent(j)->GetOffset() > tmpMaxOffset )
			{                                             	
				tmpMaxOffset = GetTrack(i)->GetEvent(j)->GetOffset();
			}
		}
	}
	return tmpMaxOffset;
}

//=============================================================================
// write file type
// 18.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetFileType( short aFileType )
{
  if (( aFileType == cFileTypeSingle ) ||
      ( aFileType == cFileTypeMultiTrack ) ||
      ( aFileType == cFileTypeMultiSequence ))
  {
    fFileType = aFileType;
	  CallMCMessage( this, cMCSequence, cMMSequenceSetFileType, "file type set");
		SetChanged( true );
  }
}

//=============================================================================
// read file type
// 18.02.2000 Adrian Meyer
//=============================================================================
short TMidiSequence::GetFileType( void )
{
  return fFileType;
}

//=============================================================================
// write track count
// 18.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetTrackCount( int aTrackCount )
{
  fTrackCount = aTrackCount;
  CallMCMessage( this, cMCSequence, cMMSequenceSetTrackCount, "track count set");
}

//=============================================================================
// read track count
// 18.02.2000 Adrian Meyer
//=============================================================================
int TMidiSequence::GetTrackCount( void )
{
  return fTrackCount;
}

//=============================================================================
// write delta time type
// 18.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetDeltaTimesType( short aDeltaTimesType )
{
/*  if (( aDeltaTimesType == cDeltaTimesTypeTicks ) ||
      ( aDeltaTimesType == cDeltaTimesTypeSync ))*/
	{
    fDeltaTimesType = aDeltaTimesType;
	  CallMCMessage( this, cMCSequence, cMMSequenceSetDeltaTimesType, "delta times type set");
		SetChanged( true );
	}
}

//=============================================================================
// read delta times type
// 18.02.2000 Adrian Meyer
//=============================================================================
short TMidiSequence::GetDeltaTimesType( void )
{
  return fDeltaTimesType;
}

//=============================================================================
// write running status
// 18.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetUseRunningStatus( bool aUseRunningStatus )
{
  fUseRunningStatus = aUseRunningStatus;
  CallMCMessage( this, cMCSequence, cMMSequenceSetRunningStatus, "running status set");
	SetChanged( true );
}

//=============================================================================
// read running status
// 18.02.2000 Adrian Meyer
//=============================================================================
bool TMidiSequence::GetUseRunningStatus( void )
{
  return fUseRunningStatus;
}

//=============================================================================
// write use note off
// 01.03.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetUseNoteOff( bool aUseNoteOff )
{
	if ( fUseNoteOff == aUseNoteOff ) return;

	fUseNoteOff = aUseNoteOff;

	short tmpStatus = 0;
	if ( fUseNoteOff )
		{ tmpStatus = cESNoteOff; }
	else
		{ tmpStatus = cESNoteOn; }

	for ( long i = 0; i<GetTrackCount(); i++ )
	{
		for ( long j = 0; j<GetTrack(i)->GetEventCount(); j++ )
		{
			if ( GetTrack(i)->GetEvent(j)->GetNoteOff())
			{
				GetTrack(i)->GetEvent(j)->SetData1(
					(GetTrack(i)->GetEvent(j)->GetData1()&0x0f)|tmpStatus);
			}
		}
	}

  CallMCMessage( this, cMCSequence, cMMSequenceSetUseNoteOff, "use note off set");
	SetChanged( true );
}

//=============================================================================
// read use note off
// 01.03.2000 Adrian Meyer
//=============================================================================
bool TMidiSequence::GetUseNoteOff( void )
{
	return fUseNoteOff;
}

//=============================================================================
// write copyright
// 18.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetCopyrightNotice( char *aCopyrightNotice )
{
  if ( !fCopyrightNotice ) delete fCopyrightNotice;
  fCopyrightNotice = new char[ strlen( aCopyrightNotice ) + 1 ];
  strcpy( fCopyrightNotice, aCopyrightNotice );

  CallMCMessage( this, cMCSequence, cMMSequenceSetCopyrightNotice, "copyright notice set");
	SetChanged( true );
}

//=============================================================================
// read copyright
// 18.02.2000 Adrian Meyer
//=============================================================================
char* TMidiSequence::GetCopyrightNotice( void )
{
	return fCopyrightNotice;
}

//=============================================================================
// write sequence name
// 18.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetSequenceName( char* aSequenceName )
{
	if ( !fSequenceName ) delete fSequenceName;
  fSequenceName = new char[ strlen( aSequenceName ) + 1 ];
  strcpy( fSequenceName, aSequenceName );

  CallMCMessage( this, cMCSequence, cMMSequenceSetSequenceName, "sequence name set");
	SetChanged( true );
}

//=============================================================================
// read sequence name
// 18.02.2000 Adrian Meyer
//=============================================================================
char* TMidiSequence::GetSequenceName( void )
{
  return fSequenceName;
}

//=============================================================================
// write sequence number
// 20.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetSequenceNumber( short aSequenceNumber )
{
  fSequenceNumber = aSequenceNumber;
  CallMCMessage( this, cMCSequence, cMMSequenceSetSequenceNumber, "sequence number set");
	SetChanged( true );
}

//=============================================================================
// read sequence number
// 20.02.2000 Adrian Meyer
//=============================================================================
short TMidiSequence::GetSequenceNumber( void )
{
  return fSequenceNumber;
}

//=============================================================================
// write SMPTE offset
// 20.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetSMPTEOffset( long aSMPTEOffset )
{
  fSMPTEOffset = aSMPTEOffset;
  CallMCMessage( this, cMCSequence, cMMSequenceSetSMPTEOffset, "SMPTE offset set");
	SetChanged( true );
}

//=============================================================================
// read SMPTE offset
// 20.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetSMPTEOffset( void )
{
  return fSMPTEOffset;
}

//=============================================================================
// constructor for marker
// 19.04.2001 Adrian Meyer
//=============================================================================
TMidiMarker::TMidiMarker( long aOffset, char* aName )
{
  fName = 0;
  Offset = aOffset;
  SetName( aName );
}

//=============================================================================
// set marker name
// 19.04.2001 Adrian Meyer
//=============================================================================
void TMidiMarker::SetName( char* aName )
{
  if (fName) delete fName;
  fName = new char[ strlen(aName)+1 ];
  strcpy( fName, aName );
}

//=============================================================================
// read marker name
// 19.04.2001 Adrian Meyer
//=============================================================================
char* TMidiMarker::GetName( void )
{
  return fName;
}

//=============================================================================
// add marker
// 20.02.2000 Adrian Meyer
//=============================================================================
TMidiMarker* TMidiSequence::AddMarker( long aOffset, char* aMarkerName )
{
  TMidiMarker* tmpMarker = new TMidiMarker( aOffset, aMarkerName );
  fMarkerList->Add( tmpMarker );

  CallMCMessage( this, cMCSequence, cMMSequenceAddMarker, "marker added");
	SetChanged( true );

	return tmpMarker;
}

//=============================================================================
// set marker offset
// 19.04.2001 Adrian Meyer
//=============================================================================
void TMidiSequence::SetMarkerOffset( TMidiMarker* aMarker, long aOffset )
{
	fMarkerList->SetOffsetSorted( aOffset, aMarker );
}

//=============================================================================
// rename marker
// 20.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::RenameMarker( TMidiMarker* aMarker, char* aMarkerName )
{
  aMarker->SetName( aMarkerName );

	CallMCMessage( this, cMCSequence, cMMSequenceRenameMarker, "marker renamed");
	SetChanged( true );
}

//=============================================================================
// delete a marker
// 19.04.2001 Adrian Meyer
//=============================================================================
void TMidiSequence::DeleteMarker( TMidiMarker* aMarker )
{
  fMarkerList->Delete( aMarker );

	CallMCMessage( this, cMCSequence, cMMSequenceDeleteMarker, "marker deleted");
	SetChanged( true );
}

//=============================================================================
// read marker list
// 20.02.2000 Adrian Meyer
//=============================================================================
TMidiMarker* TMidiSequence::GetMarker( long aID )
{
	return (TMidiMarker*)fMarkerList->GetItem( aID );
}

//=============================================================================
// read marker list count
// 20.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetMarkerCount( void )
{
	return fMarkerList->GetCount();
}

//=============================================================================
// add tempo
// 20.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::AddTempo( long aTempo, long aOffset )
{
  long tmpID = 0;
	sTempoList* tmpTempoList;

  if ( fTempoList == 0 )
  {
    tmpTempoList = fTempoList = new sTempoList;
  }
  else
  {
    tmpTempoList = fTempoList;
		while ( tmpTempoList->Next != 0 )
    {
			tmpTempoList = tmpTempoList->Next;
    }
    tmpTempoList->Next = new sTempoList;
	  tmpID = tmpTempoList->ID;
		tmpTempoList = tmpTempoList->Next;
    tmpID++;
  }

  tmpTempoList->Next = 0;
  tmpTempoList->ID = tmpID;
  tmpTempoList->Offset = aOffset;
  tmpTempoList->Tempo = aTempo;

  CallMCMessage( this, cMCSequence, cMMSequenceAddTempo, "tempo added");
	SetChanged( true );
}

//=============================================================================
// reading tempo
// 20.02.2000 Adrian Meyer
//=============================================================================
sTempoList* TMidiSequence::GetTempo( long aID )
{
  sTempoList* tmpTempoList = fTempoList;
  if ( fTempoList != 0 )
  {
    while (( tmpTempoList->Next != 0 ) && ( tmpTempoList->ID != aID ))
    {
      tmpTempoList = tmpTempoList->Next;
    }
    return tmpTempoList;
  }
  return fTempoList;
}

//=============================================================================
// get tempo at offset
// 08.12.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetTempoAt( long aOffset )
{
	long tmpTempo = 0;
	for ( int i=0; i<GetTempoCount(); i++ )
	{
		if ( GetTempo(i)->Offset <= aOffset )
			tmpTempo = GetTempo(i)->Tempo;
	}
	return tmpTempo;
}

//=============================================================================
// read tempo list count
// 20.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetTempoCount( void )
{
  sTempoList* tmpTempoList = fTempoList;
  if ( fTempoList != 0 )
  {
    while ( tmpTempoList->Next != 0 )
    {
      tmpTempoList = tmpTempoList->Next;
    }
    return tmpTempoList->ID + 1;
  }
  else
  {
	  return 0;
  }
}

//=============================================================================
// delete a tempo
// 20.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::DeleteTempo( long aID )
{
	sTempoList* tmpTempoList = 0;
	sTempoList* delTempoList = GetTempo( aID );

  if ( delTempoList != 0 )														
  {
    if ( fTempoList == delTempoList )									// deleting 1st
    {
      fTempoList = delTempoList->Next;
    }
		else
		{
			GetTempo( aID - 1 )->Next = delTempoList->Next;			
		}

    delete delTempoList;
  }

  if ( fTempoList != 0 )															// renumber ID's
  {
		tmpTempoList = fTempoList;
    long tmpID = 0;

    while ( tmpTempoList != 0 )
    {
      tmpTempoList->ID = tmpID++;
			tmpTempoList = tmpTempoList->Next;
    }
  }

  CallMCMessage( this, cMCSequence, cMMSequenceDeleteTempo, "tempo deleted");
	SetChanged( true );
}

//=============================================================================
// add time signature
// 20.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::AddTimeSignature( int aNumerator, int aDenominator,
			int aMidiClocksPerQuarter, int aNotesPerQuarter, long aOffset )
{
  long tmpID = 0;
	sTimeSignatureList* tmpTimeSignatureList;

  if ( fTimeSignatureList == 0 )
  {
    tmpTimeSignatureList = fTimeSignatureList = new sTimeSignatureList;
		tmpTimeSignatureList->Prev = 0;
  }
  else
  {
    tmpTimeSignatureList = fTimeSignatureList;
		while ( tmpTimeSignatureList->Next != 0 )
    {
			tmpTimeSignatureList = tmpTimeSignatureList->Next;
    }
    tmpTimeSignatureList->Next = new sTimeSignatureList;
		tmpTimeSignatureList->Next->Prev = tmpTimeSignatureList;

	  tmpID = tmpTimeSignatureList->ID;
		tmpTimeSignatureList = tmpTimeSignatureList->Next;
    tmpID++;
  }

  tmpTimeSignatureList->Next = 0;
  tmpTimeSignatureList->ID = tmpID;
  tmpTimeSignatureList->Offset = aOffset;
  tmpTimeSignatureList->Numerator = aNumerator;
  tmpTimeSignatureList->Denominator = aDenominator;
	tmpTimeSignatureList->MidiClocksPerQuarter = aMidiClocksPerQuarter;
  tmpTimeSignatureList->NotesPerQuarter = aNotesPerQuarter;

  CallMCMessage( this, cMCSequence, cMMSequenceAddTimeSignature, "time signature added");
	SetChanged( true );
	return tmpID;
}

//=============================================================================
// change time signature
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::ChangeTimeSignature( long aID, int aNumerator, int aDenominator,
			int aMidiClocksPerQuarter, int aNotesPerQuarter )
{
  if ( GetTimeSignature( aID ) != 0 )
	{
		sTimeSignatureList* tmpTimeSignatureList = GetTimeSignature( aID );
		tmpTimeSignatureList->Numerator = aNumerator;
		tmpTimeSignatureList->Denominator = aDenominator;
		tmpTimeSignatureList->MidiClocksPerQuarter = aMidiClocksPerQuarter;
		tmpTimeSignatureList->NotesPerQuarter = aNotesPerQuarter;

	  CallMCMessage( this, cMCSequence, cMMSequenceChangeTimeSignature, "time signature changed");
		SetChanged( true );
	}
}

//=============================================================================
// delete a time signature
// 20.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::DeleteTimeSignature( long aID )
{
	sTimeSignatureList* tmpTimeSignatureList = 0;
	sTimeSignatureList* delTimeSignatureList = GetTimeSignature( aID );

  if ( delTimeSignatureList != 0 )														
  {
    if ( fTimeSignatureList == delTimeSignatureList )	// deleting 1st
    {
      fTimeSignatureList = delTimeSignatureList->Next;
			if ( fTimeSignatureList != 0 ){	fTimeSignatureList->Prev = 0; }
    }
		else
		{
			GetTimeSignature( aID - 1 )->Next = delTimeSignatureList->Next;			
			if ( GetTimeSignature( aID - 1 )->Next != 0 )
				{ GetTimeSignature( aID - 1 )->Next->Prev = GetTimeSignature( aID - 1 ); }
		}

    delete delTimeSignatureList;
  }

  if ( fTimeSignatureList != 0 )											// renumber ID's
  {
		tmpTimeSignatureList = fTimeSignatureList;
    long tmpID = 0;

    while ( tmpTimeSignatureList != 0 )
    {
      tmpTimeSignatureList->ID = tmpID++;
			tmpTimeSignatureList = tmpTimeSignatureList->Next;
    }
  }
  CallMCMessage( this, cMCSequence, cMMSequenceDeleteTimeSignature, "time signature deleted");
	SetChanged( true );
}

//=============================================================================
// read time signature
// 20.02.2000 Adrian Meyer
//=============================================================================
sTimeSignatureList* TMidiSequence::GetTimeSignature( long aID )
{
	if ( fTimeSignatureList == 0 ) { AddTimeSignature( 4, 2, 24, 8, 0 ); }

  sTimeSignatureList* tmpTimeSignatureList = fTimeSignatureList;
  if ( fTimeSignatureList != 0 )
  {
    while (( tmpTimeSignatureList->Next != 0 ) && ( tmpTimeSignatureList->ID != aID ))
    {
      tmpTimeSignatureList = tmpTimeSignatureList->Next;
    }
    return tmpTimeSignatureList;
  }
  return fTimeSignatureList;
}

//=============================================================================
// read time signature
// 20.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetTimeSignatureCount( void )
{
  sTimeSignatureList* tmpTimeSignatureList = fTimeSignatureList;
  if ( fTimeSignatureList != 0 )
  {
    while ( tmpTimeSignatureList->Next != 0 )
    {
      tmpTimeSignatureList = tmpTimeSignatureList->Next;
    }
    return tmpTimeSignatureList->ID + 1;
  }
  else
  {
	  return 0;
  }
}

//=============================================================================
// sort time signature list
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SortTimeSignature( void )
{
  if ( fTimeSignatureList != 0 )
	{
		sTimeSignatureList** tmpSortList = new sTimeSignatureList*[ GetTimeSignatureCount() ];

		long tmpCount = GetTimeSignatureCount();
		for ( int i = 0; i<tmpCount; i++ )
			{	tmpSortList[i] = GetTimeSignature(i); }

		for ( int i = 0; i<tmpCount - 1; i++ )
		{
			for ( int j = i+1; j<tmpCount; j++ )
			{
				if ( tmpSortList[i]->Offset > tmpSortList[j]->Offset )
				{
					sTimeSignatureList* tmpTimeSignatureList = tmpSortList[i];
					tmpSortList[i] = tmpSortList[j];
					tmpSortList[j] = tmpTimeSignatureList;
				}
			}
		}

		for ( int i= tmpCount - 1; i>=0 ; i-- )
		{
      tmpSortList[i]->ID = i;
			if ( i == tmpCount - 1 )
			{
        tmpSortList[i]->Next = 0;
			}
			else
			{
        tmpSortList[i]->Next = tmpSortList[i+1];
			}
		}

		fTimeSignatureList = tmpSortList[0];

		delete tmpSortList;
	}

	CallMCMessage( this, cMCSequence, cMMSequenceSortTimeSignature, "time signature sorted");
	SetChanged( true );
}

//=============================================================================
// add cue point
// 21.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::AddCuePoint( char* aCuePointName, long aOffset )
{
  long tmpID = 0;
	sCuePointList* tmpCuePointList;

  if ( fCuePointList == 0 )
  {
    tmpCuePointList = fCuePointList = new sCuePointList;
  }
  else
  {
    tmpCuePointList = fCuePointList;
		while ( tmpCuePointList->Next != 0 )
    {
			tmpCuePointList = tmpCuePointList->Next;
    }
    tmpCuePointList->Next = new sCuePointList;
	  tmpID = tmpCuePointList->ID;
		tmpCuePointList = tmpCuePointList->Next;
    tmpID++;
  }

  tmpCuePointList->Next = 0;
  tmpCuePointList->ID = tmpID;
  tmpCuePointList->Offset = aOffset;
  tmpCuePointList->CuePointName = new char[ sizeof( aCuePointName ) + 1 ];
  strcpy( tmpCuePointList->CuePointName, aCuePointName );

  CallMCMessage( this, cMCSequence, cMMSequenceAddCuePoint, "cue point added");
	SetChanged( true );
}

//=============================================================================
// rename cue point
// 21.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::RenameCuePoint( char* aCuePointName, long aID )
{
	if ( GetCuePoint( aID ) != 0 )
	{
		delete GetCuePoint( aID )->CuePointName;
		GetCuePoint( aID )->CuePointName = new char[ sizeof( aCuePointName ) + 1 ];
    strcpy( GetCuePoint( aID )->CuePointName, aCuePointName );
	  CallMCMessage( this, cMCSequence, cMMSequenceRenameCuePoint, "cue point renamed");
		SetChanged( true );
	} 	
}

//=============================================================================
// delete a cue point
// 21.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::DeleteCuePoint( long aID )
{
	sCuePointList* tmpCuePointList = 0;
	sCuePointList* delCuePointList = GetCuePoint( aID );

  if ( delCuePointList != 0 )													// delete the CuePoint
  {
    if ( fCuePointList == delCuePointList )						// deleting 1st
    {
      fCuePointList = delCuePointList->Next;
    }
		else
		{
			GetCuePoint( aID - 1 )->Next = delCuePointList->Next;			
		}

    if ( delCuePointList->CuePointName != 0 ) 				// delete name
    {
			delete delCuePointList->CuePointName;
    }
    delete delCuePointList;
  }

  if ( fCuePointList != 0 )														// renumber ID's
  {
		tmpCuePointList = fCuePointList;
    long tmpID = 0;

    while ( tmpCuePointList != 0 )
    {
      tmpCuePointList->ID = tmpID++;
			tmpCuePointList = tmpCuePointList->Next;
    }
  }

  CallMCMessage( this, cMCSequence, cMMSequenceDeleteCuePoint, "cue point deleted");
	SetChanged( true );
}

//=============================================================================
// read cue point list
// 21.02.2000 Adrian Meyer
//=============================================================================
sCuePointList* TMidiSequence::GetCuePoint( long aID )
{
  sCuePointList* tmpCuePointList = fCuePointList;
  if ( fCuePointList != 0 )
  {
    while (( tmpCuePointList->Next != 0 ) && ( tmpCuePointList->ID != aID ))
    {
      tmpCuePointList = tmpCuePointList->Next;
    }
    return tmpCuePointList;
  }
  return fCuePointList;
}

//=============================================================================
// read cue point list count
// 21.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetCuePointCount( void )
{
  sCuePointList* tmpCuePointList = fCuePointList;
  if ( fCuePointList != 0 )
  {
    while ( tmpCuePointList->Next != 0 )
    {
      tmpCuePointList = tmpCuePointList->Next;
    }
    return tmpCuePointList->ID + 1;
  }
  else
  {
	  return 0;
  }
}

//=============================================================================
// add Key signature
// 20.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::AddKeySignature( int aSharpFlatCount, short aIsMinor,
																			 long aOffset )
{
  long tmpID = 0;
	sKeySignatureList* tmpKeySignatureList;

  if ( fKeySignatureList == 0 )
  {
    tmpKeySignatureList = fKeySignatureList = new sKeySignatureList;
  }
  else
  {
    tmpKeySignatureList = fKeySignatureList;
		while ( tmpKeySignatureList->Next != 0 )
    {
			tmpKeySignatureList = tmpKeySignatureList->Next;
    }
    tmpKeySignatureList->Next = new sKeySignatureList;
	  tmpID = tmpKeySignatureList->ID;
		tmpKeySignatureList = tmpKeySignatureList->Next;
    tmpID++;
  }

  tmpKeySignatureList->Next = 0;
  tmpKeySignatureList->ID = tmpID;
  tmpKeySignatureList->Offset = aOffset;
  tmpKeySignatureList->SharpFlatCount = aSharpFlatCount;
  tmpKeySignatureList->IsMinor = aIsMinor;

  CallMCMessage( this, cMCSequence, cMMSequenceAddKeySignature, "key signature added");
	SetChanged( true );
	return tmpID;
}

//=============================================================================
// delete a Key signature
// 20.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::DeleteKeySignature( long aID )
{
	sKeySignatureList* tmpKeySignatureList = 0;
	sKeySignatureList* delKeySignatureList = GetKeySignature( aID );

  if ( delKeySignatureList != 0 )														
  {
    if ( fKeySignatureList == delKeySignatureList )	  // deleting 1st
    {
      fKeySignatureList = delKeySignatureList->Next;
    }
		else
		{
			GetKeySignature( aID - 1 )->Next = delKeySignatureList->Next;			
		}

    delete delKeySignatureList;
  }

  if ( fKeySignatureList != 0 )											  // renumber ID's
  {
		tmpKeySignatureList = fKeySignatureList;
    long tmpID = 0;
    while ( tmpKeySignatureList != 0 )
    {
      tmpKeySignatureList->ID = tmpID++;
			tmpKeySignatureList = tmpKeySignatureList->Next;
    }
  }

  CallMCMessage( this, cMCSequence, cMMSequenceDeleteKeySignature, "key signature deleted");
	SetChanged( true );
}

//=============================================================================
// read Key signature
// 20.02.2000 Adrian Meyer
//=============================================================================
sKeySignatureList* TMidiSequence::GetKeySignature( long aID )
{
  sKeySignatureList* tmpKeySignatureList = fKeySignatureList;
  if ( fKeySignatureList != 0 )
  {
    while (( tmpKeySignatureList->Next != 0 ) && ( tmpKeySignatureList->ID != aID ))
    {
      tmpKeySignatureList = tmpKeySignatureList->Next;
    }
    return tmpKeySignatureList;
  }
  return fKeySignatureList;
}

//=============================================================================
// read Key signature
// 20.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetKeySignatureCount( void )
{
  sKeySignatureList* tmpKeySignatureList = fKeySignatureList;
  if ( fKeySignatureList != 0 )
  {
    while ( tmpKeySignatureList->Next != 0 )
    {
      tmpKeySignatureList = tmpKeySignatureList->Next;
    }
    return tmpKeySignatureList->ID + 1;
  }
  else
  {
	  return 0;
  }
}

//=============================================================================
// sort time signature list
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SortKeySignature( void )
{
  if ( fKeySignatureList != 0 )
	{
		sKeySignatureList** tmpSortList = new sKeySignatureList*[ GetKeySignatureCount() ];

		long tmpCount = GetKeySignatureCount();
		for ( int i = 0; i<tmpCount; i++ )
			{	tmpSortList[i] = GetKeySignature(i); }

		for ( int i = 0; i<tmpCount - 1; i++ )
		{
			for ( int j = i+1; j<tmpCount; j++ )
			{
				if ( tmpSortList[i]->Offset > tmpSortList[j]->Offset )
				{
					sKeySignatureList* tmpKeySignatureList = tmpSortList[i];
					tmpSortList[i] = tmpSortList[j];
					tmpSortList[j] = tmpKeySignatureList;
				}
			}
		}

		for ( int i= tmpCount - 1; i>=0 ; i-- )
		{
      tmpSortList[i]->ID = i;
			if ( i == tmpCount - 1 )
			{
        tmpSortList[i]->Next = 0;
			}
			else
			{
        tmpSortList[i]->Next = tmpSortList[i+1];
			}
		}

		fKeySignatureList = tmpSortList[0];

		delete tmpSortList;
	}

	CallMCMessage( this, cMCSequence, cMMSequenceSortKeySignature, "key signatures sorted");
	SetChanged( true );
}


//=============================================================================
// change key signature	
// 13.09.2000 Adrian Meyer
//=============================================================================								
void TMidiSequence::ChangeKeySignature( long aID, int aSharpFlatCount, short aIsMinor )
{
  if ( GetKeySignature( aID ) != 0 )
	{
		sKeySignatureList* tmpKeySignatureList = GetKeySignature( aID );

	  tmpKeySignatureList->SharpFlatCount = aSharpFlatCount;
  	tmpKeySignatureList->IsMinor = aIsMinor;

	  CallMCMessage( this, cMCSequence, cMMSequenceChangeKeySignature, "key signature changed");
		SetChanged( true );
	}
}

//=============================================================================
// add meta data
// 21.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::AddMetaData( char* aMetaData, long aSize, long aOffset )
{
  long tmpID = 0;
	sMetaDataList* tmpMetaDataList;

  if ( fMetaDataList == 0 )
  {
    tmpMetaDataList = fMetaDataList = new sMetaDataList;
  }
  else
  {
    tmpMetaDataList = fMetaDataList;
		while ( tmpMetaDataList->Next != 0 )
    {
			tmpMetaDataList = tmpMetaDataList->Next;
    }
    tmpMetaDataList->Next = new sMetaDataList;
	  tmpID = tmpMetaDataList->ID;
		tmpMetaDataList = tmpMetaDataList->Next;
    tmpID++;
  }

  tmpMetaDataList->Next = 0;
  tmpMetaDataList->ID = tmpID;
  tmpMetaDataList->Offset = aOffset;
  tmpMetaDataList->Size = aSize;
  tmpMetaDataList->MetaData = new char[ aSize + 1 ];

  for ( int i = 0; i<aSize; i++ )
  {
    tmpMetaDataList->MetaData[ i ] = aMetaData[ i ];
  }

  CallMCMessage( this, cMCSequence, cMMSequenceAddMetaData, "meta data added");
	SetChanged( true );
}

//=============================================================================
// delete a meta data
// 21.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::DeleteMetaData( long aID )
{
	sMetaDataList* tmpMetaDataList = 0;
	sMetaDataList* delMetaDataList = GetMetaData( aID );

  if ( delMetaDataList != 0 )													// delete the MetaData
  {
    if ( fMetaDataList == delMetaDataList )						// deleting 1st
    {
      fMetaDataList = delMetaDataList->Next;
    }
		else
		{
			GetMetaData( aID - 1 )->Next = delMetaDataList->Next;			
		}

    if ( delMetaDataList->MetaData != 0 ) 						// delete data
    {
			delete delMetaDataList->MetaData;
    }
    delete delMetaDataList;
  }

  if ( fMetaDataList != 0 )														// renumber ID's
  {
		tmpMetaDataList = fMetaDataList;
    long tmpID = 0;

    while ( tmpMetaDataList != 0 )
    {
      tmpMetaDataList->ID = tmpID++;
			tmpMetaDataList = tmpMetaDataList->Next;
    }
  }

  CallMCMessage( this, cMCSequence, cMMSequenceDeleteMetaData, "meta data deleted");
	SetChanged( true );
}

//=============================================================================
// read meta data list
// 21.02.2000 Adrian Meyer
//=============================================================================
sMetaDataList* TMidiSequence::GetMetaData( long aID )
{
  sMetaDataList* tmpMetaDataList = fMetaDataList;
  if ( fMetaDataList != 0 )
  {
    while (( tmpMetaDataList->Next != 0 ) && ( tmpMetaDataList->ID != aID ))
    {
      tmpMetaDataList = tmpMetaDataList->Next;
    }
    return tmpMetaDataList;
  }
  return fMetaDataList;
}

//=============================================================================
// read meta data list count
// 21.02.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetMetaDataCount( void )
{
  sMetaDataList* tmpMetaDataList = fMetaDataList;
  if ( fMetaDataList != 0 )
  {
    while ( tmpMetaDataList->Next != 0 )
    {
      tmpMetaDataList = tmpMetaDataList->Next;
    }
    return tmpMetaDataList->ID + 1;
  }
  else
  {
	  return 0;
  }
}

//=============================================================================
// add new track
// 21.02.2000 Adrian Meyer
//=============================================================================
TMidiTrack* TMidiSequence::AddTrack( void )
{
  long tmpID = 0;
	sTrackList* tmpTrackList;

  if ( fTrackList == 0 )
  {
		fTrackList = new sTrackList;
    tmpTrackList = fTrackList;
  }
  else
  {
    tmpTrackList = fTrackList;
		while ( tmpTrackList->Next != 0 )
    {
			tmpTrackList = tmpTrackList->Next;
    }
    tmpTrackList->Next = new sTrackList;
		tmpTrackList->Next->Prev = tmpTrackList;
	  tmpID = tmpTrackList->ID;
		tmpTrackList = tmpTrackList->Next;
    tmpID++;
  }

  fTrackCount++;
  tmpTrackList->Next = 0;
  tmpTrackList->ID = tmpID;
	tmpTrackList->Track = new TMidiTrack( this );

  CallMCMessage( this, cMCSequence, cMMSequenceAddTrack, "track added");
	SetChanged( true );																	// sequence changed

	return tmpTrackList->Track;
}

//=============================================================================
// delete a track
// 21.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::DeleteTrack( long aID )
{
	sTrackList* tmpTrackList = 0;
	sTrackList* delTrackList = GetTrackList( aID );
	TMidiTrack* tmpMidiTrack = 0;

  if ( delTrackList != 0 )														// delete the Track
  {
    if ( fTrackList == delTrackList )									// deleting 1st
    {
      fTrackList = delTrackList->Next;
    }
		else
		{
			GetTrackList( aID - 1 )->Next = delTrackList->Next;			
		}

    if ( delTrackList->Track != 0 ) 									// delete data
    {
			tmpMidiTrack = delTrackList->Track;
    }

		if ( fTrackList == delTrackList )									// delete struct
		{
	    delete delTrackList;
			fTrackList = 0;
		}
		else
		{
  	  delete delTrackList;
		}

    fTrackCount--;
  }

  if ( fTrackList != 0 )															// renumber ID's
  {
		tmpTrackList = fTrackList;
    long tmpID = 0;

    while ( tmpTrackList != 0 )
    {
      tmpTrackList->ID = tmpID++;
			tmpTrackList = tmpTrackList->Next;
    }
  }

  if ( tmpMidiTrack != 0 )														// destroy track
	{
		delete tmpMidiTrack;
	}

  CallMCMessage( this, cMCSequence, cMMSequenceDeleteTrack, "track deleted" );
	SetChanged( true );																	// sequence changed
}

//=============================================================================
// read track list item
// 21.02.2000 Adrian Meyer
//=============================================================================
sTrackList* TMidiSequence::GetTrackList( long aID )
{
  sTrackList* tmpTrackList = fTrackList;
  if ( fTrackList != 0 )
  {
    while (( tmpTrackList->Next != 0 ) && ( tmpTrackList->ID != aID ))
    {
      tmpTrackList = tmpTrackList->Next;
    }
    return tmpTrackList;
  }
  return fTrackList;
}

//=============================================================================
// read track
// 21.02.2000 Adrian Meyer
//=============================================================================
TMidiTrack* TMidiSequence::GetTrack( long aID )
{
  sTrackList* tmpTrackList = fTrackList;
  if ( fTrackList != 0 )
  {
    while (( tmpTrackList->Next != 0 ) && ( tmpTrackList->ID != aID ))
    {
      tmpTrackList = tmpTrackList->Next;
    }
    return tmpTrackList->Track;
  }
  return fTrackList->Track;
}


//=============================================================================
// set thchanged flag	
// 21.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetChanged( bool aChanged )
{
	if ( fChanged != aChanged )													// new value?
	{
	  fChanged = aChanged;															// take it
	  CallMCMessage( this, cMCSequence, cMMSequenceChanged, "sequence changed");		
	}
}

//=============================================================================
// read the changed flag
// 21.02.2000 Adrian Meyer
//=============================================================================
bool TMidiSequence::GetChanged( void )
{
	return fChanged;																		// read chagned flag
}													

//=============================================================================
// write file name
// 21.02.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetFileName( char* aFileName )
{
  if ( !fFileName ) delete fFileName;
  fFileName = new char[ strlen( aFileName ) + 1 ];
  strcpy( fFileName, aFileName );
}

//=============================================================================
// read file name
// 21.02.2000 Adrian Meyer
//=============================================================================
char* TMidiSequence::GetFileName( void )
{
	return fFileName;
}

//=============================================================================
// write selection start
// 09.09.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetSelectionStart( long aSelectionStart )
{
	if ( fSelectionStart == aSelectionStart )	{ return; }// no new value
	fSelectionStart = aSelectionStart;
  CallMCMessage( this, cMCSequence, cMMSequenceSetSelectionStart, "selection start set");		
	if ( fSelectionEnd < fSelectionStart )
	{
		SetSelectionEnd( aSelectionStart );
	}
}

//=============================================================================
// read selection start
// 09.09.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetSelectionStart( void )
{
	return fSelectionStart;
}

//=============================================================================
// write selection end
// 09.09.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetSelectionEnd( long aSelectionEnd )
{
	if ( fSelectionEnd == aSelectionEnd )	{ return; }		// no new value
	if ( aSelectionEnd < fSelectionStart )
	{
		fSelectionEnd = fSelectionStart;
	}
	else
	{
		fSelectionEnd = aSelectionEnd;
	}
  CallMCMessage( this, cMCSequence, cMMSequenceSetSelectionEnd, "selection end set");		
}

//=============================================================================
// read selection end
// 09.09.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetSelectionEnd( void )
{
	return fSelectionEnd;
}

//=============================================================================
// write position
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMidiSequence::SetPosition( long aPosition )
{
	fPosition = aPosition;
  CallMCMessage( this, cMCSequence, cMMSequenceSetPosition, "position changed");		
}

//=============================================================================
// read position
// 11.09.2000 Adrian Meyer
//=============================================================================
long TMidiSequence::GetPosition( void )
{
	return fPosition;
}

//=============================================================================
// write grid
// 21.02.2001 Adrian Meyer
//=============================================================================
void TMidiSequence::SetGrid( long aGrid )
{
  fGrid = aGrid;
  CallMCMessage( this, cMCSequence, cMMSequenceSetGrid, "grid set");		
}

//=============================================================================
// read grid
// 21.02.2001 Adrian Meyer
//=============================================================================
long TMidiSequence::GetGrid( void )
{
  return fGrid;
}
