//=============================================================================
//  model view control base class implementation
//=============================================================================
#include <mvcBase.h>
#include <iostream.h>
#include <string.h>
#include <stdio.h>

//=============================================================================
// constructor TMvcBase class
//=============================================================================
TMvcBase::TMvcBase(void)
{
  fWidgetList = g_list_alloc();												// alloc widget list
	fUpdateView = false;
  fLanguageFileName = 0;															// language file name
}

//=============================================================================
// destructor TMvcBase class
//=============================================================================
TMvcBase::~TMvcBase(void)
{
	g_list_free( fWidgetList );
}

//=============================================================================
// find widgets recursive															10.03.2000 Adrian Meyer
//=============================================================================
void TMvcBase::AddChildrenToWidgetList( GtkWidget* aWidget )
{
	AddWidgetList( aWidget );

	guint tmpCount = 0;

	if ( GTK_IS_CONTAINER( aWidget ))										// get count on container
	{	tmpCount = g_list_length(
			gtk_container_children( GTK_CONTAINER( aWidget )));};

	for ( guint i = 0; i<tmpCount; ++i )
	{
		GtkWidget* tmpWidget = (GtkWidget*)g_list_nth(
			gtk_container_children(	GTK_CONTAINER( aWidget )), i )->data;

 	  if ( GTK_IS_NOTEBOOK( tmpWidget ))								// handle notebook tabs
		{
      long j = 0;
			while ( gtk_notebook_get_nth_page( (GtkNotebook*)tmpWidget , j ) != 0 )
			{
        GtkWidget* tmpLabel =
          gtk_notebook_get_tab_label(
						(GtkNotebook*)tmpWidget,
						gtk_notebook_get_nth_page((GtkNotebook*)tmpWidget , j ));

        AddChildrenToWidgetList( tmpLabel );
        j++;
			}
		}

 	  if ( GTK_IS_CONTAINER( tmpWidget ))
 			{	

 				if ( GTK_IS_MENU_ITEM( tmpWidget ))  					// handle menus
 					{
 						AddChildrenToWidgetList( GTK_WIDGET( tmpWidget ));
						if ( GTK_MENU_ITEM( tmpWidget )->submenu != 0 )
							{	AddChildrenToWidgetList( GTK_MENU_ITEM( tmpWidget )->submenu ); }
					}
				else
					{	AddChildrenToWidgetList( GTK_WIDGET( tmpWidget ));}
			}
		else
 			{	AddWidgetList( tmpWidget );	}	
	}
}

//=============================================================================
// add widget to list																	10.03.2000 Adrian Meyer
//=============================================================================
void TMvcBase::AddWidgetList( GtkWidget* aWidget )
{
	if ( strncmp( gtk_widget_get_name( aWidget ), "Gtk", 3 ) != 0 )
	{
//		cout << gtk_widget_get_name( aWidget ) << endl;
		g_list_append( fWidgetList, aWidget );
	}
}

//=============================================================================
// find widget by name																10.03.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcBase::FindWidget( char* aName )
{
	for ( guint i = 0; i<g_list_length( fWidgetList ); i++ )
	{
		if ( g_list_nth( fWidgetList, i )->data != 0 )
		{
			if ( strcmp( gtk_widget_get_name( (GtkWidget*)
				(g_list_nth( fWidgetList, i )->data) ), aName ) == 0 )
				{	return ( (GtkWidget*)(g_list_nth( fWidgetList, i )->data) ); }
		}
	}
	return 0;
}

//=============================================================================
// connect signal																			23.03.2000 Adrian Meyer
//=============================================================================
void TMvcBase::HookSignal( char* aWidget, char* aSignal, GtkSignalFunc aFunc, gpointer data )
{
	gtk_signal_connect( GTK_OBJECT( FindWidget( aWidget )), aSignal, aFunc, data );
}

//=============================================================================
// set update view
// 04.09.2000 Adrian Meyer
//=============================================================================
void TMvcBase::SetUpdateView( bool aUpdateView )
{
	fUpdateView = aUpdateView;
}

//=============================================================================
// read update view
// 04.09.2000 Adrian Meyer
//=============================================================================
bool TMvcBase::GetUpdateView( void )											
{
	return fUpdateView;
}

//=============================================================================
// dump english language file
// 26.02.2001 Adrian Meyer
//=============================================================================
void TMvcBase::DumpLanguageFile( char* aFileName )
{
  fIniFile = LoadIni( aFileName );
	PutIniSetting( fIniFile, "", ";dumped language file from midimountain, Adrian Meyer" );
  PutIniSetting( fIniFile, "", ";dumping over an existing filename will add the missing" );
	PutIniSetting( fIniFile, "", ";items and leave the existing untouched! cool ey? :)" );

	PutIniSetting( fIniFile, "General", "MidiMountainVersion", "0.2.0" );

  if ( GetIniSetting( fIniFile, "General", "Language", "" ) == "" )
		PutIniSetting( fIniFile, "General", "Language", "English" );

	PutIniSetting( fIniFile,
		gtk_widget_get_name( (GtkWidget*)( g_list_nth( fWidgetList, 1 )->data ) ),
		"Comment", ";start of window section-----------------------------------" );

  GtkWidget* tmpWidget = 0;
	for ( guint i = 0; i<g_list_length( fWidgetList ); i++ )
	{
		if ( g_list_nth( fWidgetList, i )->data != 0 )
		{
      tmpWidget = (GtkWidget*)( g_list_nth( fWidgetList, i )->data );
      DumpWidget( tmpWidget );
		}
	}

	SaveIni( fIniFile, aFileName );
}

//=============================================================================
// dump widget values
// 26.02.2001 Adrian Meyer
//=============================================================================
bool TMvcBase::DumpWidget( GtkWidget* aWidget )
{
  if ( gtk_tooltips_data_get( aWidget ))            // tooltip?
	{
	  char tmpStr[255];
	  sprintf( tmpStr, "%s.tooltip", gtk_widget_get_name( aWidget ) );
		WriteWidgetToIni( tmpStr, gtk_tooltips_data_get( aWidget )->tip_text );
  }

  if ( GTK_IS_BIN( aWidget ) )											// button
  {
    if ( GTK_IS_LABEL( GTK_BIN( aWidget )->child ))
      DumpWidgetArgument( gtk_widget_get_name( aWidget ), GTK_BIN( aWidget )->child, "GtkLabel::label" );

    if ( GTK_IS_CONTAINER( GTK_BIN( aWidget )->child ))
    {
      GList* tmpList = gtk_container_children( GTK_CONTAINER( GTK_BIN( aWidget )->child ));

    	for ( guint i = 0; i<g_list_length( tmpList ); i++ )
        if ( GTK_IS_LABEL( g_list_nth_data( tmpList, i ) ))
		      DumpWidgetArgument( gtk_widget_get_name( aWidget ),
						(GtkWidget*)g_list_nth_data( tmpList, i ), "GtkLabel::label" );
    }
  }

  if ( GTK_IS_LABEL( aWidget ) )											// label
    DumpWidgetArgument( gtk_widget_get_name( aWidget ), aWidget, "GtkLabel::label" );

  if ( GTK_IS_WINDOW( aWidget ) )											// window
    DumpWidgetArgument( gtk_widget_get_name( aWidget ), aWidget, "GtkWindow::title" );

	return true;
}

//=============================================================================
// dump the objects argument
// 27.02.2001 Adrian Meyer
//=============================================================================
void TMvcBase::DumpWidgetArgument( char* aName, GtkWidget* aWidget, char* aArgument )
{
  GtkArg args[1];
  args[0].name = aArgument;
  gtk_object_getv( (GtkObject*)aWidget, 1, args);
  g_assert( args[0].type == GTK_TYPE_STRING );

  char tmpStr[255];
  sprintf( tmpStr, "%s.label", aName );

	WriteWidgetToIni( tmpStr, GTK_VALUE_STRING( args[0] ) );
}

//=============================================================================
// write widget to ini
// 27.02.2001 Adrian Meyer
//=============================================================================
void TMvcBase::WriteWidgetToIni( char* aName, char* aValue )
{
  if ( strlen( aName ) == 0 || strlen( aValue ) == 0 ) return;
  if ( index( aValue, '\n' ) != 0 ) return;		// skip newline in value

  if ( GetIniSetting( fIniFile,												// value here?
		gtk_widget_get_name( (GtkWidget*)( g_list_nth( fWidgetList, 1 )->data ) ),
		aName, "" ) == "" )

		PutIniSetting( fIniFile,													// NO! write it
			gtk_widget_get_name( (GtkWidget*)( g_list_nth( fWidgetList, 1 )->data ) ),
			aName, aValue );
}

//=============================================================================
// load language file
// 03.04.2001 Adrian Meyer
//=============================================================================
void TMvcBase::LoadLanguageFile( char* aFileName )
{
  if ( !aFileName ) return;
  if ( strlen( aFileName ) == 0 ) return;

	SetLanguageFileName( aFileName );

  fIniFile = LoadIni( fLanguageFileName );						// load language file

  GtkWidget* tmpWidget = 0;
	for ( guint i = 0; i<g_list_length( fWidgetList ); i++ )
	{
		if ( g_list_nth( fWidgetList, i )->data != 0 )
		{
      tmpWidget = (GtkWidget*)( g_list_nth( fWidgetList, i )->data );
      ReadWidget( tmpWidget );
		}
	}
}

//=============================================================================
// read widget values
// 03.04.2001 Adrian Meyer
//=============================================================================
void TMvcBase::ReadWidget( GtkWidget* aWidget  )
{
	char tmpValue[2000];

  if ( gtk_tooltips_data_get( aWidget ))            // tooltip?
	{
	  char tmpStr[255];
	  sprintf( tmpStr, "%s.tooltip", gtk_widget_get_name( aWidget ) );
		if ( ReadWidgetFromIni( tmpStr, tmpValue ))
			gtk_tooltips_set_tip(
				gtk_tooltips_data_get( aWidget )->tooltips,
				aWidget, tmpValue, NULL );
  }

  if ( GTK_IS_BIN( aWidget ) )											// button
  {
    if ( GTK_IS_LABEL( GTK_BIN( aWidget )->child ))
      ReadWidgetArgument( gtk_widget_get_name( aWidget ), GTK_BIN( aWidget )->child, "GtkLabel::label" );

    if ( GTK_IS_CONTAINER( GTK_BIN( aWidget )->child ))
    {
      GList* tmpList = gtk_container_children( GTK_CONTAINER( GTK_BIN( aWidget )->child ));

    	for ( guint i = 0; i<g_list_length( tmpList ); i++ )
        if ( GTK_IS_LABEL( g_list_nth_data( tmpList, i ) ))
		      ReadWidgetArgument( gtk_widget_get_name( aWidget ),
						(GtkWidget*)g_list_nth_data( tmpList, i ), "GtkLabel::label" );
    }
  }

  if ( GTK_IS_LABEL( aWidget ) )											// label
    ReadWidgetArgument( gtk_widget_get_name( aWidget ), aWidget, "GtkLabel::label" );

  if ( GTK_IS_WINDOW( aWidget ) )											// window
    ReadWidgetArgument( gtk_widget_get_name( aWidget ), aWidget, "GtkWindow::title" );
}

//=============================================================================
// read arguments of widget
// 03.04.2001 Adrian Meyer
//=============================================================================
void TMvcBase::ReadWidgetArgument( char* aName, GtkWidget* aWidget, char* aArgument )
{
  char tmpStr[255];
  sprintf( tmpStr, "%s.label", aName );

  char tmpValue[2000];
  if ( ReadWidgetFromIni( tmpStr, tmpValue ))
    gtk_object_set( GTK_OBJECT( aWidget ), aArgument, &tmpValue, NULL);
}

//=============================================================================
// read widget from ini
// 03.04.2001 Adrian Meyer
//=============================================================================
bool TMvcBase::ReadWidgetFromIni( char* aName, char* tmpValue )
{
  char tmpEntry[2000];
  strcpy( tmpEntry, GetIniSetting( fIniFile,
				gtk_widget_get_name( (GtkWidget*)( g_list_nth( fWidgetList, 1 )->data ) ),
				aName, "" ).c_str() );

  if ( strlen( tmpEntry ) > 0 )
  {
    strcpy( tmpValue, tmpEntry );
		return true;
  }

	return false;																				// nothing found in file
}

//=============================================================================
// read language file name
// 10.04.2001 Adrian Meyer
//=============================================================================
char* TMvcBase::GetLanguageFileName( void )
{
  return fLanguageFileName;
}

//=============================================================================
// set language file name
// 10.04.2001 Adrian Meyer
//=============================================================================
void TMvcBase::SetLanguageFileName( char* aLanguageFileName )
{
	if ( fLanguageFileName ) delete fLanguageFileName;	// delete old name	
  fLanguageFileName = new char[ strlen( aLanguageFileName ) + 1 ];
  strcpy( fLanguageFileName, aLanguageFileName );
}
