//=============================================================================
// model view control event list
//=============================================================================
#include <mvcEventList.h>

// callbacks
void EventListSelectRow( GtkWidget* aWidget,gint row,gint column,GdkEventButton *event,gpointer user_data);
void EventListFilterChange( GtkWidget* aWidget, gpointer user_data );
void EventListDeleteClicked( GtkWidget* aWidget, gpointer user_data);
void EventListAddClicked( GtkWidget* aWidget, gpointer user_data);

void NextClickedEventProp( GtkButton *button, gpointer user_data );
void PrevClickedEventProp( GtkButton *button, gpointer user_data );
void ViewEventProperties( GtkWidget* aWidget, gpointer user_data );

//=============================================================================
// constructor																				28.03.2000 Adrian Meyer
//=============================================================================
TMvcEventList::TMvcEventList( GtkWidget* aWidget, TMidiSequence* aSequence )
{
	fSequence = aSequence;
	fWidget = aWidget;
	AddChildrenToWidgetList( aWidget );                 // fetch widgets
	fSelectedRow = -1;

	HookSignal( "btnEventListProperties", "clicked", GTK_SIGNAL_FUNC( ViewEventProperties ), this );

	HookSignal( "grdEventList", "select-row", GTK_SIGNAL_FUNC( EventListSelectRow ), this );
  HookSignal( "chkEventListNotes", "toggled", GTK_SIGNAL_FUNC( EventListFilterChange ), this );
  HookSignal( "chkEventListControllers", "toggled", GTK_SIGNAL_FUNC( EventListFilterChange ), this );
  HookSignal( "chkEventListOther", "toggled", GTK_SIGNAL_FUNC( EventListFilterChange ), this );

  HookSignal( "btnEventListDelete", "clicked", GTK_SIGNAL_FUNC( EventListDeleteClicked ), this );
  HookSignal( "btnEventListAdd", "clicked", GTK_SIGNAL_FUNC( EventListAddClicked ), this );
}

//=============================================================================
// write selected row																	05.04.2000 Adrian Meyer
//=============================================================================
void TMvcEventList::SetSelectedRow( int aRow )
{
	fSelectedRow = aRow;
}

//=============================================================================
// read widget																				28.03.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcEventList::GetWidget( void )
{
	return fWidget;
}

//=============================================================================
// read first selected event													04.04.2000 Adrian Meyer
//=============================================================================
TMidiEvent* TMvcEventList::GetSelectedEvent( void )
{
	if ( fSelectedRow != -1 )
	{
	  return (TMidiEvent*)gtk_clist_get_row_data( GTK_CLIST( FindWidget( "grdEventList" )),
			fSelectedRow );
	}
	else
	{
		return 0;
	}
}

//=============================================================================
// read next event																		06.04.2000 Adrian Meyer
//=============================================================================
TMidiEvent* TMvcEventList::GetNextEvent( void )									
{
  fEventProperties->GetData( );  	
	SetRowData( fSelectedRow );

	if ( fSelectedRow < fRowCount - 1 )
		{ fSelectedRow++; }																// goto next row

																											// set in list
	gtk_clist_select_row( GTK_CLIST( FindWidget( "grdEventList" )),
		fSelectedRow, 0 );

  fEventProperties->SetEvent( GetSelectedEvent() );
	return GetSelectedEvent();													// get pointer
}

//=============================================================================
// read prevois event																	06.04.2000 Adrian Meyer
//=============================================================================
TMidiEvent* TMvcEventList::GetPrevEvent( void )
{
  fEventProperties->GetData( );  	

	SetRowData( fSelectedRow );

	if ( fSelectedRow >= 1 ) { fSelectedRow--; }				// goto previous row

																											// set list
	gtk_clist_select_row( GTK_CLIST( FindWidget( "grdEventList" )),
		fSelectedRow, 0 );

  fEventProperties->SetEvent( GetSelectedEvent() );
	return GetSelectedEvent();													// get pointer
}

//=============================================================================
// set selected tracks																29.03.2000 Adrian Meyer
//=============================================================================
void TMvcEventList::SetData( void )
{
	long tmpSelectedRow = fSelectedRow;
	GtkCList* tmpCList = GTK_CLIST( FindWidget( "grdEventList" ));

	gtk_clist_freeze( tmpCList);
	gtk_clist_clear( tmpCList);
	char* tmpList[1][6] = {{"","","","","",""}};				// empty row
	char tmpStr[2000];
	bool tmpDisplay;
	fRowCount = 0;

	GList* tmpEventList = g_list_alloc();

	for ( long i = 1; i<fSequence->GetTrackCount(); i++ )
    if ( fSequence->GetTrack(i)->GetEventCount() > 0 )
			{ g_list_append( tmpEventList, fSequence->GetTrack(i)->GetEvent(0) ); }
		else
			{ g_list_append( tmpEventList, 0 ); }

	long tmpMinValue = 0;
	int tmpDiv = 0;
	TMidiEvent* tmpEvent = 0;
	TMidiEvent* hitEvent = 0;
	int hitPos = 1;
  bool loop = fSequence->GetTrackCount()>1;	

	while ( loop )
	{
		tmpDiv = -1;
                                                      // loop viewable tracks
		for ( int i = 1; i<fSequence->GetTrackCount(); i++ )
		{
																											// take event of track
			tmpEvent = (TMidiEvent*)(g_list_nth( tmpEventList, i )->data );

			if ( tmpEvent != 0 )
			{
  			if ( tmpDiv == -1 )                           // first checked event?
  				{
						tmpDiv = tmpEvent->GetOffset() - tmpMinValue;
						hitEvent = tmpEvent;
						hitPos = i;																// changed 1 to i !!!!!!!
					}
  			else                                          // find smallest difference between events
  				{
						if ( (tmpEvent->GetOffset() - tmpMinValue) < tmpDiv )
						{
							tmpDiv = tmpEvent->GetOffset() - tmpMinValue;
							hitEvent = tmpEvent;
							hitPos = i;
						}
  				}
			}
		}

    if ( hitEvent != 0 )
		{
   		
  		tmpMinValue = hitEvent->GetOffset();              // remember position
  																											// switch to next event links to worked
  		g_list_nth( tmpEventList, hitPos )->data = hitEvent->Next;

   		tmpDisplay = true;															// display event

   		if ( !fSequence->GetTrack(hitPos)->GetSelected() )
   			{ tmpDisplay = false;	}												// don't diplay when unselected
   		if ( hitEvent->GetNoteOff() )										// don't display when note off
   			{ tmpDisplay = false;	}
   																										// note filter off?
   		if ((((hitEvent->GetData1())&0xF0 ) == cESNoteOn ) &&
   				( !gtk_toggle_button_get_active( (GtkToggleButton*)FindWidget( "chkEventListNotes" ))))
   			{ tmpDisplay = false;	}
                                                      // controller filter off?
   		if ((((hitEvent->GetData1())&0xF0 ) == cESControlChange ) &&
   				( !gtk_toggle_button_get_active( (GtkToggleButton*)FindWidget( "chkEventListControllers" ))))
   			{ tmpDisplay = false;	}

   		if ((((hitEvent->GetData1())&0xF0 ) != cESControlChange ) &&
   				(((hitEvent->GetData1())&0xF0 ) != cESNoteOn ) &&
   				( !gtk_toggle_button_get_active( (GtkToggleButton*)FindWidget( "chkEventListOther" ))))
   			{ tmpDisplay = false;	}


   		if ( tmpDisplay )																// display event ?
   		{
     		gtk_clist_append( tmpCList, tmpList[0] );
   			gtk_clist_set_row_data( tmpCList,
   				fRowCount, hitEvent );

         sprintf( tmpStr, "%d", hitPos );
         gtk_clist_set_text(	tmpCList,
         	fRowCount, 2, tmpStr );

   			SetRowData( fRowCount );											// set changable data

   			fRowCount++;
   		}
		}

		loop = false;

		for ( int i = 1; i<fSequence->GetTrackCount(); i++ )
		{
			tmpEvent = (TMidiEvent*)(g_list_nth( tmpEventList, i )->data );
			if ( tmpEvent != 0 ) { loop = true; }
		}
	}	
                                                      // reselect row
	if ( tmpSelectedRow >= fRowCount ) { tmpSelectedRow = fRowCount - 1; }
	gtk_clist_select_row( tmpCList,	tmpSelectedRow, 0 );                          		

	gtk_clist_thaw( tmpCList );

	g_list_free( tmpEventList );

	fEventFlag = true;																	// react to changes

	ValidateGui();
}

//=============================================================================
// set data of a row																	09.04.2000 Adrian Meyer
//=============================================================================
void TMvcEventList::SetRowData( int aRow )
{
	if ( aRow < 0 ) {return;}

	GtkCList* tmpList = GTK_CLIST( FindWidget( "grdEventList" ));
	char tmpStr[100];
	char tmpStr1[100];
	char tmpStr2[100];

	TMidiEvent* hitEvent = (TMidiEvent*)gtk_clist_get_row_data( tmpList, aRow );

  gtk_clist_set_text(	tmpList, aRow,
		1, fSequence->GetMeasureBeatTick( hitEvent->GetOffset() ) );

 	sprintf( tmpStr, "%d", hitEvent->GetChannel() );
 	gtk_clist_set_text(	tmpList, aRow, 3, tmpStr );

  switch ((hitEvent->GetData1())&0xF0 )
  {
  	case cESNoteOn:
  	if (!hitEvent->GetNoteOff())
  	{
  		sprintf( tmpStr1, "Note %s", hitEvent->GetNoteName() );
  		long tmpDuration = 0;
  		if ( hitEvent->Ref != 0 )	
  			{ tmpDuration = hitEvent->Ref->GetOffset() - hitEvent->GetOffset(); }

  		sprintf( tmpStr2, "Velocity:  %3d  Duration:  %d:%.4d",
  			hitEvent->GetData3(),
  			(int)(tmpDuration / (fSequence->GetDeltaTimesType())),
  			(int)(tmpDuration % (fSequence->GetDeltaTimesType())) );

  	} break;

    case cESControlChange:
  	{
  		strcpy( tmpStr1, "Controller" );
  		sprintf( tmpStr2, "%s Value: %3d", cControllerName[ hitEvent->GetData2() ], hitEvent->GetData3());
  	} break;

    case cESProgramChange:
  	{
  		strcpy( tmpStr1, "Program Change"  );
  		sprintf( tmpStr2, "[%.3d]  %s", hitEvent->GetData2() ,cGMPatchName[ hitEvent->GetData2() ] );
  	} break;

    default:
  	{
  		sprintf( tmpStr1, "Event %.2x hex", hitEvent->GetData1() );
  		sprintf( tmpStr2, "Data1: %3d  Data2: %3d",
  							hitEvent->GetData2(), hitEvent->GetData3());
  	}
  }
 	
 	gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdEventList" )),
 		aRow, 4, tmpStr1 );
  gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdEventList" )),
 		aRow, 5, tmpStr2 );	
}

//=============================================================================
// validate gui
// 03.09.2000 Adrian Meyer
//=============================================================================
void TMvcEventList::ValidateGui( void )
{
	if ( fEventFlag )
	{
    gtk_widget_set_sensitive( FindWidget( "btnEventListProperties" ), ( fRowCount > 0 ) );
    gtk_widget_set_sensitive( FindWidget( "btnEventListDelete" ), ( fRowCount > 0 ) );

		gtk_widget_set_sensitive( FindWidget( "btnEventListAdd" ), fSequence->GetTrackCount() > 1 );
		gtk_widget_set_sensitive( FindWidget( "chkEventListNotes" ), fSequence->GetTrackCount() > 1 );
		gtk_widget_set_sensitive( FindWidget( "chkEventListControllers" ), fSequence->GetTrackCount() > 1 );
		gtk_widget_set_sensitive( FindWidget( "chkEventListOther" ), fSequence->GetTrackCount() > 1 );

		fEventFlag = false;
	}
}

//=============================================================================
// message from midi classes
// 01.09.2000 Adrian Meyer
//=============================================================================
void TMvcEventList::MCMessage( TMidiBase* aMidiBase, sMCMessage* aMCMessage )
{
  switch ( aMCMessage->Message )
 	{
 		case cMMTrackSelect:															// track selection changed
 		case cMMSequenceDeleteTrack:											// track deleted
 		case cMMEventDestroy:															// event deleted
 		case cMMEventSetOffset:															// event deleted
 				fEventFlag = true;														// react to changes
				if ( GetUpdateView() ) { SetData(); }
 			break;

 		case cMMSequenceOpenFile:
 		case cMMSequenceNew:															
 				fEventFlag = true;														// react to changes
 				SetData();
 			break;
 	}

	if ( aMCMessage->Class == cMCEvent )								// event changed
	{
		if ( fSelectedRow < fRowCount && fSelectedRow > -1 )
			{	SetRowData( fSelectedRow ); }
	}

	ValidateGui();
}

//=============================================================================
// delete selected event
// 04.09.2000 Adrian Meyer
//=============================================================================
void TMvcEventList::DeleteEvent( void )
{
	TMidiEvent* tmpMidiEvent = GetSelectedEvent();			// event to delete
	tmpMidiEvent->GetOwner()->DeleteEvent( tmpMidiEvent );
}

//=============================================================================
// add new event
// 04.09.2000 Adrian Meyer
//=============================================================================
void TMvcEventList::AddEvent( void )
{
	long tmpOffset = 0;
	TMidiEvent* tmpEvent = 0;
	TMidiEvent* tmpSelectedEvent = GetSelectedEvent();
	TMidiTrack* tmpTrack = fSequence->GetTrack( 1 );		// take first track

	for ( int i = 1; i<fSequence->GetTrackCount(); i++ )// take selected track
	{
		if ( fSequence->GetTrack(i)->GetSelected() )
		{
			tmpTrack = fSequence->GetTrack( i );
		}
	}

	if ( tmpSelectedEvent != 0 )											// find offset
	{	
		tmpOffset = tmpSelectedEvent->GetOffset();
	}

	tmpEvent = tmpTrack->AddEvent( tmpOffset );					// create event;
	
	if ( tmpSelectedEvent != 0 )											// copy event
	{	
		tmpEvent->SetData1( tmpSelectedEvent->GetData1());
		tmpEvent->SetData2( tmpSelectedEvent->GetData2());
		tmpEvent->SetData3( tmpSelectedEvent->GetData3());
	}
	else
	{
		tmpEvent->SetData1( cESNoteOn );
		tmpEvent->SetChannel( tmpTrack->GetChannel());
	}
	tmpEvent->SetNoteOff( false );
		
	SetData();

	gtk_clist_select_row( GTK_CLIST( FindWidget( "grdEventList" )),
		fSelectedRow + 1, 0 );

	fEventProperties = new TMvcEventProperties( tmpEvent, true );
	fEventProperties->LoadLanguageFile( GetLanguageFileName() );
}

//=============================================================================
// show event properties
// 04.09.2000 Adrian Meyer
//=============================================================================
void TMvcEventList::ShowEventProperties( void )
{
	fEventProperties = new TMvcEventProperties( GetSelectedEvent(), false );
	fEventProperties->LoadLanguageFile( GetLanguageFileName() );
  fEventProperties->HookSignal( "btnNext", "clicked", GTK_SIGNAL_FUNC( NextClickedEventProp ), this );
  fEventProperties->HookSignal( "btnPrevious", "clicked", GTK_SIGNAL_FUNC( PrevClickedEventProp ), this );
}

//=============================================================================
// row is selected																		05.04.2000 Adrian Meyer
//=============================================================================
void EventListSelectRow( GtkWidget* aWidget,gint row,gint column,GdkEventButton *event,gpointer user_data)
{
	((TMvcEventList*)user_data)->SetSelectedRow( row );
}

//=============================================================================
// toggle event list filter														30.03.2000 Adrian Meyer
//=============================================================================
void EventListFilterChange( GtkWidget* aWidget, gpointer user_data )
{
  TMvcEventList* tmpEventList = ((TMvcEventList*)user_data );
	tmpEventList->SetData();	
}

//=============================================================================
// delete event
// 03.09.2000 Adrian Meyer
//=============================================================================
void EventListDeleteClicked( GtkWidget* aWidget, gpointer user_data)
{
  TMvcEventList* tmpEventList = ((TMvcEventList*)user_data );
	tmpEventList->DeleteEvent();	
}

//=============================================================================
// event property next clicked												
// 04.09.2000 Adrian Meyer
//=============================================================================
void NextClickedEventProp( GtkButton *button, gpointer user_data )
{
  TMvcEventList* tmpEventList = ((TMvcEventList*)user_data );
	tmpEventList->GetNextEvent();
}

//=============================================================================
// event property prev clicked												
// 04.09.2000 Adrian Meyer
//=============================================================================
void PrevClickedEventProp( GtkButton *button, gpointer user_data )
{
  TMvcEventList* tmpEventList = ((TMvcEventList*)user_data );
	tmpEventList->GetPrevEvent();
}

//=============================================================================
// view event properties
// 04.09.2000 Adrian Meyer
//=============================================================================
void ViewEventProperties( GtkWidget* aWidget, gpointer user_data )
{
  TMvcEventList* tmpEventList = ((TMvcEventList*)user_data );
	tmpEventList->ShowEventProperties();	
}

//=============================================================================
// add event
// 05.09.2000 Adrian Meyer
//=============================================================================
void EventListAddClicked( GtkWidget* aWidget, gpointer user_data)
{
  TMvcEventList* tmpEventList = ((TMvcEventList*)user_data );
	tmpEventList->AddEvent();	
}