//=============================================================================
//  model view control event properties
//=============================================================================
#include <mvcEventProperties.h>
#include <iostream.h>
#include <string.h>
#include <stdio.h>

// callbacks
void OkClickedEventProp( GtkButton *button, gpointer user_data );
void CancelClickedEventProp( GtkButton *button, gpointer user_data );
void NoteButtonClickedEventProp( GtkButton *button, gpointer user_data );
void ViewChangedEvent( GtkWidget* aWidget, GtkNotebookPage *page,
										gint page_num, gpointer user_data );

void MeasureChanged( GtkEditable *editable, gpointer user_data );

//=============================================================================
// construtor MvcEventProperties											04.04.2000 Adrian Meyer
//=============================================================================
TMvcEventProperties::TMvcEventProperties( TMidiEvent* aEvent, bool aNewEvent )
{
	fNewEvent = aNewEvent;
  wndEventProperties = create_wndEventProperties();		// call glade

	AddChildrenToWidgetList( wndEventProperties );      // fetch widgets
	fEvent = aEvent;
	
  HookSignal( "btnOk", "clicked", GTK_SIGNAL_FUNC( OkClickedEventProp ), this );
  HookSignal( "btnCancel", "clicked", GTK_SIGNAL_FUNC( CancelClickedEventProp ), this );
  HookSignal( "nbkEventProperties", "switch-page", GTK_SIGNAL_FUNC( ViewChangedEvent ), this );
  HookSignal( "spiMeasure", "changed", GTK_SIGNAL_FUNC( MeasureChanged ), this );

	if ( fNewEvent )
	{
		gtk_widget_set_sensitive( FindWidget( "btnCancel" ), false );
		gtk_widget_set_sensitive( FindWidget( "btnPrevious" ), false );
		gtk_widget_set_sensitive( FindWidget( "btnNext" ), false );
	}

  HookSignal( "btn1_1", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedEventProp ), this );
  HookSignal( "btn1_2", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedEventProp ), this );
  HookSignal( "btn1_4", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedEventProp ), this );
  HookSignal( "btn1_8", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedEventProp ), this );
  HookSignal( "btn1_16", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedEventProp ), this );
  HookSignal( "btn1_32", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedEventProp ), this );

	gtk_range_set_adjustment( (GtkRange*)FindWidget( "sclVelocity" ),
		gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiVelocity" ))));

	gtk_range_set_adjustment( (GtkRange*)FindWidget( "sclControllerValue" ),
		gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiControllerValue" ))));

	char tmpStr[100];

	fNoteNames = 0;
	for ( int i = 0; i<128; i++ )											// add only numbers
	{
		sprintf( tmpStr, "[%.3d]  %-2s%d", i, cMajorScale[ i % 12 ], ( i / 12 ));
		char* tmpItem = new char[ strlen(tmpStr) + 1 ];
		strcpy( tmpItem, tmpStr );
		fNoteNames = g_list_append( fNoteNames, tmpItem );
	}

	gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboPitch" )), fNoteNames );	

	fControllerNames = 0;
	for ( int i = 0; i<128; i++ )											// add only numbers
	{
		sprintf( tmpStr, "[%.3d]  %s", i, cControllerName[i] );
		char* tmpItem = new char[ strlen(tmpStr) +1 ];
		tmpItem[0] = 0;
		strcpy( tmpItem, tmpStr );
		fControllerNames = g_list_append( fControllerNames, tmpItem );
	}

	gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboController" )), fControllerNames );	

	fGMPatchNames = 0;
	for ( int i = 0; i<128; i++ )				
	{
		sprintf( tmpStr, "[%.3d]  %s", i, cGMPatchName[i] );
		char* tmpItem = new char[ strlen(tmpStr) +1 ];
		tmpItem[0] = 0;
		strcpy( tmpItem, tmpStr );
		fGMPatchNames = g_list_append( fGMPatchNames, tmpItem );
	}

	gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboPatch" )), fGMPatchNames );	

	if ( aEvent == 0 ) return;

  gtk_widget_show( wndEventProperties );								

	SetData();																					// set data to gui
}

//=============================================================================
// destrutor MvcEventProperties												04.04.2000 Adrian Meyer
//=============================================================================
TMvcEventProperties::~TMvcEventProperties( void )
{
	delete wndEventProperties;
}

//=============================================================================
// read widget																				04.04.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcEventProperties::GetWidget( void )
{
	return wndEventProperties;
}

//=============================================================================
// set data to gui                                    04.04.2000 Adrian Meyer
//=============================================================================
void TMvcEventProperties::SetData( void )
{
	char* tmpStr = new char[1000];
	tmpStr[0] = 0;
	GtkNotebook* nbk = (GtkNotebook*)FindWidget( "nbkEventProperties" );

  if ( fEvent != 0 )																	// edit event
	{
                                                      // set track number
		sprintf( tmpStr, "%d", fEvent->GetOwner()->GetIndex() );
		gtk_label_set_text( (GtkLabel*)FindWidget( "lblTrackNumber" ), tmpStr );

																											// set channel
	  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiChannel" )),
			fEvent->GetChannel() );

		long tmpMeasure = 0;															// set position of event
		long tmpBeat = 0;
		long tmpTick = 0;		
		fEvent->GetOwner()->GetOwner()->GetMeasureBeatTick(
			fEvent->GetOffset(), &tmpMeasure, &tmpBeat, &tmpTick );

																											// upper value ticks
		gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiTick" )))->
			upper = fEvent->GetOwner()->GetOwner()->GetDeltaTimesType() - 1;

																											// set measure
	  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiMeasure" )), tmpMeasure );
	  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiBeat" )), tmpBeat );
	  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiTick" )), tmpTick );

											// set resolution
		sprintf( tmpStr, "%d ticks per 1/4 note", fEvent->GetOwner()->GetOwner()->GetDeltaTimesType() );
		gtk_label_set_text( (GtkLabel*)FindWidget( "lblResolution" ), tmpStr );

																											// upper value for notes
		gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiDurationTicks" )))->
			upper = fEvent->GetOwner()->GetOwner()->GetDeltaTimesType() - 1;

																											// set increment for notes
		gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiDurationTicks" )))->
			page_increment = fEvent->GetOwner()->GetOwner()->GetDeltaTimesType() / 32;

		if ( !fNewEvent )                                 // disable pages for old events
		{																											
  	  gtk_widget_set_sensitive(	gtk_notebook_get_nth_page( nbk,	0 ), ((fEvent->GetData1())&0xF0) == cESNoteOn );
  	  gtk_widget_set_sensitive(	gtk_notebook_get_tab_label( nbk,
  			gtk_notebook_get_nth_page( nbk,	0 )), ((fEvent->GetData1())&0xF0) == cESNoteOn );

  	  gtk_widget_set_sensitive(	gtk_notebook_get_nth_page( nbk,	1 ), ((fEvent->GetData1())&0xF0) == cESControlChange );
  	  gtk_widget_set_sensitive(	gtk_notebook_get_tab_label( nbk,
  			gtk_notebook_get_nth_page( nbk,	1 )), ((fEvent->GetData1())&0xF0) == cESControlChange );

  	  gtk_widget_set_sensitive(	gtk_notebook_get_nth_page( nbk,	2 ), ((fEvent->GetData1())&0xF0) == cESProgramChange );
  	  gtk_widget_set_sensitive(	gtk_notebook_get_tab_label( nbk,
  			gtk_notebook_get_nth_page( nbk,	2 )), ((fEvent->GetData1())&0xF0) == cESProgramChange );

  	  gtk_widget_set_sensitive(	gtk_notebook_get_nth_page( nbk,	3 ), ((fEvent->GetData1())&0xF0) == cESPitchBend );
  	  gtk_widget_set_sensitive(	gtk_notebook_get_tab_label( nbk,
  			gtk_notebook_get_nth_page( nbk,	3 )), ((fEvent->GetData1())&0xF0) == cESPitchBend );

  	  gtk_widget_set_sensitive(	gtk_notebook_get_nth_page( nbk,	4 ),
  			(( fEvent->GetData1())&0xF0) == cESPolyphonicAfterTouch || ((fEvent->GetData1())&0xF0) == cESChannelAfterTouch );
  	  gtk_widget_set_sensitive(	gtk_notebook_get_tab_label( nbk,
  			gtk_notebook_get_nth_page( nbk,	4 )),
  				(( fEvent->GetData1())&0xF0) == cESPolyphonicAfterTouch || ((fEvent->GetData1())&0xF0) == cESChannelAfterTouch );
		}

		switch ((fEvent->GetData1())&0xF0 )
		{
			case cESNoteOn:																	// note
				{
					gtk_notebook_set_page( nbk, 0 );
                                                      // pitch
					sprintf( tmpStr, "[%.3d]  %s", fEvent->GetData2(), fEvent->GetNoteName() );
					gtk_entry_set_text( (GtkEntry*)FindWidget( "edtPitch" ), tmpStr );

                                                     	// velocity
					gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiVelocity" )),
						fEvent->GetData3());

 					long tmpDuration = 0;												// duration
 					if ( fEvent->Ref != 0 )	{ tmpDuration = fEvent->Ref->GetOffset() - fEvent->GetOffset(); }

				  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiDurationBeats" )),
							tmpDuration / (fEvent->GetOwner()->GetOwner()->GetDeltaTimesType()) );
				  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiDurationTicks" )),
							tmpDuration % (fEvent->GetOwner()->GetOwner()->GetDeltaTimesType()) );									
				} break;
      case cESControlChange:													// controller
				{
					gtk_notebook_set_page( nbk, 1 );

					sprintf( tmpStr, "[%.3d]  %s", fEvent->GetData2(), cControllerName[fEvent->GetData2()] );
					gtk_entry_set_text( (GtkEntry*)FindWidget( "edtController" ), tmpStr );

				  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiControllerValue" )),
						fEvent->GetData3());									
				
				} break;
      case cESProgramChange:													// program change
				{
					gtk_notebook_set_page( nbk, 2 );

					sprintf( tmpStr, "[%.3d]  %s", fEvent->GetData2(), cGMPatchName[fEvent->GetData2()] );
					gtk_entry_set_text( (GtkEntry*)FindWidget( "edtPatch" ), tmpStr );
				
				} break;
      case cESPitchBend:															// pitch bend
				{
					gtk_notebook_set_page( nbk, 3 );
				
				} break;
		}
	}
}

//=============================================================================
// read data from gui                                 04.04.2000 Adrian Meyer
//=============================================================================
void TMvcEventProperties::GetData( void )
{
	GtkNotebook* nbk = (GtkNotebook*)FindWidget( "nbkEventProperties" );
																											
	fEvent->SetChannel( gtk_spin_button_get_value_as_int(      // read channel
		GTK_SPIN_BUTTON( FindWidget( "spiChannel" ))));

  long tmpOffset =																		// new offset
		fEvent->GetOwner()->GetOwner()->GetOffset(
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiMeasure" ))),
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiBeat" ))),
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiTick" )))
    );

	if ( fEvent->GetOffset() != tmpOffset )     				// set new offset
	{
		fEvent->GetOwner()->SetEventOffset( fEvent, tmpOffset );
	}

	switch ((fEvent->GetData1())&0xF0 )
	{
		case cESNoteOn:																		// note
			{
				                                            	// read pitch
				for ( unsigned int i = 0; i < g_list_length( fNoteNames ); i++ )
				{
					if ( strcmp( (char*)g_list_nth_data( fNoteNames, i ),
						gtk_entry_get_text((GtkEntry*)FindWidget( "edtPitch" ))) == 0 )
					{
						fEvent->SetData2( i );
					}					
				}
                                                   		// read velocity
				fEvent->SetData3( gtk_spin_button_get_value_as_int(
					GTK_SPIN_BUTTON( FindWidget( "spiVelocity" ))));

				long tmpRefOffset =	fEvent->GetOffset() +			
							gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiDurationBeats" )))
							* fEvent->GetOwner()->GetOwner()->GetDeltaTimesType()
							+	gtk_spin_button_get_value_as_int(
								GTK_SPIN_BUTTON( FindWidget( "spiDurationTicks" )));

				if ( fEvent->Ref == 0 )
				{
					TMidiEvent* tmpRef = fEvent->GetOwner()->AddEvent( tmpRefOffset );

					tmpRef->SetData1( fEvent->GetData1() );
					tmpRef->SetData2( fEvent->GetData2() );
					tmpRef->SetData3( fEvent->GetData3() );
					tmpRef->SetNoteOff( true );
					fEvent->Ref = tmpRef;
				}
				else
				{
					fEvent->GetOwner()->SetEventOffset( fEvent->Ref, tmpRefOffset );
				}

			} break;
    case cESControlChange:														// controller
			{
				gtk_notebook_set_page( nbk, 1 );
				
				fEvent->SetData3( gtk_spin_button_get_value_as_int(
					GTK_SPIN_BUTTON( FindWidget( "spiControllerValue" ))));

				for ( unsigned int i = 0; i < g_list_length( fControllerNames ); i++ )
				{
					if ( strcmp( (char*)g_list_nth_data( fControllerNames, i ),
						gtk_entry_get_text((GtkEntry*)FindWidget( "edtController" ))) == 0 )
					{
						fEvent->SetData2( i );
					}					
				}

			} break;
    case cESProgramChange:														// program change
			{
				gtk_notebook_set_page( nbk, 2 );

				for ( unsigned int i = 0; i < g_list_length( fGMPatchNames ); i++ )
				{
					if ( strcmp( (char*)g_list_nth_data( fGMPatchNames, i ),
						gtk_entry_get_text((GtkEntry*)FindWidget( "edtPatch" ))) == 0 )
					{
						fEvent->SetData2( i );
					}					
				}
				
			} break;
    case cESPitchBend:																// pitch bend
			{
				gtk_notebook_set_page( nbk, 3 );
				
			} break;
	}
}

//=============================================================================
// set other midi event																06.04.2000 Adrian Meyer
//=============================================================================
void TMvcEventProperties::SetEvent( TMidiEvent* aEvent )
{
	GetData();
	fEvent = aEvent;
	SetData();
}

//=============================================================================
// set the note value with buttons										06.02.2000 Adrian Meyer
//=============================================================================
void TMvcEventProperties::SetNoteValue( char* aButtonName )
{
	long tmpDuration = fEvent->GetOwner()->GetOwner()->GetDeltaTimesType();

	if ( strcmp( aButtonName, "btn1_1" ) == 0 ) { tmpDuration*=4; }
	if ( strcmp( aButtonName, "btn1_2" ) == 0 ) { tmpDuration*=2; }
	if ( strcmp( aButtonName, "btn1_8" ) == 0 ) { tmpDuration/=2; }
	if ( strcmp( aButtonName, "btn1_16" ) == 0 ) { tmpDuration/=4; }
	if ( strcmp( aButtonName, "btn1_32" ) == 0 ) { tmpDuration/=8; }

	if ( gtk_toggle_button_get_active((GtkToggleButton*)FindWidget( "chkDurationDotted" )))
		{	tmpDuration = (long)(tmpDuration*1.5); }

	if ( gtk_toggle_button_get_active((GtkToggleButton*)FindWidget( "chkDurationTriplet" )))
		{	tmpDuration = (tmpDuration*2)/3; }

  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiDurationBeats" )),
			tmpDuration / (fEvent->GetOwner()->GetOwner()->GetDeltaTimesType()) );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiDurationTicks" )),
			tmpDuration % (fEvent->GetOwner()->GetOwner()->GetDeltaTimesType()) );										
}

//=============================================================================
// set type of event
// 05.09.2000 Adrian Meyer
//=============================================================================
void TMvcEventProperties::SetEventType( short aEventType )
{
	if ( fNewEvent )																		// type change only on new event
		{ fEvent->SetData1( aEventType ); }
}

//=============================================================================
// et the note value with buttons										06.02.2000 Adrian Meyer
//=============================================================================
void NoteButtonClickedEventProp( GtkButton *button, gpointer user_data )
{
	((TMvcEventProperties*)user_data)->SetNoteValue(
		gtk_widget_get_name( GTK_WIDGET( button )));
}

//=============================================================================
// ok clicked																					05.04.2000 Adrian Meyer
//=============================================================================
void OkClickedEventProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcEventProperties*)user_data)->GetWidget() );
	((TMvcEventProperties*)user_data)->GetData();
}

//=============================================================================
// cancel clicked																			05.04.2000 Adrian Meyer
//=============================================================================
void CancelClickedEventProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcEventProperties*)user_data)->GetWidget() );
}

//=============================================================================
// view changed
// 04.09.2000 Adrian Meyer
//=============================================================================
void ViewChangedEvent( GtkWidget* aWidget, GtkNotebookPage *page,
										gint page_num, gpointer user_data )
{
	TMvcEventProperties* tmpMvcEventProperties = (TMvcEventProperties*)user_data;

	short tmpType = 0;

	switch ( page_num )
	{
		case 0: tmpType = cESNoteOn; break;
		case 1: tmpType = cESControlChange; break;
		case 2: tmpType = cESProgramChange; break;
		case 3: tmpType = cESPitchBend; break;
		case 4: tmpType = cESPolyphonicAfterTouch; break;
	}

	tmpMvcEventProperties->SetEventType( tmpType );	
}

//=============================================================================
// measure changed
// 05.09.2000 Adrian Meyer
//=============================================================================
void MeasureChanged( GtkEditable *editable, gpointer user_data )
{
	TMvcEventProperties* tmpMvcEventProperties = (TMvcEventProperties*)user_data;
	tmpMvcEventProperties->AdjustToMeasure( gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON( editable )));		
}

//=============================================================================
// adjust beat/ticks to selected measure
// 05.09.2000 Adrian Meyer
//=============================================================================
void TMvcEventProperties::AdjustToMeasure( long aMeasure )
{
	long tmpOffset = fEvent->GetOwner()->GetOwner()->GetOffset( aMeasure + 1, 0, 0 ) - 1;
	long tmpMeasure = 0;
	long tmpBeat = 0;
	long tmpTick = 0;

	fEvent->GetOwner()->GetOwner()->GetMeasureBeatTick( tmpOffset, &tmpMeasure, &tmpBeat, &tmpTick );

	gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiBeat" )))->upper = tmpBeat + 1;
	gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiTick" )))->upper = tmpTick;
}