//=============================================================================
// model view control main view
//=============================================================================
#include <mvcHeader.h>

// callbacks
bool HeaderExposeHook( GtkWidget *widget, GdkEventExpose *event, gpointer user_data );

bool HeaderMouseDown( GtkWidget *widget, GdkEventButton *event, gpointer user_data);
bool HeaderMouseUp( GtkWidget *widget, GdkEventButton *event, gpointer user_data);
bool HeaderMouseMove( GtkWidget *widget, GdkEventMotion *event, gpointer user_data);

//=============================================================================
// constructor																				11.04.2000 Adrian Meyer
//=============================================================================
TMvcHeader::TMvcHeader( GtkWidget* aWidget, TMidiSequence* aSequence, GtkAdjustment* aAdjustment )
{
	fWidget = aWidget;
	fSequence = aSequence;															// take sequence
	fAdjustment = aAdjustment;

	fWidth = 0;
	fHeight = 26;
	fTicksPerPix = 1;
	fWindowWidth = 0;

	fBuffer = 0;
	fPositionActive = false;
	fLastPixPosition = 0;

	gtk_signal_connect( GTK_OBJECT( fWidget ), "expose_event", GTK_SIGNAL_FUNC( HeaderExposeHook ), this );
	gtk_signal_connect( GTK_OBJECT( fWidget ), "button_press_event", GTK_SIGNAL_FUNC( HeaderMouseDown ), this );
	gtk_signal_connect( GTK_OBJECT( fWidget ), "button_release_event",GTK_SIGNAL_FUNC( HeaderMouseUp ), this );
	gtk_signal_connect( GTK_OBJECT( fWidget ), "motion_notify_event", GTK_SIGNAL_FUNC( HeaderMouseMove ), this );	

	gtk_widget_add_events(                              // activate events
		fWidget, GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK );
	
	colGrid = new GdkColor;
	colGrid->red = 10000;	colGrid->green = 10000;	colGrid->blue = 10000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( fWidget ),	
		colGrid, true, true );

	colBackgroundHeader = new GdkColor;
	colBackgroundHeader->red = 55000;	colBackgroundHeader->green = 55000;	colBackgroundHeader->blue = 55000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( fWidget ),	
		colBackgroundHeader, true, true );

	colPosition = new GdkColor;
	colPosition->red = 32000;	colPosition->green = 0;	colPosition->blue = 0;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( fWidget ),	
		colPosition, true, true );

}

//=============================================================================
// message from midi classes
// 13.09.2000 Adrian Meyer
//=============================================================================
void TMvcHeader::MCMessage( TMidiBase* aMidiBase, sMCMessage* aMCMessage )
{
	switch ( aMCMessage->Message )
	{
		case cMMSequenceSetPosition:
				gtk_widget_queue_draw( fWidget );
			break;
	}
}

//=============================================================================
// read widget																				
// 13.04.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcHeader::GetWidget( void )
{
	return fWidget;
}

//=============================================================================
// set new with and height
// 13.09.2000 Adrian Meyer
//=============================================================================
void TMvcHeader::Resize( long aWidth, float aTicksPerPix, long aWindowWidth, long aMeasureCount )
{
	fWidth = aWidth;
	fTicksPerPix = aTicksPerPix;
	fWindowWidth = aWindowWidth;
	fMeasureCount = aMeasureCount;

	gtk_drawing_area_size( GTK_DRAWING_AREA( fWidget ), fWidth, fHeight );

	if ( fBuffer != 0 )	{ gdk_pixmap_unref( fBuffer ); }
	fBuffer = gdk_pixmap_new( fWidget->window, fWidth, fHeight, -1 );;
}

//=============================================================================
// redraw area
// 05.09.2000 Adrian Meyer
//=============================================================================
bool TMvcHeader::Expose( GtkWidget* aWidget, GdkEventExpose* aEvent )
{
	if ( fBuffer == 0 )																	// no window yet
		{ return false; }

	GdkGC* tmpGC = gdk_gc_new( aWidget->window );				// new graphic context
	gdk_gc_copy( tmpGC, aWidget->style->fg_gc[aWidget->state] );

	GdkDrawable* tmpWnd = aWidget->window;							// window to draw to
	long tmpHeight = aWidget->allocation.height - 1;		// height

	gdk_gc_set_foreground( tmpGC, colBackgroundHeader );// draw background
	gdk_draw_rectangle( fBuffer, tmpGC, TRUE,	0, 0, fWidth, tmpHeight );

	gdk_gc_set_foreground( tmpGC, colGrid );						// draw grid
	gdk_gc_set_line_attributes( tmpGC, 0, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER );
	GdkFont* tmpFont = gdk_font_load( "-adobe-courier-medium-r-normal--11-80-100-100-m-60-iso8859-1" );

	for ( long i=1; i<=fMeasureCount; i++ )
	{
		long tmpBeatCount; long tmpTicksPerBeat;
		long x = (long)(fSequence->GetOffset( i, 1, 0 ) / fTicksPerPix);

		fSequence->GetTimeInformation( i, &tmpBeatCount, &tmpTicksPerBeat );

		for ( long j=0; j<tmpBeatCount; j++ )
		{
			long tmpx = (long)(x + (( j*tmpTicksPerBeat ) / fTicksPerPix ));
			gdk_draw_line( fBuffer, tmpGC, tmpx, 23, tmpx, tmpHeight );
		}
																											// draw measure line
		gdk_draw_line( fBuffer, tmpGC, x, 20, x, tmpHeight );

		char* tmpString = new char[100];									// draw measure number
		sprintf( tmpString, "%d", (int)i );
		gdk_draw_string( fBuffer, tmpFont, tmpGC, x - 1, 26 - gdk_string_height( tmpFont, tmpString ),	tmpString );
		delete tmpString;
	}
                                                      // draw marker
	for ( long i = 0; i<fSequence->GetMarkerCount(); i++ )
	{ int tmpx = (int)(fSequence->GetMarker(i)->Offset / fTicksPerPix);
		gdk_draw_string( fBuffer, tmpFont, tmpGC, tmpx + 3, 9,	fSequence->GetMarker(i)->GetName() );
		gdk_draw_line( fBuffer, tmpGC, tmpx, 0, tmpx, 11 ); }

	gdk_gc_set_clip_rectangle( tmpGC, &aEvent->area );	// drawable section
	gdk_draw_pixmap( tmpWnd, tmpGC, fBuffer, 0, 0, 0, 0, fWidth, tmpHeight );
	gdk_gc_set_clip_rectangle( tmpGC,	NULL );           // go!
	gdk_gc_destroy( tmpGC );

	fLastPixPosition = 0;
	DrawPosition();																			// force position drawing

	return TRUE;
}

//=============================================================================
// mouse down in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
void TMvcHeader::MouseDown( long x )
{
	fPositionActive = true;
	fSequence->SetPosition( x * fTicksPerPix );
}

//=============================================================================
// mouse up in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
void TMvcHeader::MouseUp( long x )
{
	fPositionActive = false;
}

//=============================================================================
// mouse move in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
void TMvcHeader::MouseMove( long x )
{
	if ( fPositionActive )
	{
																											// end position on grid
		int tmpPosition = (int)(x * fTicksPerPix);						
		if ( tmpPosition > (int)(fWidth * fTicksPerPix) )	// more than max?
			{	tmpPosition = (int)(fWidth * fTicksPerPix); }	// correct it!
		if ( tmpPosition < 0 ) { tmpPosition = 0; }				// minimum?

		fSequence->SetPosition( tmpPosition );						// set selection end of sequence

																										
		GtkAdjustment* tmpAdj = fAdjustment;              // scroll window?	
																											// in direction x ->
		if ( tmpAdj->value < ( x - fWindowWidth ) && tmpAdj->value < fWidth - fWindowWidth )
		{
																											// increase value by 10th of mouse position
			tmpAdj->value += ( x - ( tmpAdj->value + fWindowWidth )) / 5 ;
			if ( tmpAdj->value > fWidth - fWindowWidth - 4 )// if more than max
				{ tmpAdj->value = fWidth - fWindowWidth - 4; }// correct to max
			gtk_adjustment_value_changed( tmpAdj );					// set new position in x adjustment
		}

		if ( tmpAdj->value > x && tmpAdj->value > 0 )     // in direction x <
		{
			tmpAdj->value -= (tmpAdj->value - x) / 5;

			if ( tmpAdj->value < 0 )												// if less than 0
				{ tmpAdj->value = 0; }												// correct to 0
			gtk_adjustment_value_changed( tmpAdj );					// set new position in x adjustment
		}
	}
}

//=============================================================================
// draw position in main and header
// 12.09.2000 Adrian Meyer
//============================================================================
void TMvcHeader::DrawPosition( void )
{
	int tmpx = (int)(fSequence->GetPosition() / fTicksPerPix);
	tmpx++;

	if ( fLastPixPosition == tmpx )	{ return; }
	fLastPixPosition = tmpx;

	GdkDrawable* tmpWnd = fWidget->window;							// draw to header
	GdkGC* tmpGC = gdk_gc_new( fWidget->window );				// new graphic context
	gdk_gc_copy( tmpGC, fWidget->style->fg_gc[fWidget->state] );

	GdkRectangle* tmpDrawingRec = new GdkRectangle;

	tmpDrawingRec->x = 0;//(int)gtk_scrolled_window_get_hadjustment( GTK_SCROLLED_WINDOW( FindWidget( "scrMainView" )))->value;
	tmpDrawingRec->y = 0;	
	tmpDrawingRec->width = fWidth;//fWindowWidth;
	tmpDrawingRec->height = fHeight;	

	gdk_gc_set_function( tmpGC, GDK_COPY );
	gdk_gc_set_clip_rectangle( tmpGC, tmpDrawingRec );	// drawable section
	gdk_draw_pixmap( tmpWnd, tmpGC, fBuffer, tmpDrawingRec->x, 0, tmpDrawingRec->x, 0, fWindowWidth, tmpDrawingRec->height );
	gdk_gc_set_foreground( tmpGC, colPosition );				
	gdk_gc_set_function( tmpGC, GDK_AND );
	gdk_draw_line( tmpWnd, tmpGC, tmpx, 0, tmpx, fHeight );
	gdk_gc_set_clip_rectangle( tmpGC,	NULL );           // go!

	gdk_gc_destroy( tmpGC );
}

//=============================================================================
// redraw area
// 05.09.2000 Adrian Meyer
//=============================================================================
bool HeaderExposeHook( GtkWidget *widget, GdkEventExpose *event, gpointer user_data )
{
	TMvcHeader* tmpMvcMainView = (TMvcHeader*)user_data;
  return tmpMvcMainView->Expose( widget, event );
}

//=============================================================================
// mouse down in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
bool HeaderMouseDown( GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	TMvcHeader* tmpMvcMainView = (TMvcHeader*)user_data;
	tmpMvcMainView->MouseDown( event->x );
	return true;
}

//=============================================================================
// mouse up in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
bool HeaderMouseUp( GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	TMvcHeader* tmpMvcMainView = (TMvcHeader*)user_data;
	tmpMvcMainView->MouseUp( event->x );
	return true;
}

//=============================================================================
// mouse move in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
bool HeaderMouseMove( GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
{
	TMvcHeader* tmpMvcMainView = (TMvcHeader*)user_data;
	tmpMvcMainView->MouseMove( event->x );
	return true;
}
