//=============================================================================
//  model view control input box implementation
//=============================================================================
#include <mvcBase.h>
#include <mvcInputBox.h>
#include <stdio.h>

// callbacks
void OkClickedInputBox( GtkButton *button, gpointer user_data );
void CancelClickedInputBox( GtkButton *button, gpointer user_data );
void ClosedInputBox( GtkObject *object, gpointer user_data );

//=============================================================================
// construtor MvcInputBox														
// 06.09.2001 Adrian Meyer
//=============================================================================
TMvcInputBox::TMvcInputBox( void )
{
  wndInputBox = 0;
  fText = 0;
  fResult = mrNone;
}

//=============================================================================
// destrutor
// 06.09.2001 Adrian Meyer
//=============================================================================
TMvcInputBox::~TMvcInputBox( void )
{
  if (fText) delete fText;
}

//=============================================================================
// show input box
// 06.09.2001 Adrian Meyer
//=============================================================================
short TMvcInputBox::Show( char* aTitel, char* aCaption, char* aDefault )
{
  wndInputBox = create_wndInputBox();                 // call glade
  gtk_widget_show( wndInputBox );                     // display widget

	AddChildrenToWidgetList( wndInputBox );             // fetch widgets
	
  HookSignal( "btnOK", "clicked", GTK_SIGNAL_FUNC( OkClickedInputBox ), this );
  HookSignal( "btnCancel", "clicked", GTK_SIGNAL_FUNC( CancelClickedInputBox ), this );
  HookSignal( "wndInputBox", "destroy", GTK_SIGNAL_FUNC( ClosedInputBox ), this );

	fResult = mrNone;																    // init box
  gtk_window_set_title( GTK_WINDOW( wndInputBox ), aTitel );
  gtk_label_set_text( GTK_LABEL( FindWidget( "lblInputBox" ) ), aCaption );
	gtk_entry_set_text( GTK_ENTRY( FindWidget( "edtInputBox" )), aDefault );

	while ( fResult == mrNone )													// wait for result
		{	gtk_main_iteration();	}

  if ( fResult != 9999 )
    gtk_widget_destroy( wndInputBox );                 // remove widget
  else
    fResult = mrCancel;

  wndInputBox = 0;
	return fResult;
}

//=============================================================================
// read main widget																		
// 06.09.2001 Adrian Meyer
//=============================================================================
GtkWidget* TMvcInputBox::GetWidget( void )
{
	return wndInputBox;
}

//=============================================================================
// set result																					
// 06.09.2001 Adrian Meyer
//=============================================================================
void TMvcInputBox::SetResult( short aResult )
{
  if ( fResult != mrNone ) return;                    // result already set
  if ( aResult == mrOK )                              // save result string
  {
    if ( fText ) delete fText;
    GtkEntry* tmpEntry = GTK_ENTRY( FindWidget( "edtInputBox" ));
  	fText = new char[ strlen( gtk_entry_get_text( tmpEntry )) + 1 ];
  	strcpy( fText, gtk_entry_get_text( tmpEntry ));
  }

	fResult = aResult;
}

//=============================================================================
// ok clicked																					
// 06.09.2001 Adrian Meyer
//=============================================================================
void OkClickedInputBox( GtkButton *button, gpointer user_data )
{
	((TMvcInputBox*)user_data )->SetResult( mrOK );
	gtk_widget_hide( (( TMvcInputBox*)user_data)->GetWidget( ));
}

//=============================================================================
// cancel clicked																			
// 06.09.2001 Adrian Meyer
//=============================================================================
void CancelClickedInputBox( GtkButton *button, gpointer user_data )
{
	((TMvcInputBox*)user_data )->SetResult( mrCancel );
	gtk_widget_hide( (( TMvcInputBox*)user_data)->GetWidget( ));
}

//=============================================================================
// box closed																				
// 06.09.2001 Adrian Meyer
//=============================================================================
void ClosedInputBox( GtkObject *object, gpointer user_data )
{
	((TMvcInputBox*)user_data )->SetResult( 9999 );
}

//=============================================================================
// read result text
// 06.09.2001 Adrian Meyer
//=============================================================================
char* TMvcInputBox::GetText( void )
{
  return fText;
}
