//=============================================================================
// model view control time signature list
//=============================================================================
#include <mvcKeySignatureList.h>
#include <stdio.h>

// callbacks
void KeySignatureListSelectRow( GtkWidget* aWidget,gint row,gint column,GdkEventButton *event,gpointer user_data);
void KeySignatureListDeleteClicked( GtkWidget* aWidget, gpointer user_data);
void KeySignatureListAddClicked( GtkWidget* aWidget, gpointer user_data);
void ViewKeySignatureProperties( GtkWidget* aWidget, gpointer user_data );

//=============================================================================
// constructor
// 12.09.2000 Adrian Meyer
//=============================================================================
TMvcKeySignatureList::TMvcKeySignatureList( GtkWidget* aWidget, TMidiSequence* aSequence )
{
	fSequence = aSequence;
	fWidget = aWidget;
	AddChildrenToWidgetList( aWidget );                 // fetch widgets
	fMCMessageFlag = true;
	fSelectedKeySignature = 0;

	HookSignal( "grdKeySignatureList", "select-row", GTK_SIGNAL_FUNC( KeySignatureListSelectRow ), this );
  HookSignal( "btnKeySignatureListAdd", "clicked", GTK_SIGNAL_FUNC( KeySignatureListAddClicked ), this );
  HookSignal( "btnKeySignatureListDelete", "clicked", GTK_SIGNAL_FUNC( KeySignatureListDeleteClicked ), this );
  HookSignal( "btnKeySignatureListProperties", "clicked", GTK_SIGNAL_FUNC( ViewKeySignatureProperties ), this );

	SetData();
	ValidateGui();																			// set gui
}

//=============================================================================
// read widget
// 12.09.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcKeySignatureList::GetWidget( void )
{
	return fWidget;
}

//=============================================================================
// midi class messages
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureList::MCMessage( TMidiBase* aMidiBase, sMCMessage* aMCMessage )
{
  switch ( aMCMessage->Message )
 	{
 		case cMMSequenceOpenFile:
 		case cMMSequenceNew:															
 		case cMMSequenceAddKeySignature:
 		case cMMSequenceDeleteKeySignature:
		case cMMSequenceChangeKeySignature:
 		case cMMSequenceSortKeySignature:
 				fMCMessageFlag = true;												// react to changes
 				SetData();
 			break;
 	}

	ValidateGui();
}

//=============================================================================
// set data
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureList::SetData( void )
{
	GtkCList* tmpCList = GTK_CLIST( FindWidget( "grdKeySignatureList" ));
	gtk_clist_freeze( tmpCList );
	gtk_clist_clear( tmpCList );
	char* tmpList[1][4] = {{"","","",""}};							// empty row

	for ( long i = 0; i<fSequence->GetKeySignatureCount(); i++ )
	{
 		gtk_clist_append( tmpCList, tmpList[0] );
		gtk_clist_set_row_data( tmpCList,	i, fSequence->GetKeySignature(i) );
		SetRowData( i );
		if ( fSequence->GetKeySignature(i) == fSelectedKeySignature )		// reselect KeySignature
			{	gtk_clist_select_row( tmpCList,	i, 0 ); }
	}	

	gtk_clist_thaw( tmpCList );

	fMCMessageFlag = true;															// react to changes
	ValidateGui();
}

//=============================================================================
// set data of a row
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureList::SetRowData( int aRow )
{
 	gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdKeySignatureList" )),
 		aRow, 1, fSequence->GetMeasureBeatTick( fSequence->GetKeySignature(aRow)->Offset ) );

	char* tmpStr = new char[100];

	int tmpIndex = 7 + fSequence->GetKeySignature(aRow)->SharpFlatCount;

	sprintf( tmpStr, "%s", cKeySignature[tmpIndex][ fSequence->GetKeySignature(aRow)->IsMinor ] );
 	gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdKeySignatureList" )),	aRow, 2, tmpStr );

	sprintf( tmpStr, "%s", cKeyRemark[tmpIndex] );
 	gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdKeySignatureList" )),	aRow, 3, tmpStr );

	delete tmpStr;
}

//=============================================================================
// write selected row
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureList::SetSelectedRow( int aRow )
{
	fSelectedKeySignature = fSequence->GetKeySignature( aRow );
}

//=============================================================================
// validate gui
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureList::ValidateGui( void )
{
	if ( fMCMessageFlag )
	{
    gtk_widget_set_sensitive( FindWidget( "btnKeySignatureListDelete" ), ( fSequence->GetKeySignatureCount() > 1 ) );
		fMCMessageFlag = false;
	}
}

//=============================================================================
// add new time signature
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureList::AddKeySignature( void )
{
	long tmpID = fSequence->AddKeySignature( 0, 0, 0 );

	sKeySignatureList* tmpKeySignature = fSequence->GetKeySignature( tmpID );
  fSequence->SortKeySignature();
	for ( int i = 0; i<fSequence->GetKeySignatureCount(); i++ )
	{
		if ( tmpKeySignature == fSequence->GetKeySignature(i))
		gtk_clist_select_row( GTK_CLIST( FindWidget( "grdKeySignatureList" )), i, 0 );
	}	
	ShowKeySignatureProperties();
}

//=============================================================================
// delete selected time signature
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureList::DeleteKeySignature( void )
{
	if ( fSelectedKeySignature != 0 )
	{
		int oldID = fSelectedKeySignature->ID;
		for ( int i=0; i<fSelectedKeySignature->ID; i++ )
		{
			fSelectedKeySignature = fSequence->GetKeySignature(i);
		}

		if ( fSelectedKeySignature == 0)
		  { fSelectedKeySignature = fSequence->GetKeySignature(0); }

  	fSequence->DeleteKeySignature( oldID );
	}
}

//=============================================================================
// show time signature properties
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureList::ShowKeySignatureProperties( void )
{
	if ( fSelectedKeySignature != 0 )
	{
		fKeySignatureProperties = new TMvcKeySignatureProperties( fSequence, fSelectedKeySignature->ID );
    fKeySignatureProperties->LoadLanguageFile( GetLanguageFileName() );
	}
}

//=============================================================================
// row is selected
// 12.09.2000 Adrian Meyer
//=============================================================================
void KeySignatureListSelectRow( GtkWidget* aWidget,gint row,gint column,GdkEventButton *event,gpointer user_data)
{
	((TMvcKeySignatureList*)user_data)->SetSelectedRow( row );
}

//=============================================================================
// add time signature
// 12.09.2000 Adrian Meyer
//=============================================================================
void KeySignatureListAddClicked( GtkWidget* aWidget, gpointer user_data)
{
	((TMvcKeySignatureList*)user_data)->AddKeySignature( );
}

//=============================================================================
// delete time signature
// 12.09.2000 Adrian Meyer
//=============================================================================
void KeySignatureListDeleteClicked( GtkWidget* aWidget, gpointer user_data)
{
	((TMvcKeySignatureList*)user_data)->DeleteKeySignature( );
}

//=============================================================================
// edit time signature  properties
// 12.09.2000 Adrian Meyer
//=============================================================================
void ViewKeySignatureProperties( GtkWidget* aWidget, gpointer user_data )
{
	((TMvcKeySignatureList*)user_data)->ShowKeySignatureProperties( );
}
