//=============================================================================
//  model view control time signature properties
//=============================================================================
#include <mvcKeySignatureProperties.h>
#include <stdio.h>
#include <stdlib.h>

// callbacks
void OkClickedKeySignatureProp( GtkButton *button, gpointer user_data );
void CancelClickedKeySignatureProp( GtkButton *button, gpointer user_data );
void ToggleClickedKeySignature( GtkToggleButton *togglebutton, gpointer user_data );

//=============================================================================
// construtor MvcKeySignatureProperties											
// 11.09.2000 Adrian Meyer
//=============================================================================
TMvcKeySignatureProperties::TMvcKeySignatureProperties( TMidiSequence* aSequence, long aKeySignatureID )
{
  wndKeySignatureProperties = create_wndKeySignatureProperties();	// call glade
	AddChildrenToWidgetList( wndKeySignatureProperties );// fetch widgets
	fSequence = aSequence;
	fKeySignatureID = aKeySignatureID;
	
  HookSignal( "btnOk", "clicked", GTK_SIGNAL_FUNC( OkClickedKeySignatureProp ), this );
  HookSignal( "btnCancel", "clicked", GTK_SIGNAL_FUNC( CancelClickedKeySignatureProp ), this );
  HookSignal( "rboKeySignatureMajor", "toggled", GTK_SIGNAL_FUNC( ToggleClickedKeySignature ), this );
  HookSignal( "rboKeySignatureMinor", "toggled", GTK_SIGNAL_FUNC( ToggleClickedKeySignature ), this );

																											// don't change 1st
	gtk_widget_set_sensitive( FindWidget( "spiMeasure" ), fKeySignatureID != 0 );


	char tmpStr[100];

	fKeySignaturesMajor = 0;
	fKeySignaturesMinor = 0;
	for ( int i = 0; i<15; i++ )											// add only numbers
	{
		sprintf( tmpStr, "%s - %s", cKeySignature[ i ][0], cKeyRemark[ i ]);
		char* tmpItem = new char[ strlen(tmpStr) + 1 ];
		strcpy( tmpItem, tmpStr );
		fKeySignaturesMajor = g_list_append( fKeySignaturesMajor, tmpItem );

		sprintf( tmpStr, "%s - %s", cKeySignature[ i ][1], cKeyRemark[ i ]);
		tmpItem = new char[ strlen(tmpStr) + 1 ];
		strcpy( tmpItem, tmpStr );
		fKeySignaturesMinor = g_list_append( fKeySignaturesMinor, tmpItem );
	}

	if ( fSequence == 0 ) return;
  gtk_widget_show( wndKeySignatureProperties );							
	SetData();																					// set data to gui
}

//=============================================================================
// destrutor			
// 11.09.2000 Adrian Meyer
//=============================================================================
TMvcKeySignatureProperties::~TMvcKeySignatureProperties( void )
{
	delete wndKeySignatureProperties;
}

//=============================================================================
// set minor over radio button
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureProperties::SetIsMinor( void )
{
	if ( gtk_toggle_button_get_active((GtkToggleButton*)FindWidget( "rboKeySignatureMinor" )))
	{ gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboKeySignature" )), fKeySignaturesMinor ); }
	else
	{	gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboKeySignature" )), fKeySignaturesMajor ); }

}

//=============================================================================
// set data to gui				
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureProperties::SetData( void )
{
 	long tmpMeasure = 0; long tmpBeat = 0; long tmpTick = 0;		

 	fSequence->GetMeasureBeatTick( fSequence->GetKeySignature( fKeySignatureID )->Offset, &tmpMeasure, &tmpBeat, &tmpTick );

	gtk_toggle_button_set_active((GtkToggleButton*)FindWidget( "rboKeySignatureMinor" ),
		fSequence->GetKeySignature( fKeySignatureID )->IsMinor );

	if ( fSequence->GetKeySignature( fKeySignatureID )->IsMinor == 1 )
	{ gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboKeySignature" )), fKeySignaturesMinor ); }
	else
	{	gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboKeySignature" )), fKeySignaturesMajor ); }

 																										
	char* tmpStr = new char[100];                     // set measure

	int tmpIndex = fSequence->GetKeySignature( fKeySignatureID )->SharpFlatCount + 7;

	sprintf( tmpStr, "%s - %s", cKeySignature[tmpIndex][fSequence->GetKeySignature( fKeySignatureID )->IsMinor], cKeyRemark[tmpIndex] );
	gtk_entry_set_text( GTK_ENTRY( FindWidget( "edtKeySignature" )), tmpStr );
	delete tmpStr;
}

//=============================================================================
// read data from gui into object											
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcKeySignatureProperties::GetData( void )
{
 	for ( int i = 0; i < (int)g_list_length( fKeySignaturesMajor ); i++ )
 	{
		if ( gtk_toggle_button_get_active((GtkToggleButton*)FindWidget( "rboKeySignatureMajor" )))
		{
   		if ( strcmp( (char*)g_list_nth_data( fKeySignaturesMajor, i ),
   			gtk_entry_get_text((GtkEntry*)FindWidget( "edtKeySignature" ))) == 0 )
   		{
  			fSequence->ChangeKeySignature( fKeySignatureID, i - 7, 0 );
  			break;
   		}					
		}
		else
		{
   		if ( strcmp( (char*)g_list_nth_data( fKeySignaturesMinor, i ),
   			gtk_entry_get_text((GtkEntry*)FindWidget( "edtKeySignature" ))) == 0 )
   		{
  			fSequence->ChangeKeySignature( fKeySignatureID, i - 7, 1 );
  			break;
   		}					
		}
 	}

  long tmpOffset =																		// new offset
		fSequence->GetOffset(
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiMeasure" ))), 1, 0 );

	fSequence->GetKeySignature( fKeySignatureID )->Offset = tmpOffset;
	fSequence->SortKeySignature();
}

//=============================================================================
// read widget						
// 11.09.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcKeySignatureProperties::GetWidget( void )
{
	return wndKeySignatureProperties;
}

//=============================================================================
// ok clicked																					
// 11.09.2000 Adrian Meyer
//=============================================================================
void OkClickedKeySignatureProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcKeySignatureProperties*)user_data)->GetWidget() );
	((TMvcKeySignatureProperties*)user_data)->GetData();
}

//=============================================================================
// cancel clicked																			
// 11.09.2000 Adrian Meyer
//=============================================================================
void CancelClickedKeySignatureProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcKeySignatureProperties*)user_data)->GetWidget() );
}

//=============================================================================
// major/minor selected
// 11.09.2000 Adrian Meyer
//=============================================================================
void ToggleClickedKeySignature( GtkToggleButton *togglebutton, gpointer user_data )
{
	((TMvcKeySignatureProperties*)user_data)->SetIsMinor();
}
