//=============================================================================
// model view control MAIN implementation
//=============================================================================
#include <mvcMain.h>
#include <string.h>
#include <stdio.h>
#include <sys/time.h>
#include <mvcMarkerProperties.h>
#include <mvcTimeSignatureProperties.h>
#include <mvcKeySignatureProperties.h>
#include <mvcUserSnapProperties.h>
#include <dirent.h>

//#include <mcALSAIO.h>
#include <mcOSSIO.h>

//#define __DEBUG_OUTPUT__

// callbacks
void HelpClicked( GtkWidget* aWidget, gpointer user_data);
void FileNewClicked( GtkWidget* aWidget, gpointer user_data);
void FileOpenClicked( GtkWidget* aWidget, gpointer user_data);
void FileSaveClicked( GtkWidget* aWidget, gpointer user_data);
void FileSaveAsClicked( GtkWidget* aWidget, gpointer user_data);
void FileSelectedSave( GtkWidget* aWidget, gpointer user_data);
void FileSelectedOpen( GtkWidget* aWidget, gpointer user_data);
void FileExit( GtkWidget* aWidget, gpointer user_data);
bool CheckSaved( gpointer user_data );

void LanguageDumpClicked( GtkWidget* aWidget, gpointer user_data );
void LanguageMenuClicked( GtkWidget* aWidget, gpointer user_data);
void LanguageChanged( GtkWidget* aWidget, gpointer user_data);

void ViewSongProperties( GtkWidget* aWidget, gpointer user_data);
void ViewTrackProperties( GtkWidget* aWidget, gpointer user_data);
void ViewEventPropertiesMain( GtkWidget* aWidget, gpointer user_data );
void ViewChanged( GtkWidget* aWidget, GtkNotebookPage *page,
										gint page_num, gpointer user_data );

void ViewMidiDeviceOptions( GtkWidget* aWidget, gpointer user_data);

void TrackListAddClicked( GtkWidget* aWidget, gpointer user_data);
void TrackListDeleteClicked( GtkWidget* aWidget, gpointer user_data);
void TrackListSelect( GtkWidget* aWidget, gint row, gint column,
												GdkEventButton *event, gpointer user_data );
void TrackListUnSelect( GtkWidget* aWidget, gint row, gint column,
													GdkEventButton *event, gpointer user_data );

bool SizeRequest( GtkWidget *widget, GdkEventConfigure *event, gpointer user_data );


void PlayTop( GtkWidget* aWidget, gpointer user_data );
void PlayEnd( GtkWidget* aWidget, gpointer user_data );
void PlayPlay( GtkWidget* aWidget, gpointer user_data );

// notebook pages
const short cViewMainView = 0;
const short cViewEventList = 1;

// message hook
void MCMessageHook( TMidiBase* aMidiBase, sMCMessage* aMCMessage );


//=============================================================================
// construtor MvcMain																	
// 09.03.2000 Adrian Meyer
//=============================================================================
TMvcMain::TMvcMain( char* aConfFileName )	
{
  cout << "MidiMountain, Version " << cVersion << ", " << cCopyright << endl;
  cout << "Using configuration file: " << aConfFileName << endl;
  fConfFileName = new char[ strlen( aConfFileName ) + 1 ];
  strcpy( fConfFileName, aConfFileName );
	fConfFile = LoadIni( fConfFileName );               // open conf file

  char* tmpPath = new char[2000];                     // path to pixmaps
	sprintf( tmpPath, "%s", GetIniSetting( fConfFile, "General", "Pixmaps", "./pixmaps" ).c_str() );
  add_pixmap_directory ( tmpPath );

	fTrackListFlag = true;
	fSequenceFlag = true;
	fMainView = 0;
	fEventList = 0;
  fMessageLock = false;
  fLanguageMenuList = NULL;	

  wndMain = create_wndMain ();												// call glade for main window
  gtk_widget_show( wndMain );													// display main window
	AddChildrenToWidgetList( wndMain );                 // fetch it!!!

	fSequence = new TMidiSequence( MCMessageHook, this );
	fSequence->AddTimeSignature( 4, 2, 24, 8, 0 );      // normal 4/4
	fSequence->AddKeySignature( 0, 0, 0 );							// c major
	fSequence->AddTempo(  500000 ,0 );						    	// 120 bpm
	fSequence->SetChanged( false );

//  #define ALSA_OUTPUT
  #ifdef ALSA_OUTPUT
    fIO = new TMidiALSAIO( 1 );
  #else
    fIO = new TMidiOSSIO( 1 );
  #endif

  fMapper = new TMidiMapper( fIO, fConfFileName );

  fPlayback = new TMidiPlayback( fSequence, fMapper );
  fPlayback->AddMCMessage( MCMessageHook, this );

	fMainView = new TMvcMainView( FindWidget( "vboMainView" ), fSequence );
	fEventList = new TMvcEventList( FindWidget( "vboEventList" ), fSequence );
	fMarkerList = new TMvcMarkerList( FindWidget( "vboMarkerList" ), fSequence );
	fTimeSignatureList = new TMvcTimeSignatureList( FindWidget( "vboTimeSignatureList" ), fSequence );
	fKeySignatureList = new TMvcKeySignatureList( FindWidget( "vboKeySignatureList" ), fSequence );
	fPianoRoll = new TMvcPianoRoll( FindWidget( "vboPianoRoll" ), fSequence );

	msg = new TMvcMessageBox;

	fFileDialog = gtk_file_selection_new( "Init");			// initialize file dialog
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fFileDialog)->ok_button),
                                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_hide),
                                           GTK_OBJECT(fFileDialog) );

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fFileDialog)->cancel_button),
                                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_hide),
                                           GTK_OBJECT(fFileDialog) );

  HookSignal( "wndMain", "destroy", GTK_SIGNAL_FUNC( FileExit ), this );
  HookSignal( "wndMain", "configure_event", GTK_SIGNAL_FUNC( SizeRequest ), this );

	HookSignal( "mnuFileNew", "activate", GTK_SIGNAL_FUNC( FileNewClicked ), this );
	HookSignal( "mnuFileOpen", "activate", GTK_SIGNAL_FUNC( FileOpenClicked ), this );
	HookSignal( "mnuFileSave", "activate", GTK_SIGNAL_FUNC( FileSaveClicked ), this );
	HookSignal( "mnuFileSaveAs", "activate", GTK_SIGNAL_FUNC( FileSaveAsClicked ), this );
  HookSignal( "mnuFileExit", "activate", GTK_SIGNAL_FUNC( FileExit ), this );

  HookSignal( "mnuDumpLanguageFile", "activate", GTK_SIGNAL_FUNC( LanguageDumpClicked ), this );
  HookSignal( "mnuLanguage", "activate", GTK_SIGNAL_FUNC( LanguageMenuClicked ), this );


	HookSignal( "btnFileNew", "clicked", GTK_SIGNAL_FUNC( FileNewClicked ), this );
	HookSignal( "btnFileOpen", "clicked", GTK_SIGNAL_FUNC( FileOpenClicked ), this );
	HookSignal( "btnFileSave", "clicked", GTK_SIGNAL_FUNC( FileSaveClicked ), this );
  HookSignal( "btnFileExit", "clicked", GTK_SIGNAL_FUNC( FileExit ), this );

  HookSignal( "nbkMain", "switch-page", GTK_SIGNAL_FUNC( ViewChanged ), this );
  HookSignal( "mnuViewSongProperties", "activate", GTK_SIGNAL_FUNC( ViewSongProperties ), this );
  HookSignal( "mnuViewTrackProperties", "activate", GTK_SIGNAL_FUNC( ViewTrackProperties ), this );
  HookSignal( "mnuViewEventProperties", "activate", GTK_SIGNAL_FUNC( ViewEventPropertiesMain ), this );
  HookSignal( "mnuHelpAbout", "activate", GTK_SIGNAL_FUNC( HelpClicked ), msg );

  HookSignal( "mnuMidiDeviceOptions", "activate", GTK_SIGNAL_FUNC( ViewMidiDeviceOptions ), this );

  HookSignal( "btnTrackListAdd", "clicked", GTK_SIGNAL_FUNC( TrackListAddClicked ), this );
  HookSignal( "btnTrackListDelete", "clicked", GTK_SIGNAL_FUNC( TrackListDeleteClicked ), this );
  HookSignal( "btnTrackListProperties", "clicked", GTK_SIGNAL_FUNC( ViewTrackProperties ), this );
  HookSignal( "grdTrackList", "select-row", GTK_SIGNAL_FUNC( TrackListSelect ), this );
  HookSignal( "grdTrackList", "unselect-row", GTK_SIGNAL_FUNC( TrackListUnSelect ), this );

  HookSignal( "btnTop", "clicked", GTK_SIGNAL_FUNC( PlayTop ), this );
  HookSignal( "btnEnd", "clicked", GTK_SIGNAL_FUNC( PlayEnd ), this );
  HookSignal( "btnStop", "clicked", GTK_SIGNAL_FUNC( PlayPlay ), this );
  HookSignal( "btnPlay", "clicked", GTK_SIGNAL_FUNC( PlayPlay ), this );


	gtk_scrolled_window_set_vadjustment(								// synchronize track list and main view
		GTK_SCROLLED_WINDOW( FindWidget( "scrMainView" )),
		gtk_scrolled_window_get_vadjustment( GTK_SCROLLED_WINDOW( FindWidget( "scrTrackList" ))));

	fCurrentView = 0;

//	fSequence->LoadFromFile( "./midi files/vocal1.mid" );
//	fSequence->AddTrack();
//	fSequence->AddTrack()->SetTrackName( "Hallo" );

	SetData();
	SetTitle();
	ValidateGui();
//  fSequence->SetSequenceName( "" );
}

//=============================================================================
// destrutor MvcMain						 											09.03.2000 Adrian Meyer
//=============================================================================
TMvcMain::~TMvcMain( void )
{
  PutIniSetting( fConfFile, "General", "LastVersion", cVersion );
	SaveIni( fConfFile, fConfFileName );
	delete wndMain;
}

//=============================================================================
// message from midi classes
// 01.09.2000 Adrian Meyer
//=============================================================================
void TMvcMain::MCMessage( TMidiBase* aMidiBase, sMCMessage* aMCMessage )
{
	#ifdef __DEBUG_OUTPUT__
	char* MessageType = "???";													// debugging system

	switch ( aMCMessage->Class )												// set message type
	{
    case cMCSequence: MessageType = "TMS";	break;
    case cMCTrack: 		MessageType = "TMT";	break;
    case cMCEvent: 		MessageType = "TME";	break;
    case cMCPlayback: MessageType = "TMP";	break;
	}

	char* tmpFormat = new char[1000];
	sprintf( tmpFormat, "%s - #%.6x - %s",
		MessageType, (int)aMCMessage->Message, aMCMessage->Text );

	cout << tmpFormat << endl;
	delete tmpFormat;
	#endif

  if ( fMessageLock )                                 // only pass new and open
  {
    if (( aMCMessage->Message == cMMSequenceOpenFile ) ||
        ( aMCMessage->Message == cMMSequenceNew ))
  	if ( fEventList != 0 )														// pass message
  	{
  		fMainView->MCMessage( aMidiBase, aMCMessage );	
  		fEventList->MCMessage( aMidiBase, aMCMessage );	
  		fMarkerList->MCMessage( aMidiBase, aMCMessage );	
  		fTimeSignatureList->MCMessage( aMidiBase, aMCMessage );	
  		fKeySignatureList->MCMessage( aMidiBase, aMCMessage );	
	  	fPianoRoll->MCMessage( aMidiBase, aMCMessage );	
  	}
    return;																					  // no messages !!!
  }

	char* tmpStr = new char[100];

  switch ( aMCMessage->Message )
	{
  	case cMMPlaybackStop:                  			  	  // playback stopped
        Play( FindWidget( "btnStop" ));
			break;

  	case cMMSequenceChanged:                  				// sequence changed
				SetTitle();
  			ValidateGui();
			break;

		case cMMSequenceSaveFile:                         // sequence/file name changed
  	case cMMSequenceSetSequenceName:               		
				SetTitle();
			break;

		case cMMSequenceSetSelectionStart:								// new selection start
				gtk_label_set_text(
					(GtkLabel*)FindWidget( "lblSelectionStart" ),
					fSequence->GetMeasureBeatTick( fSequence->GetSelectionStart()));
			break;
		
		case cMMSequenceSetSelectionEnd:									// new selection end
				gtk_label_set_text(
					(GtkLabel*)FindWidget( "lblSelectionEnd" ),
					fSequence->GetMeasureBeatTick( fSequence->GetSelectionEnd()));
			break;

		case cMMSequenceSetPosition:										  // new position
        {
				gtk_label_set_text(
					(GtkLabel*)FindWidget( "lblNavigationPosition" ),
					fSequence->GetMeasureBeatTick( fSequence->GetPosition()));

       	long tmpMeasure; long tmpBeat; long tmpTick;
       	long tmpBeatCount; long tmpTicksPerBeat;
     		fSequence->GetMeasureBeatTick( fSequence->GetPosition(), &tmpMeasure, &tmpBeat, &tmpTick );
     		fSequence->GetTimeInformation( tmpMeasure, &tmpBeatCount, &tmpTicksPerBeat );

				sprintf( tmpStr, "%d", (int)tmpBeatCount );
				gtk_label_set_text((GtkLabel*)FindWidget( "lblTimeSignatureNumerator" ), tmpStr );
				sprintf( tmpStr, "%d", (int)(4 * ((fSequence->GetDeltaTimesType()&0x7FFF)/tmpTicksPerBeat)));
				gtk_label_set_text((GtkLabel*)FindWidget( "lblTimeSignatureDenominator" ), tmpStr );

        sTempoList* tmpTempoList = fSequence->GetTempo(0);
        while ( tmpTempoList->Next && tmpTempoList->Offset < fSequence->GetPosition() )
          tmpTempoList = tmpTempoList->Next;
				sprintf( tmpStr, "Tempo:\n%.2f", 60000000.0 / tmpTempoList->Tempo );
				gtk_label_set_text((GtkLabel*)FindWidget( "lblSignatureTempo" ), tmpStr );
        };
			break;
		
		case cMMTrackSelect:															// track selection changed
				SetLastTrack( ((TMidiTrack*)aMidiBase)->GetIndex() );
			break;

		case cMMTrackSetName:															// new track name
				gtk_clist_set_text(	
					GTK_CLIST( FindWidget( "grdTrackList" )),
					((TMidiTrack*)aMidiBase)->GetIndex() - 1,
					1, ((TMidiTrack*)aMidiBase)->GetTrackName() );
			break;
	}

	if ( fEventList != 0 )															// pass message
	{
		fMainView->MCMessage( aMidiBase, aMCMessage );	
		fEventList->MCMessage( aMidiBase, aMCMessage );	
		fMarkerList->MCMessage( aMidiBase, aMCMessage );	
		fTimeSignatureList->MCMessage( aMidiBase, aMCMessage );	
		fKeySignatureList->MCMessage( aMidiBase, aMCMessage );	
		fPianoRoll->MCMessage( aMidiBase, aMCMessage );	
	}
	delete tmpStr;
}

//=============================================================================
// read first selected event													04.04.2000 Adrian Meyer
//=============================================================================
TMidiEvent* TMvcMain::GetSelectedEvent( void )
{
	int tmpPage = gtk_notebook_get_current_page(
										GTK_NOTEBOOK( FindWidget( "nbkMain" )));

	switch ( tmpPage ) 																	
	{
		case cViewEventList:
      return fEventList->GetSelectedEvent(); break;
	}

	return 0;
}

//=============================================================================
// validate gui																				27.03.2000 Adrian Meyer
//=============================================================================
void TMvcMain::ValidateGui( void )
{
	if ( fTrackListFlag != ( fSequence->GetTrackCount() > 1 ) )
	{	
		fTrackListFlag = ( fSequence->GetTrackCount() > 1 );
    gtk_widget_set_sensitive( FindWidget( "btnTrackListProperties" ), fSequence->GetTrackCount() > 1 );
    gtk_widget_set_sensitive( FindWidget( "mnuViewTrackProperties" ), fSequence->GetTrackCount() > 1 );
    gtk_widget_set_sensitive( FindWidget( "grdTrackList" ), fSequence->GetTrackCount() > 1 );
    gtk_widget_set_sensitive( FindWidget( "btnTrackListDelete" ), fSequence->GetTrackCount() > 1 );
	}

	if ( fSequenceFlag != fSequence->GetChanged() )
	{
		fSequenceFlag = fSequence->GetChanged();		
  	gtk_widget_set_sensitive( FindWidget( "btnFileSave" ), fSequence->GetChanged() );
	  gtk_widget_set_sensitive( FindWidget( "mnuFileSave" ), fSequence->GetChanged() );
	}
}

//=============================================================================
// set data to gui																		27.03.2000 Adrian Meyer
//=============================================================================
void TMvcMain::SetData( void )
{
	// track list
	gtk_clist_clear( GTK_CLIST( FindWidget( "grdTrackList" )));
	char* tmpList[1][2] = {{"x"," "}};

	// list tracks without master track (number 0)
	for ( int i = 1; i<fSequence->GetTrackCount(); i++ )
	{	
		char* tmpStr = new char[200];
		if ( strlen( fSequence->GetTrack(i)->GetTrackName() ) == 0 && fSequence->GetTrack(i)->GetInstrumentPatch() > 0 )
		{
	    strcpy( tmpStr, cGMPatchName[ fSequence->GetTrack(i)->GetInstrumentPatch() ] );
		}
		else
		{
	    strcpy( tmpStr, fSequence->GetTrack(i)->GetTrackName() );
		}
		
		gtk_clist_append( GTK_CLIST( FindWidget( "grdTrackList" )),tmpList[0] );
		gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdTrackList" )),
			i - 1, 1, tmpStr );

		if ( fSequence->GetTrack(i)->GetSelected() )			// set old selection
			{ gtk_clist_select_row( GTK_CLIST( FindWidget( "grdTrackList" )), i-1, 1 ); }

		delete tmpStr;
	}
}

//=============================================================================
// new file
// 31.08.2000 Adrian Meyer
//=============================================================================
void TMvcMain::NewFile( void )
{
	fTrackListFlag = true;
	fSequenceFlag = true;
	fSequence->New();																		// new sequence
	fSequence->AddTimeSignature( 4, 2, 24, 8, 0 );
	fSequence->AddKeySignature( 0, 0, 0 );
	fSequence->AddTempo(  500000 ,0 );						    	// 120 bpm
	SetData();																					// set data to gui
	fSequence->SetChanged( false );
}

//=============================================================================
// open a file
// 26.03.2000 Adrian Meyer
//=============================================================================
void TMvcMain::OpenFile( char* aFileName )
{
  fMessageLock = true;																// turn messaging off
	fSequence->LoadFromFile( aFileName );								// load file

	fTrackListFlag = false;
	fSequenceFlag = false;
	SetData();				              										// set data to gui
	SetTitle();
	ValidateGui();																	
  fMessageLock = false;																// turn messaging on
}

//=============================================================================
// save a file
// 31.08.2000 Adrian Meyer
//=============================================================================
void TMvcMain::SaveFile( char* aFileName )
{
	fSequence->SaveToFile( aFileName );
}

//=============================================================================
// set title of main window
// 31.08.2000 Adrian Meyer
//=============================================================================
void TMvcMain::SetTitle( void )
{
	char* tmpTitle = new char[1000];
 	char* tmpSequenceName = new char[500];
 	char* tmpFileName = new char[500];		

	if ( fSequence != 0 )																// sequence here?
	{
		
		if ( strlen( fSequence->GetSequenceName()) == 0 )	// sequence name
			{	strcpy( tmpSequenceName, "unknown" ); }
		else
			{	strcpy( tmpSequenceName, fSequence->GetSequenceName() ); }
		
		if ( strlen( fSequence->GetFileName()) == 0 )		// file name
			{	strcpy( tmpFileName, "unknown.mid" ); }
		else
			{	strcpy( tmpFileName, strrchr( fSequence->GetFileName(), '/' ) + 1 ); }

    sprintf( tmpTitle, "MidiMountain - %s [ %s ]", tmpSequenceName, tmpFileName );			
	  if ( fSequence->GetChanged() )	{ strcat( tmpTitle, "*" ); }

	}
	else
	{
		strcpy( tmpTitle, "MidiMountain" );
	}
	
	gtk_window_set_title( GTK_WINDOW( wndMain ), tmpTitle );

	delete tmpTitle;
	delete tmpSequenceName;
	delete tmpFileName;
}

//=============================================================================
// read midi sequence																	26.03.2000 Adrian Meyer
//=============================================================================
TMidiSequence* TMvcMain::GetSequence( void )
{
  return fSequence;
}

//=============================================================================
// read midi IO
// 05.03.2001 Adrian Meyer
//=============================================================================
TMidiIO* TMvcMain::GetIO( void )
{
  return fIO;
}

//=============================================================================
// read device mapper
// 20.07.2001 Adrian Meyer
//=============================================================================
TMidiMapper* TMvcMain::GetMapper( void )
{
  return fMapper;
}

//=============================================================================
// read conf file name
// 22.07.2001 Adrian Meyer
//=============================================================================
char* TMvcMain::GetConfFileName( void )
{
  return fConfFileName;
}

//=============================================================================
// read main widget																		27.03.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcMain::GetWidget( void )
{
	return wndMain;
}

//=============================================================================
// read file dialog
// 01.09.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcMain::GetFileDialog( void )
{
	return fFileDialog;
}

//=============================================================================
// read view																					09.04.2000 Adrian Meyer
//=============================================================================
int TMvcMain::GetCurrentView( void )
{
	return fCurrentView;
}

//=============================================================================
// write view																					09.04.2000 Adrian Meyer
//=============================================================================
void TMvcMain::SetCurrentView( int aView )
{
	fCurrentView = aView;

	fMainView->SetUpdateView( false );
	fEventList->SetUpdateView( false );

	switch ( fCurrentView )
	{
		case 000: ; break;

		case 100: fEventList->SetUpdateView( true );
							fEventList->SetData();
							break;
	}
}

//=============================================================================
// set last touched track
// 31.08.2000 Adrian Meyer
//=============================================================================
void TMvcMain::SetLastTrack( int aLastTrack )							
{
	fLastTrack = aLastTrack;
}

//=============================================================================
// read last touched track
// 31.08.2000 Adrian Meyer
//=============================================================================
int TMvcMain::GetLastTrack( void )
{
	return fLastTrack;
}

//=============================================================================
// add new track
// 31.08.2000 Adrian Meyer
//=============================================================================
void TMvcMain::AddTrack( void )
{
	if ( fSequence->GetTrackCount() == 0 )							// master track here?
		{ fSequence->AddTrack(); };

	TMidiTrack* tmpTrack = fSequence->AddTrack();

  char tmpTrackName[20];															// set track name
	sprintf( tmpTrackName, "Track %d", tmpTrack->GetIndex() );
	tmpTrack->SetTrackName( tmpTrackName );
	tmpTrack->SetSelected( true );

	SetData();	
	ValidateGui();

	fLastTrack = tmpTrack->GetIndex();
	
	ViewTrackProperties( 0, this );
}

//=============================================================================
// delete track
// 01.09.2000 Adrian Meyer
//=============================================================================
void TMvcMain::DeleteTrack( void )
{
	fSequence->DeleteTrack( fLastTrack );								// delete track

	if ( fLastTrack > (fSequence->GetTrackCount()-1) )	// deletet track
		{	fLastTrack = fSequence->GetTrackCount()-1; }	

	SetData();	
	ValidateGui();
}

//=============================================================================
// resize main window
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMain::Resize( void )
{
	if ( fMainView != 0 ) { fMainView->Resize(); }
}

//=============================================================================
// start or stop playback
// 12.02.2001 Adrian Meyer
//=============================================================================
void TMvcMain::Play( GtkWidget* aWidget )
{
  GtkWidget* tmpPlay = FindWidget( "btnPlay" );
																											// start playback
  if ( aWidget == tmpPlay && gtk_toggle_button_get_active( (GtkToggleButton*)tmpPlay ))
  {
    fPlayback->Start();
  }
  else																								// stop playback
  {
    fPlayback->Stop();
    if ( gtk_toggle_button_get_active( (GtkToggleButton*)tmpPlay ))
    gtk_toggle_button_set_active( (GtkToggleButton*)tmpPlay, false );
  }
}																

//=============================================================================
// dump language file
// 27.02.2001 Adrian Meyer
//=============================================================================
void TMvcMain::DumpLanguage( char* aFileName )
{
  DumpLanguageFile( aFileName );											// main window	

	(new TMvcSongProperties( 0 ))->DumpLanguageFile( aFileName );
	(new TMvcTrackProperties( 0, 0 ))->DumpLanguageFile( aFileName );
	(new TMvcEventProperties( 0, true ))->DumpLanguageFile( aFileName );
	(new TMvcMarkerProperties( 0, 0 ))->DumpLanguageFile( aFileName );
	(new TMvcTimeSignatureProperties( 0, 0 ))->DumpLanguageFile( aFileName );
	(new TMvcKeySignatureProperties( 0, 0 ))->DumpLanguageFile( aFileName );
	(new TMvcUserSnapProperties( 0 ))->DumpLanguageFile( aFileName );
	(new TMvcMapperProperties( 0 ))->DumpLanguageFile( aFileName );
}

//=============================================================================
// show about 																				23.03.2000 Adrian Meyer
//=============================================================================
void HelpClicked( GtkWidget* aWidget, gpointer user_data )
{
	GtkWidget* tmp = create_wndAbout();
	gtk_widget_show( tmp );
}

//=============================================================================
// check if file is saved															24.03.2000 Adrian Meyer
//=============================================================================
bool CheckSaved( gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data);
	char tmpMessage[1000];
	sprintf( tmpMessage, "The current file \"%s\" is not saved. Save it?", main->GetSequence()->GetFileName() );
	
	if ( main->GetSequence()->GetChanged() )
	{
		switch ( main->msg->Show( "saving file", tmpMessage, mrYes+mrNo+mrCancel ))
		{
			case mrNo:  { return true; } 										// don't save
			case mrYes: {																		// save it
     							  FileSaveClicked( 0, user_data );
										if ( !main->GetSequence()->GetChanged() )
											{ return true; }
								  }
		}
	}
	else
	{	return true; }																		// saved

	return false;																				// canceled
}

//=============================================================================
// new file																						24.03.2000 Adrian Meyer
//=============================================================================
void FileNewClicked( GtkWidget* aWidget, gpointer user_data)
{
  TMvcMain* main = ((TMvcMain*)user_data);

	if ( CheckSaved( user_data ))
	{
		main->NewFile();																	// new file		
	}
}

//=============================================================================
// open file																					24.03.2000 Adrian Meyer
//=============================================================================
void FileOpenClicked( GtkWidget* aWidget, gpointer user_data)
{
  TMvcMain* main = ((TMvcMain*)user_data);

	if ( CheckSaved( user_data ))
	{
		gtk_window_set_title( GTK_WINDOW( main->GetFileDialog() ), "Open...");

//		gtk_window_set_modal( GTK_WINDOW( main->GetFileDialog() ), true );
	  gtk_window_set_position( GTK_WINDOW( main->GetFileDialog() ), GTK_WIN_POS_CENTER);

    gtk_signal_connect( GTK_OBJECT (GTK_FILE_SELECTION(main->GetFileDialog())->ok_button ),
                            "clicked", GTK_SIGNAL_FUNC( FileSelectedOpen ), user_data );

    gtk_widget_show_now( main->GetFileDialog() );
	}
}

//=============================================================================
// save file																					24.03.2000 Adrian Meyer
//=============================================================================
void FileSaveClicked( GtkWidget* aWidget, gpointer user_data)
{
  TMvcMain* main = ((TMvcMain*)user_data);
	if ( strlen( main->GetSequence()->GetFileName() ) == 0 )
	{
		FileSaveAsClicked( aWidget, user_data );					// find file name
	}
	else
	{                                                   // save file
		main->SaveFile( main->GetSequence()->GetFileName() );
	}
}

//=============================================================================
// save file as...																		24.03.2000 Adrian Meyer
//=============================================================================
void FileSaveAsClicked( GtkWidget* aWidget, gpointer user_data)
{
  TMvcMain* main = ((TMvcMain*)user_data);

	gtk_window_set_title( GTK_WINDOW( main->GetFileDialog() ), "Save as...");

  gtk_window_set_position( GTK_WINDOW( main->GetFileDialog() ), GTK_WIN_POS_CENTER);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(main->GetFileDialog())->ok_button),
                          "clicked", GTK_SIGNAL_FUNC( FileSelectedSave ), user_data );

  gtk_widget_show_now( main->GetFileDialog() );
}

//=============================================================================
// file selected in save dialog												24.03.2000 Adrian Meyer
//=============================================================================
void FileSelectedSave( GtkWidget* aWidget, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data);
	main->SaveFile(	gtk_file_selection_get_filename (GTK_FILE_SELECTION(main->GetFileDialog())));
}

//=============================================================================
// file selected in open dialog												24.03.2000 Adrian Meyer
//=============================================================================
void FileSelectedOpen( GtkWidget* aWidget, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data);
	main->OpenFile( gtk_file_selection_get_filename (GTK_FILE_SELECTION(main->GetFileDialog())));
}

//=============================================================================
// exit application																		24.03.2000 Adrian Meyer
//=============================================================================
void FileExit( GtkWidget* aWidget, gpointer user_data)
{
	if ( CheckSaved( user_data ))
	{
		gtk_main_quit();
	}
}

//=============================================================================
// add track to list																	24.03.2000 Adrian Meyer
//=============================================================================
void TrackListAddClicked( GtkWidget* aWidget, gpointer user_data)
{
  TMvcMain* main = ((TMvcMain*)user_data );
	main->AddTrack();
}

//=============================================================================
// delete track from list																	
// 01.09.2000 Adrian Meyer
//=============================================================================
void TrackListDeleteClicked( GtkWidget* aWidget, gpointer user_data)
{
  TMvcMain* main = ((TMvcMain*)user_data );
	main->DeleteTrack();
}

//=============================================================================
// select track in track list													29.03.2000 Adrian Meyer
//=============================================================================
void TrackListSelect( GtkWidget* aWidget, gint row, gint column,
				GdkEventButton *event, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data );
	main->GetSequence()->GetTrack( row + 1 )->SetSelected( true );
}

//=============================================================================
// unselect track in track list												29.03.2000 Adrian Meyer
//=============================================================================
void TrackListUnSelect( GtkWidget* aWidget, gint row, gint column,
				GdkEventButton *event, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data );
	main->GetSequence()->GetTrack( row + 1 )->SetSelected( false );
}

//=============================================================================
// view song properties																26.03.2000 Adrian Meyer
//=============================================================================
void ViewSongProperties( GtkWidget* aWidget, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data);
	main->fSongProperties = new TMvcSongProperties( main->GetSequence() );
	main->fSongProperties->LoadLanguageFile( main->GetLanguageFileName() );
}

//=============================================================================
// track properties list															27.03.2000 Adrian Meyer
//=============================================================================
void ViewTrackProperties( GtkWidget* aWidget, gpointer user_data)
{
  TMvcMain* main = ((TMvcMain*)user_data );
	main->fTrackProperties = new TMvcTrackProperties( main->GetSequence()->GetTrack( main->GetLastTrack() ), main->GetMapper() );
	main->fTrackProperties->LoadLanguageFile( main->GetLanguageFileName() );
	main->SetData();	
}

//=============================================================================
// click event properties clicked											04.04.2000 Adrian Meyer
//=============================================================================
void ViewEventPropertiesMain( GtkWidget* aWidget, gpointer user_data )
{
//TMvcMain* main = ((TMvcMain*)user_data );
	cout << "view event properties\n";
}

//=============================================================================
// view midi device options
// 05.03.2001 Adrian Meyer
//=============================================================================
void ViewMidiDeviceOptions( GtkWidget* aWidget, gpointer user_data)
{
  TMvcMain* main = ((TMvcMain*)user_data);
	main->fMapperProperties = new TMvcMapperProperties( main->GetMapper() );
	main->fMapperProperties->LoadLanguageFile( main->GetLanguageFileName() );
}

//=============================================================================
// change the main view																29.03.2000 Adrian MEyer
//=============================================================================
void ViewChanged( GtkWidget* aWidget, GtkNotebookPage *page,
										gint page_num, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data );
	main->SetCurrentView( page_num * 100 );
}

//=============================================================================
// hook for message from midi classes
// 28.08.2000 Adrian Meyer
//=============================================================================
void MCMessageHook( TMidiBase* aMidiBase, sMCMessage* aMCMessage )
{
  TMvcMain* main = ((TMvcMain*)aMCMessage->UserData );
	main->MCMessage( aMidiBase, aMCMessage );						// forward message to main window
}

//=============================================================================
// new eindow size
// 11.09.2000 Adrian Meyer
//=============================================================================
bool SizeRequest( GtkWidget *widget, GdkEventConfigure *event, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data );
	main->Resize();
	return true;
}

//=============================================================================
// go to top of sequence
// 24.11.2000 Adrian Meyer
//=============================================================================
void PlayTop( GtkWidget* aWidget, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data );
  main->GetSequence()->SetPosition( 0 );
}

//=============================================================================
// go to end of sequence
// 24.11.2000 Adrian Meyer
//=============================================================================
void PlayEnd( GtkWidget* aWidget, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data );
  main->GetSequence()->SetPosition( main->GetSequence()->GetMaxOffset() );
}

//=============================================================================
// start or stop the sequence
// 12.02.2001 Adrian Meyer
//=============================================================================
void PlayPlay( GtkWidget* aWidget, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data );
  main->Play( aWidget );
}

//=============================================================================
// dump language file
// 27.02.2001 Adrian Meyer
//=============================================================================
void LanguageDumpClicked( GtkWidget* aWidget, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data );
	main->DumpLanguage( "midimountain_en.lang" );
}

//=============================================================================
// dump language file
// 29.03.2001 Adrian Meyer
//=============================================================================
void LanguageMenuClicked( GtkWidget* aWidget, gpointer user_data )
{
  TMvcMain* main = ((TMvcMain*)user_data );
	main->BuildLanguageMenu();
}

//=============================================================================
// build language menu
// 29.03.2001 Adrian Meyer
//=============================================================================
void TMvcMain::BuildLanguageMenu( void )
{
  char* tmpPath = new char[2000];
	sprintf( tmpPath, "%s", GetIniSetting( fConfFile, "General", "Languages", "./languages" ).c_str() );

	if ( fLanguageMenuList != NULL )                    // delete old menu
  	for ( int i=g_slist_length( fLanguageMenuList	)-1; i>=0 ; i-- )
  	{
			gtk_widget_destroy( GTK_WIDGET( g_slist_nth_data( fLanguageMenuList, i )));			
  	}
	fLanguageMenuList = NULL;														// reset menu list

  DIR* dirp = opendir( tmpPath );											// open dir
  dirent* dp;
  while ( (dp = readdir( dirp )) != NULL )
  {
    if ( strncasecmp( dp->d_name + strlen( dp->d_name ) - 5, ".LANG", 5 ) == 0 )
		{
      char* tmpFileName = new char[ strlen( tmpPath ) + strlen( dp->d_name ) + 2 ];
			sprintf( tmpFileName, "%s/%s", tmpPath, dp->d_name );

			INIFile tmpIni = LoadIni( tmpFileName );				// open language file

			if ( GetIniSetting( tmpIni, "General", "Language", "" ) == "" )
  		{
        cout << "Language file error: filename: " << tmpFileName << endl;
        cout << "Language file error: Section [General] with item Language=<language> missing or empty." << endl;
			}
			else
			{                                               // everything cool!
																											// create menu item
        GtkWidget* tmpMenu = gtk_menu_item_new_with_label (
                               GetIniSetting( tmpIni, "General", "Language", "" ).c_str() );
        fLanguageMenuList = g_slist_append( fLanguageMenuList, tmpMenu );
        gtk_widget_set_name( tmpMenu, tmpFileName );
        gtk_widget_ref ( tmpMenu );                   // hang to menu
        gtk_object_set_data_full( GTK_OBJECT( wndMain ), tmpFileName, tmpMenu,
                                  (GtkDestroyNotify) gtk_widget_unref );
        gtk_widget_show( tmpMenu );										// add to menu
        gtk_container_add( GTK_CONTAINER( FindWidget( "mnuLanguage_menu" ) ), tmpMenu );

				gtk_signal_connect( GTK_OBJECT( tmpMenu ), "activate", (GtkSignalFunc)LanguageChanged, this );
			}

			delete tmpFileName;
		}
  }

  closedir( dirp );
}

//=============================================================================
// set other language
// 03.04.2001 Adrian Meyer
//=============================================================================          	
void TMvcMain::SetLanguage( char* aLanguageFileName )
{
  LoadLanguageFile( aLanguageFileName );

	fEventList->SetLanguageFileName( aLanguageFileName );
	fMarkerList->SetLanguageFileName( aLanguageFileName );
	fPianoRoll->SetLanguageFileName( aLanguageFileName );
	fTimeSignatureList->SetLanguageFileName( aLanguageFileName );
	fKeySignatureList->SetLanguageFileName( aLanguageFileName );
}

//=============================================================================
// language changed
// 03.04.2001 Adrian Meyer
//=============================================================================
void LanguageChanged( GtkWidget* aWidget, gpointer user_data)
{
  TMvcMain* main = ((TMvcMain*)user_data );
	main->SetLanguage( gtk_widget_get_name( aWidget ) );
}
