//=============================================================================
// model view control main view
//=============================================================================
#include <mvcMainView.h>

// callbacks
void TimeZoomChanged( GtkAdjustment *adjustment, gpointer user_data );
bool MainViewExposeHook( GtkWidget *widget, GdkEventExpose *event, gpointer user_data );

bool MainViewMouseDown( GtkWidget *widget, GdkEventButton *event, gpointer user_data);
bool MainViewMouseUp( GtkWidget *widget, GdkEventButton *event, gpointer user_data);
bool MainViewMouseMove( GtkWidget *widget, GdkEventMotion *event, gpointer user_data);

static short cXMaxMeasureCount = 20;
static short cXMinMeasureCount = 2;
static short cCountMeasureTail = 5;

//=============================================================================
// constructor																				
// 11.04.2000 Adrian Meyer
//=============================================================================
TMvcMainView::TMvcMainView( GtkWidget* aWidget, TMidiSequence* aSequence )
{
	fWidget = aWidget;
	AddChildrenToWidgetList( aWidget );                 // fetch widgets
	fSequence = aSequence;															// take sequence
	fWidth = 0;
	fHeight = 0;
	fTicksPerPix = 0;
	fSelectionActive = false;
	fWindowWidth = 0;
	fBufferMainView = 0;
	fHeader = new TMvcHeader ( FindWidget( "drwMainViewHeader" ), fSequence,
		gtk_scrolled_window_get_hadjustment( GTK_SCROLLED_WINDOW( FindWidget( "scrMainView" ))));

	gtk_signal_connect(	GTK_OBJECT( gtk_range_get_adjustment( GTK_RANGE(
			FindWidget(	"sclMainViewTimeZoom" )))),	"value-changed",
				GTK_SIGNAL_FUNC( TimeZoomChanged ),	this );

  HookSignal( "drwMainView", "expose_event", GTK_SIGNAL_FUNC( MainViewExposeHook ), this );

  HookSignal( "drwMainView", "button_press_event", GTK_SIGNAL_FUNC( MainViewMouseDown ), this );
  HookSignal( "drwMainView", "button_release_event",GTK_SIGNAL_FUNC( MainViewMouseUp ), this );
  HookSignal( "drwMainView", "motion_notify_event", GTK_SIGNAL_FUNC( MainViewMouseMove ), this );	

                                                      // activate events
	gtk_widget_add_events(
		FindWidget( "drwMainView" ),
		GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK );

	gtk_scrolled_window_set_hadjustment(								// synchronize main and header area
		GTK_SCROLLED_WINDOW( FindWidget( "scrMainViewHeader" )),
		gtk_scrolled_window_get_hadjustment( GTK_SCROLLED_WINDOW( FindWidget( "scrMainView" ))));

	colBackground = new GdkColor;
	colBackground->red = 65535;	colBackground->green = 65535;	colBackground->blue = 65535;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( FindWidget( "drwMainView" )),	
		colBackground, true, true );

	colGrid = new GdkColor;
	colGrid->red = 10000;	colGrid->green = 10000;	colGrid->blue = 10000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( FindWidget( "drwMainView" )),	
		colGrid, true, true );

	colSubGrid = new GdkColor;
	colSubGrid->red = 55000;	colSubGrid->green = 55000;	colSubGrid->blue = 55000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( FindWidget( "drwMainView" )),	
		colSubGrid, true, true );

	colMidi = new GdkColor;
	colMidi->red = 0;	colMidi->green = 0;	colMidi->blue = 32000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( FindWidget( "drwMainView" )),	
		colMidi, true, true );

	colSelection = new GdkColor;
	colSelection->red = colBackground->red - 65535;
	colSelection->green = colBackground->green - 55000;
	colSelection->blue = colBackground->blue - 55000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( FindWidget( "drwMainView" )),	
		colSelection, true, true );

	Resize( );
}

//=============================================================================
// message from midi classes
// 01.09.2000 Adrian Meyer
//=============================================================================
void TMvcMainView::MCMessage( TMidiBase* aMidiBase, sMCMessage* aMCMessage )
{
	switch ( aMCMessage->Message )
	{
		case cMMSequenceAddTrack:
		case cMMSequenceOpenFile:
		case cMMSequenceNew:
				Resize();
			break;

		case cMMSequenceSetSelectionStart:
		case cMMSequenceSetSelectionEnd:
				gtk_widget_queue_draw( FindWidget( "drwMainView" ));
			break;

	}

	if ( fHeader != 0 ) { fHeader->MCMessage( aMidiBase, aMCMessage );}
}

//=============================================================================
// read widget																				11.04.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcMainView::GetWidget( void )
{
	return fWidget;
}

//=============================================================================
// set new with and height
// 05.09.2000 Adrian Meyer
//=============================================================================
void TMvcMainView::Resize( void )
{
	long m; long b; long t;
	fSequence->GetMeasureBeatTick( fSequence->GetMaxOffset(), &m, &b, &t );
	long maxOffset = fSequence->GetOffset( m + 1, 1, 0 ) - 1 + fSequence->GetOffset( cCountMeasureTail + 1, 1, 0 ) - 1;
	long maxOffset20 = fSequence->GetOffset( cXMaxMeasureCount + 1, 1, 0 ) - 1;
	if ( maxOffset < maxOffset20 )											// set measure count
	{
		fMeasureCount = cXMaxMeasureCount;                // correct to max
	}
	else
	{
		long tmpBeat = 0;	long tmpTick = 0;								// more than max
		fSequence->GetMeasureBeatTick( fSequence->GetMaxOffset(),
			&fMeasureCount, &tmpBeat, &tmpTick );		
	}

	maxOffset = 0;
	for ( int i=1; i<=fMeasureCount; i++ )
	{
		long tmpBeatCount; long tmpTicksPerBeat;
  	fSequence->GetTimeInformation( i, &tmpBeatCount, &tmpTicksPerBeat );
		maxOffset += tmpBeatCount * tmpTicksPerBeat;
	}	

	long minOffset = fSequence->GetOffset( cXMinMeasureCount + 1, 1, 0 ) - 1;
  long tmpZoomFactor = (int)gtk_range_get_adjustment( GTK_RANGE( FindWidget(	"sclMainViewTimeZoom" )))->value;

																											// width of viewable area
	gdk_window_get_size( FindWidget( "vpoMainViewDrawingArea" )->window, &fWindowWidth, &fHeight );

	fWindowWidth -= 5; fHeight -= 4;										// correct border width

	if ( (fSequence->GetTrackCount() - 1 ) * 16 > fHeight )
		{ fHeight = (fSequence->GetTrackCount() - 1 ) * 16; }
	
																											// calculate tix in window
	float tmpTicksInWindow = (( maxOffset20 - minOffset) / 99.00) * (tmpZoomFactor-1) + minOffset;
  fTicksPerPix = tmpTicksInWindow / fWindowWidth;			// get tixperpix
	fWidth = (int)(maxOffset / fTicksPerPix);

	gtk_drawing_area_size( GTK_DRAWING_AREA( FindWidget( "drwMainView" )), fWidth, fHeight );

	if ( fBufferMainView != 0 )			  { gdk_pixmap_unref( fBufferMainView ); }

	fBufferMainView = gdk_pixmap_new( FindWidget( "drwMainView" )->window,	fWidth, fHeight, -1 );;

																											// resize header
	if ( fHeader != 0 )	
	{	fHeader->Resize( fWidth, fTicksPerPix, fWindowWidth, fMeasureCount + cCountMeasureTail ); }
}

//=============================================================================
// redraw area
// 05.09.2000 Adrian Meyer
//=============================================================================
bool TMvcMainView::MainViewExpose( GtkWidget* aWidget, GdkEventExpose* aEvent )
{
	GdkGC* tmpGC = gdk_gc_new( aWidget->window );				// new graphic context
	gdk_gc_copy( tmpGC, aWidget->style->fg_gc[aWidget->state] );

	GdkDrawable* tmpWnd = aWidget->window;							// window to draw to

	fWidth = aWidget->allocation.width - 1;							// width
	fHeight = aWidget->allocation.height - 1;						// height

	long tmpFromX = aEvent->area.x - 100;
	long tmpToX = aEvent->area.width + aEvent->area.x + 100;

	gdk_gc_set_foreground( tmpGC, colBackground );			// draw background
	gdk_draw_rectangle( fBufferMainView, tmpGC, TRUE,	1, 1, fWidth, fHeight );

	gdk_gc_set_foreground( tmpGC, colGrid );						// draw grid
	gdk_gc_set_line_attributes( tmpGC, 0, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER );

  for ( int i = 0; i<fSequence->GetTrackCount(); i++ )
		{	gdk_draw_line( fBufferMainView, tmpGC, 1, i*16, fWidth, i*16 ); }

	for ( long i=1; i<=fMeasureCount + cCountMeasureTail; i++ )
	{
		long tmpBeatCount;
		long tmpTicksPerBeat;
		long x = (long)(fSequence->GetOffset( i, 1, 0 ) / fTicksPerPix);

		fSequence->GetTimeInformation( i, &tmpBeatCount, &tmpTicksPerBeat );

		for ( long j=0; j<tmpBeatCount; j++ )
		{
			long tmpx = (long)(x + (( j*tmpTicksPerBeat ) / fTicksPerPix ));
			
			if ( tmpx >= tmpFromX && tmpx <= tmpToX )
			{
  			gdk_gc_set_foreground( tmpGC, colSubGrid );						// draw grid
  			gdk_draw_line( fBufferMainView, tmpGC, tmpx, 1, tmpx, fHeight );
			}
		}

		if ( x >= tmpFromX && x <= tmpToX )
		{
  		gdk_gc_set_foreground( tmpGC, colGrid );						// draw grid
  		gdk_draw_line( fBufferMainView, tmpGC, x, 1, x, fHeight );
		}
	}

	gdk_gc_set_line_attributes( tmpGC, 1, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER );
	gdk_gc_set_foreground( tmpGC, colMidi );

	if ( fSequence->GetTrackCount() > 0 )
	{
		for ( long i=1; i<fSequence->GetTrackCount(); i++ )
		{
			if ( fSequence->GetTrack( i )->GetEventCount() > 0 )
			{
				TMidiEvent* tmpEvent = fSequence->GetTrack( i )->GetEvent(0);
				while (	tmpEvent != 0 )
				{
					long cSpace = 2;
					long tmpx1 = (long)(tmpEvent->GetOffset() / fTicksPerPix);
					long tmpx2 = cSpace;

					if (((( tmpEvent->GetData1()) & 0xF0 ) == cESNoteOn ) & !(tmpEvent->GetNoteOff() ))
					{
						TMidiEvent* tmpEventRef = tmpEvent->Ref;
						if ( tmpEventRef != 0 )
							{ tmpx2 = (long)(tmpEventRef->GetOffset() / fTicksPerPix) - tmpx1;	};

						if ( tmpx1 >= tmpFromX && tmpx1 <= tmpToX )
						{
							gdk_draw_rectangle( fBufferMainView, tmpGC, TRUE,	
								tmpx1, (i-1)*16 + cSpace , tmpx2 - cSpace , 16 - 2 * cSpace );
						}
					}
		
					tmpEvent = (TMidiEvent*)tmpEvent->Next;
				}
			}			
		}
	}

	gdk_gc_set_function( tmpGC, GDK_XOR );
	gdk_gc_set_foreground( tmpGC, colSelection );				// draw selection
	gdk_draw_rectangle( fBufferMainView, tmpGC, TRUE,	
		(int)( fSequence->GetSelectionStart() / fTicksPerPix ), 1,
		(int)(( fSequence->GetSelectionEnd() / fTicksPerPix ) -
			( fSequence->GetSelectionStart() / fTicksPerPix )), fHeight );	

	gdk_gc_set_function( tmpGC, GDK_COPY );
	gdk_gc_set_clip_rectangle( tmpGC, &aEvent->area );	// drawable selection
	gdk_draw_pixmap( tmpWnd, tmpGC, fBufferMainView, 0, 0, 0, 0, fWidth, fHeight );
	gdk_gc_set_clip_rectangle( tmpGC,	NULL );           // go!
	gdk_gc_destroy( tmpGC );

	return TRUE;
}

//=============================================================================
// mouse down in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
void TMvcMainView::MouseDown( long x )
{
	fSelectionActive = true;
	fSequence->SetSelectionStart( GetOffsetOnGrid( x * fTicksPerPix, true ));
}

//=============================================================================
// mouse up in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
void TMvcMainView::MouseUp( long x )
{
	fSelectionActive = false;
	fSequence->SetSelectionEnd( GetOffsetOnGrid( x * fTicksPerPix, false ));
}

//=============================================================================
// mouse move in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
void TMvcMainView::MouseMove( long x )
{
	if ( fSelectionActive )
	{
																											// end position on grid
		int tmpSelectionEnd = GetOffsetOnGrid( x * fTicksPerPix, false );						
		if ( tmpSelectionEnd > fWidth * fTicksPerPix )		// more than max?
			{	tmpSelectionEnd = (int)(fWidth * fTicksPerPix); }	// correct it!

		if ( tmpSelectionEnd >= fSequence->GetSelectionStart() )
		{
			fSequence->SetSelectionEnd( tmpSelectionEnd );		// set selection end of sequence
		}

																											// scroll window?	
		GtkAdjustment* tmpAdj = gtk_scrolled_window_get_hadjustment( GTK_SCROLLED_WINDOW( FindWidget( "scrMainView" )));

																											// in direction x ->
		if ( tmpAdj->value < ( x - fWindowWidth ) && tmpAdj->value < fWidth - fWindowWidth )
		{
																											// increase value by 10th of mouse position
			tmpAdj->value += ( x - ( tmpAdj->value + fWindowWidth )) / 5 ;
			if ( tmpAdj->value > fWidth - fWindowWidth )		// if more than max
				{ tmpAdj->value = fWidth - fWindowWidth; }		// correct to max
			gtk_adjustment_value_changed( tmpAdj );					// set new position in x adjustment
		}

																											// in direction x <
		if ( tmpAdj->value > x && tmpAdj->value > 0 )
		{
			tmpAdj->value -= (tmpAdj->value - x) / 5;

			if ( tmpAdj->value < 0 )												// if less than 0
				{ tmpAdj->value = 0; }												// correct to 0
			gtk_adjustment_value_changed( tmpAdj );					// set new position in x adjustment
		}
	}
}

//=============================================================================
// read offset on grid
// 09.09.2000 Adrian Meyer
//=============================================================================
long TMvcMainView::GetOffsetOnGrid( long aOffset, bool aRoundDown )
{
	long tmpMeasure; long tmpBeat; long tmpTick;
	long tmpBeatCount; long tmpTicksPerBeat;

	if (gtk_toggle_button_get_active( (GtkToggleButton*)FindWidget( "rboMainViewMeasure" )))
	{
		fSequence->GetMeasureBeatTick( aOffset, &tmpMeasure, &tmpBeat, &tmpTick );
		if ( aRoundDown )																	// round down
		{	return fSequence->GetOffset( tmpMeasure, 1, 0 ); }
		else																							// round up
		{	return fSequence->GetOffset( tmpMeasure + 1, 1, 0 ) - 1; }
	}	

	if (gtk_toggle_button_get_active( (GtkToggleButton*)FindWidget( "rboMainViewBeat" )))
	{
		fSequence->GetMeasureBeatTick( aOffset, &tmpMeasure, &tmpBeat, &tmpTick );
		if ( aRoundDown )																	// round down
		{
  		fSequence->GetMeasureBeatTick( aOffset, &tmpMeasure, &tmpBeat, &tmpTick );
  		fSequence->GetTimeInformation( tmpMeasure, &tmpBeatCount, &tmpTicksPerBeat );
			return fSequence->GetOffset( tmpMeasure, tmpBeat, 0 );
		}
		else
		{
  		fSequence->GetMeasureBeatTick( aOffset, &tmpMeasure, &tmpBeat, &tmpTick );
  		fSequence->GetTimeInformation( tmpMeasure, &tmpBeatCount, &tmpTicksPerBeat );
			return fSequence->GetOffset( tmpMeasure, tmpBeat, tmpTicksPerBeat - 1 );
		}
	}	

	return aOffset;
}

//=============================================================================
// time zoom changed
// 05.09.2000 Adrian Meyer
//=============================================================================
void TimeZoomChanged( GtkAdjustment *adjustment, gpointer user_data )
{
	TMvcMainView* tmpMvcMainView = (TMvcMainView*)user_data;
  tmpMvcMainView->Resize();
}

//=============================================================================
// redraw area
// 05.09.2000 Adrian Meyer
//=============================================================================
bool MainViewExposeHook( GtkWidget *widget, GdkEventExpose *event, gpointer user_data )
{
	TMvcMainView* tmpMvcMainView = (TMvcMainView*)user_data;
  return tmpMvcMainView->MainViewExpose( widget, event );
}

//=============================================================================
// size change of area
// 05.09.2000 Adrian Meyer
//=============================================================================
void MainViewSizeRequest( GtkWidget *widget, GtkRequisition *requisition, gpointer user_data )
{
	TMvcMainView* tmpMvcMainView = (TMvcMainView*)user_data;
  tmpMvcMainView->Resize();
}

//=============================================================================
// mouse down in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
bool MainViewMouseDown( GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	TMvcMainView* tmpMvcMainView = (TMvcMainView*)user_data;
	tmpMvcMainView->MouseDown( event->x );
	return true;
}

//=============================================================================
// mouse up in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
bool MainViewMouseUp( GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	TMvcMainView* tmpMvcMainView = (TMvcMainView*)user_data;
	tmpMvcMainView->MouseUp( event->x );
	return true;
}

//=============================================================================
// mouse move in main view
// 09.09.2000 Adrian Meyer
//=============================================================================
bool MainViewMouseMove( GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
{
	TMvcMainView* tmpMvcMainView = (TMvcMainView*)user_data;
	tmpMvcMainView->MouseMove( event->x );
	return true;
}
