//=============================================================================
//  model view control setup midi devices
//=============================================================================
#include <mvcMapperProperties.h>
#include <stdio.h>

void CloseClickedMapperProperties( GtkButton *button, gpointer user_data );
void AddPresetClickedMapperProperties( GtkButton *button, gpointer user_data );
void DeletePresetClickedMapperProperties( GtkButton *button, gpointer user_data );
void SelectPresetMapperProperties( GtkEntry *entry, gpointer user_data );

//=============================================================================
// constructor
// 20.07.2001 Adrian Meyer
//=============================================================================
TMvcMapperProperties::TMvcMapperProperties( TMidiMapper* aMapper )
{
  wndMapperProperties = create_wndMapperProperties();
	AddChildrenToWidgetList( wndMapperProperties );   // fetch widgets

  HookSignal( "btnClose", "clicked", GTK_SIGNAL_FUNC( CloseClickedMapperProperties ), this );
  HookSignal( "btnPresetAdd", "clicked", GTK_SIGNAL_FUNC( AddPresetClickedMapperProperties ), this );
  HookSignal( "btnPresetDelete", "clicked", GTK_SIGNAL_FUNC( DeletePresetClickedMapperProperties ), this );
  HookSignal( "edtPresetName", "changed", GTK_SIGNAL_FUNC( SelectPresetMapperProperties ), this );

  fMapper = aMapper;									             	  // init values

	fPresetNames = 0;

  for (int i=0; i<fMapper->GetPresetCount(); i++ )
  {
    char* tmpItem = new char[ strlen( fMapper->GetPresetName(i) ) + 1 ];
    strcpy( tmpItem, fMapper->GetPresetName(i) );
  	fPresetNames = g_list_append( fPresetNames, tmpItem );
  }

  gtk_widget_show( wndMapperProperties );							
  SetData();
  ValidateGui();
}

//=============================================================================
// destructor
// 20.02.2001 Adrian Meyer
//=============================================================================
TMvcMapperProperties::~TMvcMapperProperties( )
{
	delete wndMapperProperties;                       // remove window
}

//=============================================================================
// read widget
// 20.02.2001 Adrian Meyer
//=============================================================================
GtkWidget* TMvcMapperProperties::GetWidget( void )
{
  return wndMapperProperties;                       // return widget
}

//=============================================================================
// set data to gui
// 21.02.2001 Adrian Meyer
//=============================================================================
void TMvcMapperProperties::SetData( void )
{
	gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboPresetName" )), fPresetNames );	
	gtk_entry_set_text( GTK_ENTRY( FindWidget( "edtPresetName" )),	
    fMapper->GetPresetName( fMapper->GetPresetID() ));

	GtkCList* tmpInputDevicesCList = GTK_CLIST( FindWidget( "grdInputDevices" ));
	GtkCList* tmpOutputDevicesCList = GTK_CLIST( FindWidget( "grdOutputDevices" ));

	gtk_clist_freeze( tmpInputDevicesCList);            // freeze lists
	gtk_clist_freeze( tmpOutputDevicesCList);

	gtk_clist_clear( tmpInputDevicesCList);             // clear them
	gtk_clist_clear( tmpOutputDevicesCList);

	char* tmpList[1][3] = {{"","",""}};				          // empty row
  char tmpStr[10];

  // add input devices
  for ( int i=0; i<fMapper->GetInputDeviceCount(); i++ )
  {
   	gtk_clist_append( tmpInputDevicesCList, tmpList[0] );
    gtk_clist_set_text(	tmpInputDevicesCList, i, 0, fMapper->GetInputDevice(i)->GetName() );
    gtk_clist_set_text(	tmpInputDevicesCList, i, 1, fMapper->GetInputDevice(i)->GetDevice() );
    sprintf( tmpStr, "%d", fMapper->GetInputDevice(i)->GetPort() );
    gtk_clist_set_text(	tmpInputDevicesCList, i, 2, tmpStr );
  }

  // add output devices
  for ( int i=0; i<fMapper->GetOutputDeviceCount(); i++ )
  {
   	gtk_clist_append( tmpOutputDevicesCList, tmpList[0] );
    gtk_clist_set_text(	tmpOutputDevicesCList, i, 0, fMapper->GetOutputDevice(i)->GetName() );
    gtk_clist_set_text(	tmpOutputDevicesCList, i, 1, fMapper->GetOutputDevice(i)->GetDevice() );
    sprintf( tmpStr, "%d", fMapper->GetOutputDevice(i)->GetPort() );
    gtk_clist_set_text(	tmpOutputDevicesCList, i, 2, tmpStr );
  }

	gtk_clist_thaw( tmpInputDevicesCList );             // update lists
	gtk_clist_thaw( tmpOutputDevicesCList );

}

//=============================================================================
// read data from gui
// 21.02.2001 Adrian Meyer
//=============================================================================
void TMvcMapperProperties::GetData( void )
{
}

//=============================================================================
// OK clicked
// 20.02.2001 Adrian Meyer
//=============================================================================
void CloseClickedMapperProperties( GtkButton *button, gpointer user_data )
{
  (( TMvcMapperProperties* )user_data)->GetData();
	gtk_widget_hide( ((TMvcMapperProperties*)user_data)->GetWidget() );
}

//=============================================================================
// read selected preset name
// 06.09.2001 Adrian Meyer
//=============================================================================
char* TMvcMapperProperties::GetSelectedPreset( void )
{
  return gtk_entry_get_text( GTK_ENTRY( FindWidget( "edtPresetName" )));
}

//=============================================================================
// validate buttons on gui
// 06.09.2001 Adrian Meyer
//=============================================================================
void TMvcMapperProperties::ValidateGui( void )
{
  bool tmpPreset = ( g_list_length( fPresetNames ) > 1 ) &&
    ( strcmp( GetSelectedPreset(), cDefaultPresetName ) != 0 );

	gtk_widget_set_sensitive( FindWidget( "btnPresetDelete" ), tmpPreset );
	gtk_widget_set_sensitive( FindWidget( "btnPresetProperties" ), tmpPreset );
}

//=============================================================================
// add a preset
// 06.09.2001 Adrian Meyer
//=============================================================================
void AddPresetClickedMapperProperties( GtkButton *button, gpointer user_data )
{
  (( TMvcMapperProperties* )user_data)->AddPreset();
}

//=============================================================================
// select a preset
// 06.09.2001 Adrian Meyer
//=============================================================================
void SelectPresetMapperProperties( GtkEntry *entry, gpointer user_data )
{
  (( TMvcMapperProperties* )user_data)->SelectPreset();
}

//=============================================================================
// delete a preset
// 06.09.2001 Adrian Meyer
//=============================================================================
void DeletePresetClickedMapperProperties( GtkButton *button, gpointer user_data )
{
  (( TMvcMapperProperties* )user_data)->DeletePreset();
}

//=============================================================================
// add a preset
// 06.09.2001 Adrian Meyer
//=============================================================================
void TMvcMapperProperties::AddPreset( void )
{
  TMvcInputBox tmpInputBox;
  if ( tmpInputBox.Show( "Add New Preset", "Preset Name", "My Preset" ) == mrOK )
  {
    char* tmpStr = new char[ strlen( tmpInputBox.GetText() ) + 1 ];
    strcpy( tmpStr, tmpInputBox.GetText() );
  	fPresetNames = g_list_append( fPresetNames, tmpStr );
    fMapper->ResetToDefault( );
    SetData();
    ValidateGui();
  }
}

//=============================================================================
// delete a preset
// 06.09.2001 Adrian Meyer
//=============================================================================
void TMvcMapperProperties::DeletePreset( void )
{
  cout << "delete preset\n";
}

//=============================================================================
// select a preset
// 06.09.2001 Adrian Meyer
//=============================================================================
void TMvcMapperProperties::SelectPreset( void )
{
  // set new preset if different
  for (unsigned int i=0; i<g_list_length( fPresetNames );i++)
  {
    if ( strcmp( (char*)g_list_nth( fPresetNames, i ), GetSelectedPreset()) == 0 )
    {
      fMapper->LoadPreset( i );
      ValidateGui();
    }
  }
}