//=============================================================================
// model view control marker list
//=============================================================================
#include <mvcMarkerList.h>

// callbacks
void MarkerListSelectRow( GtkWidget* aWidget,gint row,gint column,GdkEventButton *event,gpointer user_data);
void MarkerListDeleteClicked( GtkWidget* aWidget, gpointer user_data);
void MarkerListAddClicked( GtkWidget* aWidget, gpointer user_data);
void ViewMarkerProperties( GtkWidget* aWidget, gpointer user_data );

//=============================================================================
// constructor
// 11.09.2000 Adrian Meyer
//=============================================================================
TMvcMarkerList::TMvcMarkerList( GtkWidget* aWidget, TMidiSequence* aSequence )
{
	fSequence = aSequence;
	fWidget = aWidget;
	AddChildrenToWidgetList( aWidget );                 // fetch widgets
	fMCMessageFlag = true;
	fSelectedMarker = 0;

	HookSignal( "grdMarkerList", "select-row", GTK_SIGNAL_FUNC( MarkerListSelectRow ), this );
  HookSignal( "btnMarkerListAdd", "clicked", GTK_SIGNAL_FUNC( MarkerListAddClicked ), this );
  HookSignal( "btnMarkerListDelete", "clicked", GTK_SIGNAL_FUNC( MarkerListDeleteClicked ), this );
  HookSignal( "btnMarkerListProperties", "clicked", GTK_SIGNAL_FUNC( ViewMarkerProperties ), this );

	ValidateGui();																			// set gui
}

//=============================================================================
// read widget
// 11.09.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcMarkerList::GetWidget( void )
{
	return fWidget;
}

//=============================================================================
// midi class messages
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerList::MCMessage( TMidiBase* aMidiBase, sMCMessage* aMCMessage )
{
  switch ( aMCMessage->Message )
 	{
 		case cMMSequenceOpenFile:
 		case cMMSequenceNew:															
 		case cMMSequenceAddMarker:
 		case cMMSequenceDeleteMarker:
 		case cMMSequenceRenameMarker:
 		case cMMSequenceSortMarker:
 				fMCMessageFlag = true;												// react to changes
 				SetData();
 			break;
 	}

	ValidateGui();
}

//=============================================================================
// set data
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerList::SetData( void )
{
	GtkCList* tmpCList = GTK_CLIST( FindWidget( "grdMarkerList" ));
	gtk_clist_freeze( tmpCList );
	gtk_clist_clear( tmpCList );
	char* tmpList[1][3] = {{"","",""}};									// empty row

	for ( long i = 0; i<fSequence->GetMarkerCount(); i++ )
	{
 		gtk_clist_append( tmpCList, tmpList[0] );
		gtk_clist_set_row_data( tmpCList,	i, fSequence->GetMarker(i) );
		SetRowData( i );
		if ( fSequence->GetMarker(i) == fSelectedMarker )		// reselect marker
			{	gtk_clist_select_row( tmpCList,	i, 0 ); }
	}	

	gtk_clist_thaw( tmpCList );

	fMCMessageFlag = true;															// react to changes
	ValidateGui();
}

//=============================================================================
// set data of a row
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerList::SetRowData( int aRow )
{
 	gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdMarkerList" )),
 		aRow, 1, fSequence->GetMeasureBeatTick( fSequence->GetMarker(aRow)->Offset ) );

 	gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdMarkerList" )),
 		aRow, 2, fSequence->GetMarker(aRow)->GetName() );
}

//=============================================================================
// write selected row
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerList::SetSelectedRow( int aRow )
{
	fSelectedMarker = fSequence->GetMarker( aRow );
}

//=============================================================================
// validate gui
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerList::ValidateGui( void )
{
	if ( fMCMessageFlag )
	{
    gtk_widget_set_sensitive( FindWidget( "btnMarkerListProperties" ), ( fSequence->GetMarkerCount() > 0 ) );
    gtk_widget_set_sensitive( FindWidget( "btnMarkerListDelete" ), ( fSequence->GetMarkerCount() > 0 ) );
		fMCMessageFlag = false;
	}
}

//=============================================================================
// add new marker
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerList::AddMarker( void )
{
	TMidiMarker* tmpMarker = fSequence->AddMarker( fSequence->GetPosition(), "unnamed" );
	for ( int i = 0; i<fSequence->GetMarkerCount(); i++ )
	{
		if ( tmpMarker == fSequence->GetMarker(i))
		gtk_clist_select_row( GTK_CLIST( FindWidget( "grdMarkerList" )), i, 0 );
	}	
	ShowMarkerProperties();
}

//=============================================================================
// delete selected marker
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerList::DeleteMarker( void )
{
	if ( fSelectedMarker != 0 )
	{
  	fSequence->DeleteMarker( fSelectedMarker );
	}
}

//=============================================================================
// show marker properties
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerList::ShowMarkerProperties( void )
{
	if ( fSelectedMarker != 0 )
	{
		fMarkerProperties = new TMvcMarkerProperties( fSequence, fSelectedMarker );
    fMarkerProperties->LoadLanguageFile( GetLanguageFileName() );
	}
}

//=============================================================================
// row is selected
// 11.09.2000 Adrian Meyer
//=============================================================================
void MarkerListSelectRow( GtkWidget* aWidget,gint row,gint column,GdkEventButton *event,gpointer user_data)
{
	((TMvcMarkerList*)user_data)->SetSelectedRow( row );
}

//=============================================================================
// add marker
// 11.09.2000 Adrian Meyer
//=============================================================================
void MarkerListAddClicked( GtkWidget* aWidget, gpointer user_data)
{
	((TMvcMarkerList*)user_data)->AddMarker( );
}

//=============================================================================
// delete marker
// 11.09.2000 Adrian Meyer
//=============================================================================
void MarkerListDeleteClicked( GtkWidget* aWidget, gpointer user_data)
{
	((TMvcMarkerList*)user_data)->DeleteMarker( );
}

//=============================================================================
// edit marker properties
// 11.09.2000 Adrian Meyer
//=============================================================================
void ViewMarkerProperties( GtkWidget* aWidget, gpointer user_data )
{
	((TMvcMarkerList*)user_data)->ShowMarkerProperties( );
}
