//=============================================================================
//  model view control marker properties
//=============================================================================
#include <mvcMarkerProperties.h>

// callbacks
void OkClickedMarkerProp( GtkButton *button, gpointer user_data );
void CancelClickedMarkerProp( GtkButton *button, gpointer user_data );
void MeasureChangedMarkerProp( GtkEditable *editable, gpointer user_data );

//=============================================================================
// construtor MvcMarkerProperties											
// 11.09.2000 Adrian Meyer
//=============================================================================
TMvcMarkerProperties::TMvcMarkerProperties( TMidiSequence* aSequence, TMidiMarker* aMarker )
{
  wndMarkerProperties = create_wndMarkerProperties();	// call glade
	AddChildrenToWidgetList( wndMarkerProperties );     // fetch widgets
	fSequence = aSequence;
	fMarker = aMarker;
	
  HookSignal( "btnOk", "clicked", GTK_SIGNAL_FUNC( OkClickedMarkerProp ), this );
  HookSignal( "btnCancel", "clicked", GTK_SIGNAL_FUNC( CancelClickedMarkerProp ), this );
  HookSignal( "spiMeasure", "changed", GTK_SIGNAL_FUNC( MeasureChangedMarkerProp ), this );

	if ( aSequence == 0 ) return;
  gtk_widget_show( wndMarkerProperties );							

	SetData();																					// set data to gui
}

//=============================================================================
// destrutor MvcMain			
// 11.09.2000 Adrian Meyer
//=============================================================================
TMvcMarkerProperties::~TMvcMarkerProperties( void )
{
	delete wndMarkerProperties;
}

//=============================================================================
// set data to gui				
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerProperties::SetData( void )
{
 	long tmpMeasure = 0; long tmpBeat = 0; long tmpTick = 0;		

 	fSequence->GetMeasureBeatTick( fMarker->Offset, &tmpMeasure, &tmpBeat, &tmpTick );

 																										// set measure
   gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiMeasure" )), tmpMeasure );
   gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiBeat" )), tmpBeat );
   gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiTick" )), tmpTick );

	gtk_entry_set_text( GTK_ENTRY( FindWidget( "edtMarkerName" )), fMarker->GetName() );
}

//=============================================================================
// read data from gui into object											
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerProperties::GetData( void )
{
																											// rename marker
	fSequence->RenameMarker( fMarker, gtk_entry_get_text( GTK_ENTRY( FindWidget( "edtMarkerName" ))));								

  long tmpOffset =																		// new offset
		fSequence->GetOffset(
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiMeasure" ))),
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiBeat" ))),
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiTick" )))
    );

	fSequence->SetMarkerOffset( fMarker, tmpOffset );
}

//=============================================================================
// read widget						
// 11.09.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcMarkerProperties::GetWidget( void )
{
	return wndMarkerProperties;
}

//=============================================================================
// adjust beat/ticks to selected measure
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerProperties::AdjustToMeasure( long aMeasure )
{
	long tmpOffset = fSequence->GetOffset( aMeasure + 1, 0, 0 ) - 1;
	long tmpMeasure = 0;
	long tmpBeat = 0;
	long tmpTick = 0;

	fSequence->GetMeasureBeatTick( tmpOffset, &tmpMeasure, &tmpBeat, &tmpTick );

	gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiBeat" )))->upper = tmpBeat + 1;
	gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiTick" )))->upper = tmpTick;
}

//=============================================================================
// ok clicked																					
// 11.09.2000 Adrian Meyer
//=============================================================================
void OkClickedMarkerProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcMarkerProperties*)user_data)->GetWidget() );
	((TMvcMarkerProperties*)user_data)->GetData();
}

//=============================================================================
// cancel clicked																			
// 11.09.2000 Adrian Meyer
//=============================================================================
void CancelClickedMarkerProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcMarkerProperties*)user_data)->GetWidget() );
}

//=============================================================================
// measure changed
// 12.09.2000 Adrian Meyer
//============================================================================
void MeasureChangedMarkerProp( GtkEditable *editable, gpointer user_data )
{
	((TMvcMarkerProperties*)user_data)->AdjustToMeasure( gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON( editable )));		
}
