//=============================================================================
// model view control time signature list
//=============================================================================
#include <mvcTimeSignatureList.h>
#include <stdio.h>

// callbacks
void TimeSignatureListSelectRow( GtkWidget* aWidget,gint row,gint column,GdkEventButton *event,gpointer user_data);
void TimeSignatureListDeleteClicked( GtkWidget* aWidget, gpointer user_data);
void TimeSignatureListAddClicked( GtkWidget* aWidget, gpointer user_data);
void ViewTimeSignatureProperties( GtkWidget* aWidget, gpointer user_data );

//=============================================================================
// constructor
// 12.09.2000 Adrian Meyer
//=============================================================================
TMvcTimeSignatureList::TMvcTimeSignatureList( GtkWidget* aWidget, TMidiSequence* aSequence )
{
	fSequence = aSequence;
	fWidget = aWidget;
	AddChildrenToWidgetList( aWidget );                 // fetch widgets
	fMCMessageFlag = true;
	fSelectedTimeSignature = 0;

	HookSignal( "grdTimeSignatureList", "select-row", GTK_SIGNAL_FUNC( TimeSignatureListSelectRow ), this );
  HookSignal( "btnTimeSignatureListAdd", "clicked", GTK_SIGNAL_FUNC( TimeSignatureListAddClicked ), this );
  HookSignal( "btnTimeSignatureListDelete", "clicked", GTK_SIGNAL_FUNC( TimeSignatureListDeleteClicked ), this );
  HookSignal( "btnTimeSignatureListProperties", "clicked", GTK_SIGNAL_FUNC( ViewTimeSignatureProperties ), this );

	SetData();
	ValidateGui();																			// set gui
}

//=============================================================================
// read widget
// 12.09.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcTimeSignatureList::GetWidget( void )
{
	return fWidget;
}

//=============================================================================
// midi class messages
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcTimeSignatureList::MCMessage( TMidiBase* aMidiBase, sMCMessage* aMCMessage )
{
  switch ( aMCMessage->Message )
 	{
 		case cMMSequenceOpenFile:
 		case cMMSequenceNew:															
 		case cMMSequenceAddTimeSignature:
 		case cMMSequenceDeleteTimeSignature:
		case cMMSequenceChangeTimeSignature:
 		case cMMSequenceSortTimeSignature:
 				fMCMessageFlag = true;												// react to changes
 				SetData();
 			break;
 	}

	ValidateGui();
}

//=============================================================================
// set data
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcTimeSignatureList::SetData( void )
{
	GtkCList* tmpCList = GTK_CLIST( FindWidget( "grdTimeSignatureList" ));
	gtk_clist_freeze( tmpCList );
	gtk_clist_clear( tmpCList );
	char* tmpList[1][5] = {{"","","","",""}};						// empty row

	for ( long i = 0; i<fSequence->GetTimeSignatureCount(); i++ )
	{
 		gtk_clist_append( tmpCList, tmpList[0] );
		gtk_clist_set_row_data( tmpCList,	i, fSequence->GetTimeSignature(i) );
		SetRowData( i );
		if ( fSequence->GetTimeSignature(i) == fSelectedTimeSignature )		// reselect TimeSignature
			{	gtk_clist_select_row( tmpCList,	i, 0 ); }
	}	

	gtk_clist_thaw( tmpCList );

	fMCMessageFlag = true;															// react to changes
	ValidateGui();
}

//=============================================================================
// set data of a row
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcTimeSignatureList::SetRowData( int aRow )
{
 	gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdTimeSignatureList" )),
 		aRow, 1, fSequence->GetMeasureBeatTick( fSequence->GetTimeSignature(aRow)->Offset ) );

	char* tmpStr = new char[100];
	sprintf( tmpStr, "%d/%d", fSequence->GetTimeSignature(aRow)->Numerator, 1 << fSequence->GetTimeSignature(aRow)->Denominator );
 	gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdTimeSignatureList" )),	aRow, 2, tmpStr );

	sprintf( tmpStr, "%d", fSequence->GetTimeSignature(aRow)->MidiClocksPerQuarter );
 	gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdTimeSignatureList" )),	aRow, 3, tmpStr );

	sprintf( tmpStr, "%d", fSequence->GetTimeSignature(aRow)->NotesPerQuarter );
 	gtk_clist_set_text(	GTK_CLIST( FindWidget( "grdTimeSignatureList" )),	aRow, 4, tmpStr );

	delete tmpStr;
}

//=============================================================================
// write selected row
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcTimeSignatureList::SetSelectedRow( int aRow )
{
	fSelectedTimeSignature = fSequence->GetTimeSignature( aRow );
}

//=============================================================================
// validate gui
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcTimeSignatureList::ValidateGui( void )
{
	if ( fMCMessageFlag )
	{
    gtk_widget_set_sensitive( FindWidget( "btnTimeSignatureListDelete" ), ( fSequence->GetTimeSignatureCount() > 1 ) );
		fMCMessageFlag = false;
	}
}

//=============================================================================
// add new time signature
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcTimeSignatureList::AddTimeSignature( void )
{
	long tmpID = fSequence->AddTimeSignature( 4, 2, 24, 8, 0 );
	sTimeSignatureList* tmpTimeSignature = fSequence->GetTimeSignature( tmpID );
  fSequence->SortTimeSignature();
	for ( int i = 0; i<fSequence->GetTimeSignatureCount(); i++ )
	{
		if ( tmpTimeSignature == fSequence->GetTimeSignature(i))
		gtk_clist_select_row( GTK_CLIST( FindWidget( "grdTimeSignatureList" )), i, 0 );
	}	
	ShowTimeSignatureProperties();
}

//=============================================================================
// delete selected time signature
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcTimeSignatureList::DeleteTimeSignature( void )
{
	if ( fSelectedTimeSignature != 0 )
	{
		int oldID = fSelectedTimeSignature->ID;
		for ( int i=0; i<fSelectedTimeSignature->ID; i++ )
		{
			fSelectedTimeSignature = fSequence->GetTimeSignature(i);
		}

		if ( fSelectedTimeSignature == 0)
		  { fSelectedTimeSignature = fSequence->GetTimeSignature(0); }

  	fSequence->DeleteTimeSignature( oldID );
	}
}

//=============================================================================
// show time signature properties
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcTimeSignatureList::ShowTimeSignatureProperties( void )
{
	if ( fSelectedTimeSignature != 0 )
	{
		fTimeSignatureProperties = new TMvcTimeSignatureProperties( fSequence, fSelectedTimeSignature->ID );
		fTimeSignatureProperties->LoadLanguageFile( GetLanguageFileName() );
	}
}

//=============================================================================
// row is selected
// 12.09.2000 Adrian Meyer
//=============================================================================
void TimeSignatureListSelectRow( GtkWidget* aWidget,gint row,gint column,GdkEventButton *event,gpointer user_data)
{
	((TMvcTimeSignatureList*)user_data)->SetSelectedRow( row );
}

//=============================================================================
// add time signature
// 12.09.2000 Adrian Meyer
//=============================================================================
void TimeSignatureListAddClicked( GtkWidget* aWidget, gpointer user_data)
{
	((TMvcTimeSignatureList*)user_data)->AddTimeSignature( );
}

//=============================================================================
// delete time signature
// 12.09.2000 Adrian Meyer
//=============================================================================
void TimeSignatureListDeleteClicked( GtkWidget* aWidget, gpointer user_data)
{
	((TMvcTimeSignatureList*)user_data)->DeleteTimeSignature( );
}

//=============================================================================
// edit time signature  properties
// 12.09.2000 Adrian Meyer
//=============================================================================
void ViewTimeSignatureProperties( GtkWidget* aWidget, gpointer user_data )
{
	((TMvcTimeSignatureList*)user_data)->ShowTimeSignatureProperties( );
}
