//=============================================================================
//  model view control time signature properties
//=============================================================================
#include <mvcTimeSignatureProperties.h>
#include <stdio.h>
#include <stdlib.h>

// callbacks
void OkClickedTimeSignatureProp( GtkButton *button, gpointer user_data );
void CancelClickedTimeSignatureProp( GtkButton *button, gpointer user_data );

//=============================================================================
// construtor MvcTimeSignatureProperties											
// 11.09.2000 Adrian Meyer
//=============================================================================
TMvcTimeSignatureProperties::TMvcTimeSignatureProperties( TMidiSequence* aSequence, long aTimeSignatureID )
{
  wndTimeSignatureProperties = create_wndTimeSignatureProperties();	// call glade
	AddChildrenToWidgetList( wndTimeSignatureProperties );// fetch widgets
	fSequence = aSequence;
	fTimeSignatureID = aTimeSignatureID;
	
  HookSignal( "btnOk", "clicked", GTK_SIGNAL_FUNC( OkClickedTimeSignatureProp ), this );
  HookSignal( "btnCancel", "clicked", GTK_SIGNAL_FUNC( CancelClickedTimeSignatureProp ), this );

																											// don't change 1st
	gtk_widget_set_sensitive( FindWidget( "spiMeasure" ), fTimeSignatureID != 0 );

	if ( aSequence == 0 ) return;
  gtk_widget_show( wndTimeSignatureProperties );							
	SetData();																					// set data to gui
}

//=============================================================================
// destrutor			
// 11.09.2000 Adrian Meyer
//=============================================================================
TMvcTimeSignatureProperties::~TMvcTimeSignatureProperties( void )
{
	delete wndTimeSignatureProperties;
}

//=============================================================================
// set data to gui				
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcTimeSignatureProperties::SetData( void )
{
 	long tmpMeasure = 0; long tmpBeat = 0; long tmpTick = 0;		

 	fSequence->GetMeasureBeatTick( fSequence->GetTimeSignature( fTimeSignatureID )->Offset, &tmpMeasure, &tmpBeat, &tmpTick );

 																										// set measure
  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiMeasure" )), tmpMeasure );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiNumerator" )), fSequence->GetTimeSignature( fTimeSignatureID )->Numerator );

	char* tmpStr = new char[100];
	sprintf( tmpStr, "%d", 1 << fSequence->GetTimeSignature( fTimeSignatureID )->Denominator );
	gtk_entry_set_text( GTK_ENTRY( FindWidget( "edtDenominator" )), tmpStr );
	delete tmpStr;
}

//=============================================================================
// read data from gui into object											
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcTimeSignatureProperties::GetData( void )
{
	int tmpNumerator = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiNumerator" )));
	int tmpDenominator = atoi( gtk_entry_get_text( GTK_ENTRY( FindWidget( "edtDenominator" ))));
	for (int i = 0; i<8; i++ )
	{
		if ( 1 << i == tmpDenominator )
		{
			fSequence->ChangeTimeSignature( fTimeSignatureID, tmpNumerator, i, 24, 8 );
			break;
		}
	}


  long tmpOffset =																		// new offset
		fSequence->GetOffset(
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiMeasure" ))),
			1, 0 );

	fSequence->GetTimeSignature( fTimeSignatureID )->Offset = tmpOffset;
	fSequence->SortTimeSignature();
}

//=============================================================================
// read widget						
// 11.09.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcTimeSignatureProperties::GetWidget( void )
{
	return wndTimeSignatureProperties;
}

//=============================================================================
// ok clicked																					
// 11.09.2000 Adrian Meyer
//=============================================================================
void OkClickedTimeSignatureProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcTimeSignatureProperties*)user_data)->GetWidget() );
	((TMvcTimeSignatureProperties*)user_data)->GetData();
}

//=============================================================================
// cancel clicked																			
// 11.09.2000 Adrian Meyer
//=============================================================================
void CancelClickedTimeSignatureProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcTimeSignatureProperties*)user_data)->GetWidget() );
}
