//=============================================================================
//  model view control Track properties
//=============================================================================
#include <mvcTrackProperties.h>
#include <iostream.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <mcStatics.h>

// callbacks
void OkClickedTrackProp( GtkButton *button, gpointer user_data );
void CancelClickedTrackProp( GtkButton *button, gpointer user_data );
bool TrackColorExposeHook( GtkWidget *widget, GdkEventExpose *event, gpointer user_data );

//=============================================================================
// construtor MvcTrackProperties											
// 27.03.2000 Adrian Meyer
//=============================================================================
TMvcTrackProperties::TMvcTrackProperties( TMidiTrack* aTrack, TMidiMapper* aMapper )
{
  wndTrackProperties = create_wndTrackProperties();		// call glade
	AddChildrenToWidgetList( wndTrackProperties );      // fetch widgets
	fTrack = aTrack;
  fMapper = aMapper;

	colTrackColor = new GdkColor;
	colTrackColor->red = 0;	colTrackColor->green = 0;	colTrackColor->blue = 32000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( FindWidget( "drwColor" )),	
		colTrackColor, true, true );
	
  HookSignal( "btnOk", "clicked", GTK_SIGNAL_FUNC( OkClickedTrackProp ), this );
  HookSignal( "btnCancel", "clicked", GTK_SIGNAL_FUNC( CancelClickedTrackProp ), this );
//  HookSignal( "drwColor", "expose_event", GTK_SIGNAL_FUNC( TrackColorExposeHook ), &colTrackColor );

	char tmpStr[500];
	char* tmpItem = new char[ strlen("[none]")];				// no patch set

	fGMPatchNames = NULL;
	strcpy( tmpItem, "[none]" );
	fGMPatchNames = g_list_append( fGMPatchNames, tmpItem );

	for ( int i = 0; i<128; i++ )												// add only numbers
	{
		sprintf( tmpStr, "[%.3d] %s", i+1, cGMPatchName[ i ]);
		char* tmpItem = new char[ strlen(tmpStr) + 1 ];
		strcpy( tmpItem, tmpStr );
		fGMPatchNames = g_list_append( fGMPatchNames, tmpItem );
	}

	gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboInstrumentPatch" )), fGMPatchNames );	

  // port names
  fPortNames = 0;
  for ( int i = 0; i<fMapper->GetOutputDeviceCount(); i++ )
  {
 		sprintf( tmpStr, "[%.3d] %s", i+1, fMapper->GetOutputDevice(i)->GetName() );
		char* tmpItem = new char[ strlen(tmpStr) + 1 ];
		strcpy( tmpItem, tmpStr );
		fPortNames = g_list_append( fPortNames, tmpItem );
  }

	gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboPort" )), fPortNames );	

	if ( aTrack == 0 ) return;

  gtk_widget_show( wndTrackProperties );							

	SetData();																					// set data to gui
}

//=============================================================================
// destrutor MvcMain						 											27.03.2000 Adrian Meyer
//=============================================================================
TMvcTrackProperties::~TMvcTrackProperties( void )
{
	delete wndTrackProperties;
}

//=============================================================================
// set data to gui																		27.03.2000 Adrian Meyer
//=============================================================================
void TMvcTrackProperties::SetData( void )
{
	char* tmpTrackName = new char[ strlen( fTrack->GetTrackName()) + 1 ];
	strcpy( tmpTrackName, fTrack->GetTrackName() );
	gtk_entry_set_text( GTK_ENTRY( FindWidget( "edtTrackName" )),
		tmpTrackName);

	char* tmpInstrumentName = new char[ strlen( fTrack->GetInstrumentName()) + 1 ];
	strcpy( tmpInstrumentName, fTrack->GetInstrumentName() );
	gtk_entry_set_text( GTK_ENTRY( FindWidget( "edtInstrumentName" )),
		tmpInstrumentName );

  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiChannel" )),
		fTrack->GetChannel() );

  char* tmpStr = new char[200];

	strcpy( tmpStr, "[none]" );

	if ( fTrack->GetInstrumentPatch() > -1 )
	{
		sprintf( tmpStr, "[%.3d] %s", fTrack->GetInstrumentPatch() + 1,
				cGMPatchName[fTrack->GetInstrumentPatch()] );
	}

	gtk_entry_set_text( (GtkEntry*)FindWidget( "edtInstrumentPatch" ), tmpStr );

  // set the port
	gtk_entry_set_text( (GtkEntry*)FindWidget( "edtPort" ),
     (char*)g_list_nth_data( fPortNames, fTrack->GetPort() ) );

	delete tmpStr;
}

//=============================================================================
// read data from gui into object											27.03.2000 Adrian Meyer
//=============================================================================
void TMvcTrackProperties::GetData( void )
{
	fTrack->SetTrackName(
		gtk_entry_get_text( GTK_ENTRY( FindWidget( "edtTrackName" ))));

	fTrack->SetInstrumentName(
		gtk_entry_get_text( GTK_ENTRY( FindWidget( "edtInstrumentName" ))));

	fTrack->SetChannel(
	  gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiChannel" ))));

  short tmpInstrumentPatch = -1;											// set instrument patch
	for ( unsigned int i = 0; i < g_list_length( fGMPatchNames ); i++ )
	{
		if ( strcmp( (char*)g_list_nth_data( fGMPatchNames, i ),
			gtk_entry_get_text((GtkEntry*)FindWidget( "edtInstrumentPatch" ))) == 0 )
		{
			tmpInstrumentPatch = i-1;
		}					
	}

	fTrack->SetInstrumentPatch( tmpInstrumentPatch );

	for ( unsigned int i = 0; i < g_list_length( fPortNames ); i++ )
	{
		if ( strcmp( (char*)g_list_nth_data( fPortNames, i ),
			gtk_entry_get_text((GtkEntry*)FindWidget( "edtPort" ))) == 0 )
		{
			fTrack->SetPort( i );
		}					
	}
}

//=============================================================================
// read widget																				27.03.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcTrackProperties::GetWidget( void )
{
	return wndTrackProperties;
}

//=============================================================================
// ok clicked																					27.03.2000 Adrian Meyer
//=============================================================================
void OkClickedTrackProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcTrackProperties*)user_data)->GetWidget() );
	((TMvcTrackProperties*)user_data)->GetData();
}

//=============================================================================
// cancel clicked																			27.03.2000 Adrian Meyer
//=============================================================================
void CancelClickedTrackProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcTrackProperties*)user_data)->GetWidget() );
}

//=============================================================================
// redrawing color button
// 01.05.2001 Adrian Meyer
//=============================================================================
bool TrackColorExposeHook( GtkWidget *widget, GdkEventExpose *event, gpointer user_data )
{
	GdkGC* tmpGC = gdk_gc_new( widget->window );				// new graphic context
	gdk_gc_copy( tmpGC, widget->style->fg_gc[widget->state] );

	GdkDrawable* tmpWnd = widget->window;							  // window to draw to

	int tmpWidth = 0;
	int tmpHeight = 0;
	gdk_window_get_size( tmpWnd, &tmpWidth, &tmpHeight );
	gdk_gc_set_function( tmpGC, GDK_COPY );
	gdk_gc_set_foreground( tmpGC, (GdkColor*)user_data );	// draw button

	gdk_draw_rectangle( tmpWnd, tmpGC, TRUE,	0, 0, tmpWidth, tmpHeight );	

  return true;
}
