//=============================================================================
//  model view control user snap properties
//=============================================================================
#include <mvcUserSnapProperties.h>
#include <stdio.h>

void OkClickedUserSnapProp( GtkButton *button, gpointer user_data );
void CancelClickedUserSnapProp( GtkButton *button, gpointer user_data );
void NoteButtonClickedUserSnapProp( GtkButton *button, gpointer user_data );

//=============================================================================
// constructor
// 20.02.2001 Adrian Meyer
//=============================================================================
TMvcUserSnapProperties::TMvcUserSnapProperties( TMidiSequence* aSequence )
{
  wndUserSnapProperties = create_wndUserSnapProperties();
	AddChildrenToWidgetList( wndUserSnapProperties );   // fetch widgets

  HookSignal( "btnOk", "clicked", GTK_SIGNAL_FUNC( OkClickedUserSnapProp ), this );
  HookSignal( "btnCancel", "clicked", GTK_SIGNAL_FUNC( CancelClickedUserSnapProp ), this );

  HookSignal( "btn1_1", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedUserSnapProp ), this );
  HookSignal( "btn1_2", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedUserSnapProp ), this );
  HookSignal( "btn1_4", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedUserSnapProp ), this );
  HookSignal( "btn1_8", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedUserSnapProp ), this );
  HookSignal( "btn1_16", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedUserSnapProp ), this );
  HookSignal( "btn1_32", "clicked", GTK_SIGNAL_FUNC( NoteButtonClickedUserSnapProp ), this );

  fSequence = aSequence;															// init values
  fText = 0;

	if ( fSequence == 0 ) return;
  gtk_widget_show( wndUserSnapProperties );							
  SetData();
}

//=============================================================================
// destructor
// 20.02.2001 Adrian Meyer
//=============================================================================
TMvcUserSnapProperties::~TMvcUserSnapProperties( )
{
  if ( fText ) delete fText;													// remove text
	delete wndUserSnapProperties;                       // remove window
}


//=============================================================================
// read widget
// 20.02.2001 Adrian Meyer
//=============================================================================
GtkWidget* TMvcUserSnapProperties::GetWidget( void )
{
  return wndUserSnapProperties;                       // return widget
}

//=============================================================================
// set data to gui
// 21.02.2001 Adrian Meyer
//=============================================================================
void TMvcUserSnapProperties::SetData( void )
{
  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiDurationBeats" )),
			fSequence->GetGrid() / ( fSequence->GetDeltaTimesType()) );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiDurationTicks" )),
			fSequence->GetGrid() % ( fSequence->GetDeltaTimesType()) );										
}

//=============================================================================
// read data from gui
// 21.02.2001 Adrian Meyer
//=============================================================================
void TMvcUserSnapProperties::GetData( void )
{
  fSequence->SetGrid(
              gtk_spin_button_get_value_as_int(
								GTK_SPIN_BUTTON( FindWidget( "spiDurationBeats" ))) *
              fSequence->GetDeltaTimesType() +
							gtk_spin_button_get_value_as_int(
								GTK_SPIN_BUTTON( FindWidget( "spiDurationTicks" ))));
}

//=============================================================================
// read text of duration
// 20.02.2001 Adrian Meyer
//=============================================================================
char* TMvcUserSnapProperties::GetText()
{
  if ( fText ) delete fText;													// delete old value
  fText = new char[5];																// create new value

	long tmpDuration = fSequence->GetDeltaTimesType();

  if ( fSequence->GetGrid() / (tmpDuration*4*1.5) == 1 ) sprintf( fText, "1/1." ); else
  if ( fSequence->GetGrid() / (tmpDuration*4.0)   == 1 ) sprintf( fText, "1/1" ); else
  if ( fSequence->GetGrid() / (tmpDuration*2*1.5) == 1 ) sprintf( fText, "1/2." ); else
  if ( fSequence->GetGrid() / (tmpDuration*2.0)   == 1 ) sprintf( fText, "1/2" ); else
  if ( fSequence->GetGrid() / (tmpDuration*1.5  ) == 1 ) sprintf( fText, "1/4." ); else
  if ( fSequence->GetGrid() / (tmpDuration*1.0  ) == 1 ) sprintf( fText, "1/4" ); else
  if ( fSequence->GetGrid() / (tmpDuration*1.5/2) == 1 ) sprintf( fText, "1/8." ); else
  if ( fSequence->GetGrid() / (tmpDuration/2.0)   == 1 ) sprintf( fText, "1/8" ); else
  if ( fSequence->GetGrid() / (tmpDuration*1.5/4) == 1 ) sprintf( fText, "1/16." ); else
  if ( fSequence->GetGrid() / (tmpDuration/4.0)   == 1 ) sprintf( fText, "1/16" ); else
  if ( fSequence->GetGrid() / (tmpDuration*1.5/8) == 1 ) sprintf( fText, "1/32." ); else
  if ( fSequence->GetGrid() / (tmpDuration/8.0)   == 1 ) sprintf( fText, "1/32" ); else
  sprintf( fText, "%d ticks", (int)fSequence->GetGrid() );

  return fText;																				// return new text
}

//=============================================================================
// set the note value with buttons										
// 20.02.2001 Adrian Meyer
//=============================================================================
void TMvcUserSnapProperties::SetNoteValue( char* aButtonName )
{
	long tmpDuration = fSequence->GetDeltaTimesType();

	if ( strcmp( aButtonName, "btn1_1" ) == 0 ) { tmpDuration*=4; }
	if ( strcmp( aButtonName, "btn1_2" ) == 0 ) { tmpDuration*=2; }
	if ( strcmp( aButtonName, "btn1_8" ) == 0 ) { tmpDuration/=2; }
	if ( strcmp( aButtonName, "btn1_16" ) == 0 ) { tmpDuration/=4; }
	if ( strcmp( aButtonName, "btn1_32" ) == 0 ) { tmpDuration/=8; }

	// dotted
	if ( gtk_toggle_button_get_active((GtkToggleButton*)FindWidget( "chkDurationDotted" )))
		{	tmpDuration = (long)(tmpDuration*1.5); }

  // triplet
	if ( gtk_toggle_button_get_active((GtkToggleButton*)FindWidget( "chkDurationTriplet" )))
		{	tmpDuration = (long)(tmpDuration*2)/3; }

  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiDurationBeats" )),
			tmpDuration / ( fSequence->GetDeltaTimesType()) );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiDurationTicks" )),
			tmpDuration % ( fSequence->GetDeltaTimesType()) );										
}

//=============================================================================
// OK clicked
// 20.02.2001 Adrian Meyer
//=============================================================================
void OkClickedUserSnapProp( GtkButton *button, gpointer user_data )
{
  (( TMvcUserSnapProperties* )user_data)->GetData();
	gtk_widget_hide( ((TMvcUserSnapProperties*)user_data)->GetWidget() );
}

//=============================================================================
// cancel clicked
// 20.02.2001 Adrian Meyer
//=============================================================================
void CancelClickedUserSnapProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcUserSnapProperties*)user_data)->GetWidget() );
}

//=============================================================================
// et the note value with buttons										06.02.2000 Adrian Meyer
//=============================================================================
void NoteButtonClickedUserSnapProp( GtkButton *button, gpointer user_data )
{
	((TMvcUserSnapProperties*)user_data)->SetNoteValue(
		gtk_widget_get_name( GTK_WIDGET( button )));
}
